"""Copy default colorspaces.txt file into user directory and read it.

There are lots and lots of colorspaces to choose from. We provide a set of
default colorspaces in a text file. The first time the user launches
smartMedia, this default colorspaces.txt file will be copied into the user
folder. The user can then manipulate that file in their user folder.
This txt file's content will be parsed and feed into LineInputDoubleColorspaces
details widget to into the completer, so that when the user types something
into that widget, the completer shows matching colorspaces.

"""

# Import built-in modules
import logging
import os
import shutil

# Import local modules
from smartMedia import paths


def copy_to_user_folder():
    """Copy the default colorspaces.txt file into the default user location.

    Only perform this when the file does not already exist.

    """
    dest = get_user_colorspace_autocomplete_path()
    if os.path.isfile(dest):
        return

    logger = logging.getLogger(__file__)

    parent = os.path.dirname(dest)
    if not os.path.isdir(parent):
        os.makedirs(parent)

    logger.info("Creating colorspaces.txt in %s", dest)
    shutil.copy(
        get_package_colorspace_autocomplete_path(),
        get_user_colorspace_autocomplete_path()
    )


def get_user_colorspace_autocomplete_path():
    """Get the user path for the colorspace autocomplete text file.

    This is the txt file that resides in the local user environment and
    that is supposed to be altered.

    Returns:
        str: The user path for the colorspace autocomplete text file.

    """
    return os.path.join(
        paths.get_package_dir(private=False),
        "colorspaces.txt"
    )


def get_package_colorspace_autocomplete_path():
    """Get the default package path for the colorspace autocomplete text file.

    This is the txt file of this package that is supposed to be used as a
    base and that is --not-- supposed to be altered.

    Returns:
        str: The package path for the colorspace autocomplete text file.

    """
    return os.path.join(
        os.path.dirname(__file__),
        "colorspaces.txt"
    )


def load_colorspaces():
    """Load all colorspaces from the user colorspaces.txt file.

    Returns:
        :obj:`list` of :obj:`str`: Sequence of colorspaces from the
            colorspaces.txt file in the user directory.

    """
    path = get_user_colorspace_autocomplete_path()

    with open(path, "r") as file_:
        return sorted(list(set([
            line.replace("\n", "") for line in file_.readlines()
            if line != "\n" and not line.startswith("#")
        ])))
