"""Map task details to widgets.

Each task's data.json file contains information about the job to render.
On render time all these settings get translated into actions. Here we provide
an interface in form of custom widgets for the main window that let you and
other user update the data.json's settings via the main window.

As an example to illustrate this idea have a look at the following.
Each task's data.json file contains these key-value pairs (and a lot more):
{
    ...
    "target_start": 1001,
    "target_end": 1010,
    ...
}


The details widgets are custom widgets that can read these frame range details
and provide sub widgets to update these. We pass the updated values then back
to the data.json file on disk. So this creates an interface between the main
window and a task's data.json file on disk.

In this module we can configure which keys of a data.json file to drive by
which detail widget. You can either drive single keys or multiple keys with
one details widget.

"""

# Import built-in modules
from collections import OrderedDict

# Import local modules
from smartMedia.mvc.details_widgets.spinbox_range_source import SpinboxRangeSource
from smartMedia.mvc.details_widgets.spinbox_range_target import SpinboxRangeTarget
from smartMedia.mvc.details_widgets.line_input import LineInput
from smartMedia.mvc.details_widgets.line_input_double import LineInputDouble
from smartMedia.mvc.details_widgets.line_input_double_colorspaces import LineInputDoubleColorspaces
from smartMedia.mvc.details_widgets.line_input_readonly import LineInputReadOnly
from smartMedia.mvc.details_widgets.reformat import Reformat
from smartMedia.mvc.details_widgets.checkbox import Checkbox
from smartMedia.mvc.details_widgets.custom_attributes import CustomAttributes

# Map a task's data.json keys to widgets in order to be able to manipulate the
# task's data.json file.
#
# Note: The order in here determines the order of the task's widgets in the
# details section in the main window, so you can change the order here to your
# needs.
DETAILS_WIDGETS = OrderedDict()

def register_details_widget(key, widget, custom_label=""):
    """Register a details widget to make it available in the main window.

    Args:
        key (str): The name of the detail setting to represent. In case you
            want to map several settings to one widget then concatenate them
            with a pipe symbol --> | <--
        widget (smartMedia.mvc.details_widgets.Any): The details widget to use
            in order to represent the given key.
        custom_label (str, optional): A custom label to use in the UI. if not
            given we use the given key name.

    """
    DETAILS_WIDGETS[key] = {
        "widget": widget,
        "custom_label": custom_label
    }

# Register all details widgets here.
# 1) The first argument is the name of the detail setting to represent. In case
#    you want to map several settings to one widget then concatenate them with
#    a pipe symbol ->   |   <- as also illustrated below. When editing a detail
#    widget then this will update all the mentioned detail settings as defined
#    in the detail widget.
# 2) The second argument is the details widget to use in order to represent the
#    detail(s) in 1.
# 3) An optional custom_label to use as name in the UI. if not given we use the
#    detail name from 1 and replace underscores to white space for better
#    readability, e.g. source_path -> source path
#
register_details_widget("source_path", LineInputReadOnly)
register_details_widget("target_path", LineInputReadOnly)
register_details_widget("preset_info", LineInputReadOnly)
register_details_widget("source_start|source_end", SpinboxRangeSource, custom_label="source range")
register_details_widget("target_start|target_end", SpinboxRangeTarget, custom_label="target range")
register_details_widget("source_colorspace|target_colorspace", LineInputDoubleColorspaces, custom_label="target range")
register_details_widget("comment", LineInput)
register_details_widget("reformat", Reformat)
register_details_widget("use_slate", Checkbox)
register_details_widget("use_burnins", Checkbox)
register_details_widget("custom", CustomAttributes)
