"""Provide an interface to set up the reformat using multiple options.

When it comes to reformating media, smartMedia provides all three modes that
Nuke provides internally:

  - Reformat to a specific format
  - Reformat using a scale value
  - Reformat to an explicit width and height or a width with a calculated
    height.

This Reformat details widget provides functionality to change between all these
options. Consequently, it involves a bit more setup here.

"""


# Import third-party modules
try:
    from PySide import QtCore
    from PySide import QtGui as QtWidgets
except ImportError:
    from PySide2 import QtCore
    from PySide2 import QtGui
    from PySide2 import QtWidgets

# Import local modules
from smartMedia.constants import PATTERN_ALL_NUMBERS
from smartMedia.constants import PATTERN_FLOATS
from smartMedia.mvc.details_widgets.base import Base
from smartMedia.mvc import helper
from smartMedia.settings import Settings
from smartMedia.mvc.dialogs.new_format import NewFormatDialog


class Reformat(Base):
    """Provide an interface to set up the reformat using multiple options."""

    def __init__(self, preset, keys, custom_label=None):
        """Initialize the Reformat instance.

        Args:
            preset (smartMedia.oreset.Preset): The preset instance that holds
                all task's settings. This is needed because depending on the
                specific setting that we want to represent we need to have
                access to the other settings of the same task.
            keys (:obj:`list` of :obj:`str` or str): Sequence of setting keys
                or exactly -one- settings key to represent.
            custom_label (str, optional): The label to use for the widget.
                If not given we use the key and replace underscores with white
                space.

        """
        super(Reformat, self).__init__(preset, keys, custom_label=custom_label)

        self.settings = Settings()
        self.setup_widgets()
        self.set_widget_visibility()
        self.set_validators()

    def create_widgets(self):
        """Create widgets."""
        super(Reformat, self).create_widgets()

        self.label = QtWidgets.QLabel(self.key.replace("_", " "))
        self.label.setMinimumWidth(120)
        self.combo_type = QtWidgets.QComboBox()
        self.combo_type.addItems(
            [
                "same as source",
                "scale",
                "to format",
                "to box",
            ]
        )
        helper.fix_combo_solid_bg(self.combo_type)
        self.slider_scale = QtWidgets.QSlider()

        self.slider_scale.setVisible(False)
        self.slider_scale.setRange(0, 300)
        self.slider_scale.setOrientation(QtCore.Qt.Horizontal)
        self.spin_scale_value = QtWidgets.QDoubleSpinBox()
        self.spin_scale_value.setSingleStep(0.01)
        helper.fix_no_outline(self.spin_scale_value)

        self.combo_format = QtWidgets.QComboBox()
        helper.fix_combo_solid_bg(self.combo_format)
        self.combo_format.setMinimumWidth(400)
        self.combo_format.setVisible(False)

        self.label_box_width = QtWidgets.QLabel("width")
        self.input_box_width = QtWidgets.QLineEdit()
        self.label_box_height = QtWidgets.QLabel("height")
        self.input_box_height = QtWidgets.QLineEdit()
        self.input_box_height.setEnabled(False)
        self.check_box_force_shape = QtWidgets.QCheckBox(" Force shape")
        self.label_box_pixel_aspect = QtWidgets.QLabel("Pixel Aspect")
        self.input_box_pixel_aspect = QtWidgets.QLineEdit()

        helper.fix_no_outline(self.input_box_width)
        helper.fix_no_outline(self.input_box_height)
        helper.fix_no_outline(self.input_box_pixel_aspect)

    def create_layouts(self):
        """Create layouts."""
        super(Reformat, self).create_layouts()

        self.layout.addWidget(self.label)
        self.layout.addWidget(self.combo_type)

        self.layout.addWidget(self.slider_scale)
        self.layout.addWidget(self.spin_scale_value)

        self.layout.addWidget(self.combo_format, 2)

        self.group_box_format = QtWidgets.QGroupBox()
        self.group_box_format.setProperty("style", "plain_group")
        self.group_box_format.setVisible(False)
        self.layout_box_format = QtWidgets.QHBoxLayout()
        self.layout_box_format.setContentsMargins(0, 0, 0, 0)
        self.layout_box_format.addWidget(self.label_box_width)
        self.layout_box_format.addWidget(self.input_box_width)
        self.layout_box_format.addWidget(self.label_box_height)
        self.layout_box_format.addWidget(self.input_box_height)
        self.layout_box_format.addWidget(self.check_box_force_shape)
        self.layout_box_format.addWidget(self.label_box_pixel_aspect)
        self.layout_box_format.addWidget(self.input_box_pixel_aspect)

        self.group_box_format.setLayout(self.layout_box_format)

        self.layout.addWidget(self.group_box_format)

        self.layout.addStretch()

        helper.set_style_sheet(self)

    def create_signals(self):
        """Create signals."""
        super(Reformat, self).create_signals()

        self.combo_type.currentIndexChanged.connect(
            lambda: self.set_widget_visibility()
        )

        self.spin_scale_value.valueChanged.connect(
            self.update_scale
        )
        self.slider_scale.valueChanged.connect(
            self.update_scale
        )
        self.combo_format.currentIndexChanged.connect(
            lambda: self.check_new_format_dialog()
        )
        self.combo_format.currentIndexChanged.connect(
            lambda: self.update()
        )
        self.input_box_width.textChanged.connect(
            lambda: self.update()
        )
        self.input_box_height.textChanged.connect(
            lambda: self.update()
        )
        self.check_box_force_shape.stateChanged.connect(
            lambda: self.update()
        )
        self.input_box_pixel_aspect.textChanged.connect(
            lambda: self.update()
        )
        self.check_box_force_shape.stateChanged.connect(
            lambda: self.set_box_height_editable()
        )

    def setup_widgets(self):
        """Setup widgets."""
        reformat_type = self.preset["reformat"]["type"]

        # Block signal, otherwise this would reset the previously stored
        # reformat value.
        self.combo_type.blockSignals(True)
        helper.combo_set_text(self.combo_type, reformat_type)
        self.combo_type.blockSignals(False)

        if isinstance(self.preset["reformat"]["value"], float):
            self.spin_scale_value.setValue(self.preset["reformat"]["value"])

        self.populate_root_formats()

        if isinstance(self.preset["reformat"]["value"], dict):
            values = self.preset["reformat"]["value"]
            self.input_box_width.setText(str(values["width"]))
            self.input_box_height.setText(str(values["height"]))
            self.check_box_force_shape.setChecked(values["force_shape"])
            self.input_box_pixel_aspect.setText(str(values["pixel_aspect"]))

            self.set_box_height_editable()

    def set_box_height_editable(self):
        """Drive the editable state of the box height widget.

        When 'Force Shape' is disabled then we make the height input disabled.
        This will then calculate the correct height automtically by Nuke and
        mimics behavior in Nuke's Reformat node.

        """
        self.input_box_height.setEnabled(
            self.check_box_force_shape.isChecked()
        )

    def populate_root_formats(self):
        """Populate root formats."""
        # Block signals on combo_format because we re-paint the widget which
        # causes triggering the indexChanged signal on this combo which is
        # not what we want here.
        self.combo_format.blockSignals(True)

        self.combo_format.clear()

        formats = sorted(
            self.settings["formats"],
            key=lambda item: item[0].lower()
        )

        for format_ in formats:
            self.combo_format.addItem(format_[0], userData=format_[1])
        self.combo_format.addItem("new...")

        self.combo_format.setCurrentIndex(
            self.combo_format.findData(self.preset["reformat"]["value"])
        )
        # The format does not exist, yet in the local user settings file. Add
        # it to the local settings file and to the dropdown.
        if self.preset["reformat"]["type"] == "to format" and \
                self.combo_format.currentIndex() == -1:

            format_label = self.preset["reformat"]["value"].split(" ")[-1]
            format_value = self.preset["reformat"]["value"]

            self.settings["formats"].append(
                (
                    format_label,
                    format_value
                ),
            )
            self.settings.save()

            self.combo_format.insertItem(0, format_label, userData=format_value)

            self.combo_format.setCurrentIndex(
                self.combo_format.findData(format_value))

        self.combo_format.blockSignals(False)

    def check_new_format_dialog(self):
        """Check if we need to show the 'new format' dialog."""
        if self.combo_format.currentText() == "new...":
            format_data = NewFormatDialog.show()
            if not format_data:
                return

            format_label = "{name} {width}x{height}".format(**format_data)
            format_value = \
                "{width} {height} 0 0 {width} {height} {pixel_aspect} {name}".format(
                    **format_data
                )
            self.settings["formats"].append(
                (
                    format_label,
                    format_value
                ),
            )
            self.settings.save()
            self.preset["reformat"]["type"] = "to format"
            self.preset["reformat"]["value"] = format_value

            self.populate_root_formats()

    def update_scale(self):
        """Update the counter part scale widget as there are two."""
        sender = self.sender()

        self.spin_scale_value.valueChanged.disconnect()
        self.slider_scale.valueChanged.disconnect()

        if isinstance(sender, QtWidgets.QSlider):
            self.spin_scale_value.setValue(self.slider_scale.value() * 0.01)
        elif isinstance(sender, QtWidgets.QDoubleSpinBox):
            self.slider_scale.setValue(self.spin_scale_value.value() * 100)

        self.spin_scale_value.valueChanged.connect(self.update_scale)
        self.slider_scale.valueChanged.connect(self.update_scale)

        self.update()

    def set_widget_visibility(self):
        """Set up the visibility of the widgets based on the reformat mode."""
        current_mode = self.combo_type.currentText()
        self.preset["reformat"]["type"] = current_mode
        self.preset.save()

        is_mode_scale = current_mode == "scale"
        is_mode_reformat = current_mode in ("to format", "format")
        is_mode_box = current_mode == "to box"

        self.slider_scale.setVisible(is_mode_scale)
        self.spin_scale_value.setVisible(is_mode_scale)

        self.combo_format.setVisible(is_mode_reformat)

        self.group_box_format.setVisible(is_mode_box)

        self.update()

    def set_validators(self):
        """Set validators for widgets."""
        # Allow all numbers for width and height widget.
        for widget in (self.input_box_width, self.input_box_height):
            widget.setValidator(
                QtGui.QRegExpValidator(QtCore.QRegExp(PATTERN_ALL_NUMBERS))
            )

        # Allow numbers and floating numbers for pixel aspect.
        self.input_box_pixel_aspect.setValidator(
            QtGui.QRegExpValidator(QtCore.QRegExp(PATTERN_FLOATS))
        )

    def update(self):
        """Update the preset and thus the task's data.json"""
        self.preset[self.key]["type"] = self.combo_type.currentText()

        if self.preset[self.key]["type"] == "scale":
            self.preset[self.key]["value"] = self.spin_scale_value.value()

        elif self.preset[self.key]["type"] in ("to format", "format"):
            self.preset[self.key]["value"] = self.combo_format.itemData(
                self.combo_format.currentIndex()
            )

        elif self.preset[self.key]["type"] == "same as source":
            self.preset[self.key]["value"] = ""

        elif self.preset[self.key]["type"] == "to box":
            self.preset[self.key]["value"] = {
                "width": int(self.input_box_width.text() or 0),
                "height": int(self.input_box_height.text() or 0),
                "force_shape": self.check_box_force_shape.isChecked(),
                "pixel_aspect": float(self.input_box_pixel_aspect.text() or 1.0)
            }

        self.preset.save()
