"""Dialog to confirm action by user."""

# Import third-party modules
try:
    from PySide import QtCore
    from PySide import QtGui as QtWidgets
except ImportError:
    from PySide2 import QtCore
    from PySide2 import QtWidgets

# Import local modules
from smartMedia.mvc import helper


class Confirm(QtWidgets.QDialog):
    """Confirm action."""

    def __init__(self, title, message, accept_label, accept_color,
                 reject_label, remember_checkbox):
        """Initialize the Confirm instance.

        title (str): Title to use for the dialog.
            message (str): Message to display in the dialog.
            accept_label (str): Label to use for the 'accept' button.
            accept_color (str): Color to use for the accept button. Use 'blue'
                for a blue button, otherwise we use red.
            reject_label (str): Label to use for the 'reject' button.
            remember_checkbox (bool): If True show a remember checkbox as long
                as being in the loop of creating the dialogs and remember the
                last state that was checked. Defaults to False meaning not
                showing a remember checkbox.

        """
        super(Confirm, self).__init__()

        self.title = title
        self.message = message
        self.accept_label = accept_label
        self.accept_color = accept_color
        self.reject_label = reject_label
        self.remember_checkbox = remember_checkbox

        self.setup()
        self.build_ui()

    @classmethod
    def ask(cls, title, message, accept_label="OK", accept_color="blue",
            reject_label="Cancel", remember_checkbox=False):
        """Factory method to create a dialog.
        
        Args:
            title (str): Title to use for the dialog.
            message (str): Message to display in the dialog.
            accept_label (str, optional): Label to use for the 'accept' button.
            accept_color (str, optional): Color to use for the accept button.
                Use 'blue' for a blue button, otherwise we use red.
            reject_label (str, optional): Label to use for the 'reject' button.
            remember_checkbox (bool, optional): If True show a remember
                checkbox as long as being in the loop of creating the dialogs
                and remember the last state that was checked. Defaults to False
                meaning not showing a remember checkbox.

        Returns:
            bool, bool: The value if accepted or rejected and the value if
                we should remember the value for the current session.

        """
        obj = cls(
            title, message, accept_label, accept_color, reject_label,
            remember_checkbox
        )
        obj.raise_()
        obj.show()
        return obj.exec_(), obj.check_remember.isChecked()

    def setup(self):
        """Set up UI."""
        self.setWindowTitle(self.title)
        self.setModal(True)
        self.setMinimumSize(QtCore.QSize(400, 70))
        self.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint)

    def build_ui(self):
        """Build UI."""
        self.create_widgets()
        self.create_layouts()
        self.create_signals()

    def create_widgets(self):
        """Create widgets."""
        self.label_message = QtWidgets.QLabel(self.message)
        self.label_message.setWordWrap(True)

        self.button_accept = QtWidgets.QPushButton(self.accept_label)

        style = "action_button" if self.accept_color == "blue" else "red"
        self.button_accept.setProperty("style", style)

        self.button_reject = QtWidgets.QPushButton(self.reject_label)
        self.check_remember = QtWidgets.QCheckBox("remember")

    def create_layouts(self):
        """Create layouts."""
        self.layout_master = QtWidgets.QVBoxLayout()
        self.layout_button = QtWidgets.QHBoxLayout()

        self.layout_button.addWidget(self.button_accept)
        self.layout_button.addWidget(self.button_reject)
        if self.remember_checkbox:
            self.layout_button.addWidget(self.check_remember)

        self.layout_master.addWidget(self.label_message)
        self.layout_master.addLayout(self.layout_button)

        self.setLayout(self.layout_master)
        helper.set_style_sheet(self)
        self.button_accept.setFocus()

    def create_signals(self):
        """Create signals."""
        self.button_accept.clicked.connect(lambda: self.accept())
        self.button_reject.clicked.connect(lambda: self.reject())
