"""Show browser to browse to path and return browsed path."""

# Import third-party modules
try:
    from PySide import QtCore
    from PySide import QtGui as QtWidgets
except ImportError:
    from PySide2 import QtCore
    from PySide2 import QtWidgets


def show(
    title,
    only_directory=False,
    save=False,
    name_filters=None,
    start_directory=None,
    start_filename=None
):
    """Show browser to browse to path and return browsed path.

    Args:
        title (str): Title to display for browser.
        only_directory (bool, optional): If true be only able to select
            folders.
        save (bool, optional): If True use save mode, otherwise use load mode.
        name_filters (:obj:`list` of :obj:`str`, optional): Name filter that
            are allowed to select. If not provided, all files can be selected.
                Example: ["Images (*.png *.jpg)"]
        start_directory (str, optional): Absolute path to the directory to use
            as start directory, i.e. where the browser starts.
            ATTENTION: QT requires that this directory exists, otherwise the
            folder will not start at this folder, how could it.
        start_filename (str): The file name to use as default when the
            dialog launches.

    Returns:
        str: Absolute path that was browsed to.

    """
    dialog = QtWidgets.QFileDialog()
    dialog.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint)
    dialog.setWindowTitle(title)

    if only_directory:
        dialog.setFileMode(QtWidgets.QFileDialog.Directory)
        dialog.setOption(QtWidgets.QFileDialog.ShowDirsOnly)

    if save:
        dialog.setAcceptMode(QtWidgets.QFileDialog.AcceptSave)

    if name_filters:
        dialog.setNameFilters(name_filters)

    if start_directory:
        dialog.setDirectory(start_directory)

    if start_filename:
        dialog.selectFile(start_filename)

    if dialog.exec_() == QtWidgets.QDialog.Accepted:
        return dialog.selectedFiles()[0]
    return ""
