"""Entry point to create the preset wizard."""

# Import third-party modules
try:
    from PySide import QtCore
    from PySide import QtGui
    from PySide import QtGui as QtWidgets
except ImportError:
    from PySide2 import QtCore
    from PySide2 import QtGui
    from PySide2 import QtWidgets

# Import local modules
from smartMedia.mvc import helper
from smartMedia.mvc.dialogs.preset_wizard.page_code_injections import PageCodeInjections
from smartMedia.mvc.dialogs.preset_wizard.page_generate_template import PageGenerateTemplate
from smartMedia.mvc.dialogs.preset_wizard.page_intro import PageIntro
from smartMedia.mvc.dialogs.preset_wizard.page_output import PageOutput
from smartMedia.mvc.dialogs.preset_wizard.page_preset_info import PagePresetInfo
from smartMedia.mvc.dialogs.preset_wizard.page_reformat import PageReformat
from smartMedia.mvc.dialogs.preset_wizard.page_slate_burnins import PageSlateBurnins
from smartMedia.mvc.dialogs.preset_wizard.page_time import PageTime


# Use wizard as global to avoid garbage collection.
WIZARD = None


def _translate_reformat(wizard):
    """Translate the reformat value from the reformat page.

    Args:
        wizard (QtWidgets.QWizard): The wizard that holds information about
            all values.

    Returns:
        dict: The reformat to apply in the format:
        {
            "type": <str: reformat type>,
            "value": <dict|float|str: reformat value>,
        }

    """
    type_ = None

    if wizard.field("reformat_same_as_source"):
        type_ = "same as source"
    elif wizard.field("reformat_scale"):
        type_ = "scale"
    elif wizard.field("reformat_format"):
        type_ = "to format"
    elif wizard.field("reformat_box"):
        type_ = "box"

    return {
        "type": type_,
        "value": wizard.field("reformat_value"),
    }


def _create_wizard():
    """Create the wizard with all its sub pages.

    Returns:
        dict: The entered wizard data to use for creating a new preset.

    """
    # Use wizard as global to avoid garbage collection.
    global WIZARD

    WIZARD = QtWidgets.QWizard()
    WIZARD.setMinimumSize(QtCore.QSize(800, 500))

    pages = (
        PageIntro,
        PagePresetInfo,
        PageReformat,
        PageTime,
        PageSlateBurnins,
        PageOutput,
        PageGenerateTemplate,
        PageCodeInjections,
    )
    for index, page in enumerate(pages, 1):
        WIZARD.addPage(page(index, len(pages)))

    helper.set_style_sheet(WIZARD)
    WIZARD.setWindowTitle("Create new preset")
    WIZARD.show()
    WIZARD.raise_()

    data = {}
    if WIZARD.exec_():
        data = {
            # Anything path related.
            "paths": {
                "preset_name": WIZARD.field("preset_name"),
                "preset_root": WIZARD.field("preset_root"),
                "slate_path": WIZARD.field("slate_path"),
                "burnins_path": WIZARD.field("burnins_path"),
            },
            # Anything that goes into the preset's data.json.
            "data": {
                "preset_info": WIZARD.field("preset_info"),
                "reformat": _translate_reformat(WIZARD),
                "use_slate": WIZARD.field("use_slate"),
                "use_burnins": WIZARD.field("use_burnins"),
                "source_frame_mode": WIZARD.field("source_frame_mode"),
                "target_ext": WIZARD.field("target_ext"),
                "target_fps": WIZARD.field("target_fps"),
                "target_colorspace": WIZARD.field("target_colorspace"),
                "target_codec": WIZARD.field("target_codec"),
            },
            "general": {
                "generate_template": WIZARD.field("generate_template"),
                "code_inject-data": WIZARD.field("code_inject_data"),
                "code_inject-pre_render": WIZARD.field("code_inject_pre_render"),
                "code_inject-post_render": WIZARD.field("code_inject_post_render"),
            }
        }

        source_start = WIZARD.field("source_start")
        if source_start:
            data["data"]["source_start"] = int(source_start)

    return data


def get_preset_user_data():
    """Show the wizard to create a new preset.

    Returns:
        dict: The entered wizard data to use for creating a new preset.

    """
    return _create_wizard()


if __name__ == "__main__":
    app = QtWidgets.QApplication([])
    wizard_data = get_preset_user_data()
    print(wizard_data)
    app.exec_()
