"""Custom widgets."""

# Import third-party modules
try:
    from PySide import QtCore
    from PySide import QtGui as QtWidgets
except ImportError:
    from PySide2 import QtCore
    from PySide2 import QtGui
    from PySide2 import QtWidgets

# Import local modules
from smartMedia import constants
from smartMedia.mvc import helper
from smartMedia.settings import Settings

_ICONS = helper.load_icons()

if constants.PY2:
    unicode = unicode
else:
    unicode = str


class JobWidget(QtWidgets.QWidget):
    """A widget that shows a source to run a conversion on."""

    double_clicked = QtCore.Signal()
    entered = QtCore.Signal()

    def __init__(self):
        """Initialize the JobWidget instance."""
        super(JobWidget, self).__init__()

        self.create_widgets()
        self.create_layouts()
        self.setup_ui()

    def setup_ui(self):
        """Setup ui."""
        self.setAttribute(QtCore.Qt.WA_StyledBackground, True)
        self.setStyleSheet('JobWidget {background-color: #1d1d1d};')
        self.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.setMouseTracking(True)

    def create_widgets(self):
        """Create widgets."""
        self.thumbnail = QtWidgets.QLabel()
        self.label_source_offline = QtWidgets.QLabel("Source is offline!")
        self.label_source_offline.setProperty("style", "offline_notify")
        self.label_source_offline.setVisible(False)
        self.label_source_offline.setAlignment(QtCore.Qt.AlignCenter)
        self.top = WidgetDoubleClickable()
        self.button_expand_collapse = QtWidgets.QToolButton()
        helper.set_icon(self.button_expand_collapse, "expand")
        self.button_naming = QtWidgets.QToolButton()
        helper.set_icon(self.button_naming, "naming")
        self.title = QtWidgets.QLabel()
        self.title.setProperty("style", "title")
        self.button_reveal = QtWidgets.QToolButton()
        helper.set_icon(self.button_reveal, "reveal")
        self.button_add_task = QtWidgets.QToolButton()
        helper.set_icon(self.button_add_task, "plus")

        self.table_specs = QtWidgets.QTableWidget()
        self.table_specs.setRowCount(3)
        self.table_specs.setColumnCount(3)
        for col in range(3):
            self.table_specs.setColumnWidth(col, self.table_specs.width() / 3.33)
        self.table_specs.horizontalHeader().setStretchLastSection(True)
        self.table_specs.verticalHeader().hide()
        self.table_specs.horizontalHeader().hide()
        self.table_specs.setMaximumHeight(100)
        self.table_specs.setShowGrid(False)
        self.table_specs.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.table_specs.setFocusPolicy(QtCore.Qt.NoFocus)
        self.table_specs.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        self.table_specs.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.table_specs.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.table_specs.setMinimumHeight(90)
        self.table_specs.setAttribute(QtCore.Qt.WA_TransparentForMouseEvents)

        self.group_naming = QtWidgets.QGroupBox()
        self.group_naming.setVisible(False)
        self.text_naming_values = QtWidgets.QTextEdit()
        self.text_naming_values.setMinimumHeight(80)
        self.text_naming_values.setReadOnly(True)
        self.text_naming_values.setProperty("style", "lg")
        self.combo_naming = QtWidgets.QComboBox()
        helper.fix_combo_solid_bg(self.combo_naming)
        self.button_reload_naming = QtWidgets.QToolButton()
        helper.set_icon(self.button_reload_naming, "reload")
        self.input_naming_pattern = QtWidgets.QTextEdit()
        self.input_naming_pattern.setMinimumHeight(80)
        self.input_naming_pattern.setReadOnly(True)
        self.input_naming_pattern.setProperty("style", "lg")

    def create_layouts(self):
        """Create layouts."""
        self.layout_main = QtWidgets.QVBoxLayout()
        self.layout_top = QtWidgets.QHBoxLayout()

        self.layout_top_right = QtWidgets.QHBoxLayout()
        self.layout_top_right.setContentsMargins(0,0,0,0)
        self.layout_top_right.addWidget(self.button_expand_collapse)
        self.layout_top_right.addWidget(self.button_naming)
        self.layout_top_right.addWidget(self.title)
        self.layout_top_right.addWidget(self.button_add_task)
        self.layout_top_right.addWidget(self.button_reveal)
        self.top.setLayout(self.layout_top_right)

        self.layout_naming_main = QtWidgets.QVBoxLayout()
        self.group_naming.setLayout(self.layout_naming_main)
        self.layout_naming_top = QtWidgets.QHBoxLayout()
        self.layout_naming_top.addWidget(self.combo_naming)
        self.layout_naming_top.addWidget(self.button_reload_naming)
        self.layout_naming_main.addLayout(self.layout_naming_top)
        self.layout_naming_main.addWidget(self.input_naming_pattern)
        self.layout_naming_main.addWidget(self.text_naming_values)

        self.layout_right = QtWidgets.QVBoxLayout()
        self.layout_right.setContentsMargins(0,0,0,0)
        self.layout_right.addWidget(self.top)
        self.layout_right.addWidget(self.table_specs)
        self.layout_right.setAlignment(QtCore.Qt.AlignTop)

        self.layout_tasks = QtWidgets.QVBoxLayout()
        self.layout_tasks.setContentsMargins(0,0,0,0)
        self.layout_tasks.setAlignment(QtCore.Qt.AlignTop)

        self.layout_thumb = QtWidgets.QVBoxLayout()
        self.layout_thumb.addWidget(self.thumbnail)
        self.layout_thumb.setAlignment(QtCore.Qt.AlignTop)
        self.layout_thumb.addWidget(self.label_source_offline)

        self.layout_top.addLayout(self.layout_thumb)
        self.layout_top.addLayout(self.layout_right)

        self.layout_main.addLayout(self.layout_top)
        self.layout_main.addWidget(self.group_naming)
        self.layout_main.addLayout(self.layout_tasks)

        self.setLayout(self.layout_main)
        helper.set_style_sheet(self)

    def mouseMoveEvent(self, event):
        """Overwrite mouseMoveEvent to keep track of currently hovered job.

        Args:
            event (QtCore.QEvent): The event that got triggered.

        """
        super(JobWidget, self).mouseMoveEvent(event)

        self.entered.emit()
        event.accept()

    def mousePressEvent(self, event):
        """Make JobWidget the currently hovered job."""
        super(JobWidget, self).mousePressEvent(event)

        self.entered.emit()
        event.accept()


class TaskWidget(QtWidgets.QWidget):
    """Widget to represent one task to process on a source using a preset."""

    finished = QtCore.Signal(bool)
    clicked = QtCore.Signal()
    entered = QtCore.Signal()

    settings = Settings()

    def __init__(self):
        """Initialize the TasWidget instance."""
        super(TaskWidget, self).__init__()

        self.create_widgets()
        self.create_layouts()

        self.setup_ui()
        self.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)

    def setup_ui(self):
        self.setAttribute(QtCore.Qt.WA_StyledBackground, True)
        self.setMouseTracking(True)

    def create_widgets(self):
        """Create widgets."""
        self.button_select = QtWidgets.QToolButton()
        helper.set_icon(self.button_select, "unselected")
        self.button_select.setVisible(False)
        self.button_details = QtWidgets.QToolButton()
        helper.set_icon(self.button_details, "arrow_right")

        if self.settings["preset_dropdown_type"] == "completer":
            self.combo_presets = ComboBoxWithCompleter()
        else:
            self.combo_presets = QtWidgets.QComboBox()

        helper.fix_combo_solid_bg(self.combo_presets)
        self.combo_presets.setFixedWidth(200)
        self.button_reload = QtWidgets.QToolButton()
        helper.set_icon(self.button_reload, "reload")
        self.button_output_path = QtWidgets.QPushButton()
        self.button_output_path.setProperty("style", "output_link")
        self.label_status = QtWidgets.QLabel("")
        self.label_status.setProperty("style", "status")
        self.label_status.setFixedWidth(100)
        self.label_status.setAlignment(QtCore.Qt.AlignCenter)
        self.button_render_log = QtWidgets.QToolButton()
        helper.set_icon(self.button_render_log, "document")

        self.label_processing = QtWidgets.QLabel()
        self.label_processing.setFixedWidth(30)
        self.label_processing.setVisible(False)
        self.label_processing.setProperty("style", "status")
        movie_process = QtGui.QMovie(_ICONS["processing"])
        self.label_processing.setMovie(movie_process)
        movie_process.start()

        self.button_reveal = QtWidgets.QToolButton()
        self.button_lock = QtWidgets.QToolButton()
        helper.set_icon(self.button_lock, "locked_red")
        self.button_start = QtWidgets.QToolButton()
        self.button_start.setFixedWidth(30)
        helper.set_icon(self.button_start, "play")
        self.widget_details = QtWidgets.QWidget()
        self.widget_log = QtWidgets.QWidget()
        self.widget_log.setVisible(False)
        self.widget_log.setMinimumHeight(500)
        self.text_log = QtWidgets.QTextEdit()

    def create_layouts(self):
        """Create layouts."""
        self.layout_main = QtWidgets.QVBoxLayout()
        self.layout_main.setContentsMargins(10, 10, 10, 0)
        self.layout_top = QtWidgets.QHBoxLayout()
        self.layout_top.setContentsMargins(0, 0, 0, 0)
        self.layout_details = QtWidgets.QVBoxLayout()
        self.layout_details.setContentsMargins(0, 0, 0, 0)

        self.layout_top.addWidget(self.button_select)
        self.layout_top.addWidget(self.button_details)
        self.layout_top.addWidget(self.combo_presets)
        self.layout_top.addWidget(self.button_reload)
        self.layout_top.addWidget(self.button_output_path)
        self.layout_top.addWidget(self.label_status)
        self.layout_top.addWidget(self.button_reveal)
        self.layout_top.addWidget(self.button_render_log)
        self.layout_top.addWidget(self.button_lock)
        self.layout_top.addWidget(self.button_start)
        self.layout_top.addWidget(self.label_processing)

        self.layout_log = QtWidgets.QVBoxLayout()
        self.layout_log.setContentsMargins(0,0,0,10)
        self.layout_log.addWidget(self.text_log)

        self.widget_details.setLayout(self.layout_details)
        self.widget_log.setLayout(self.layout_log)

        self.layout_main.addLayout(self.layout_top)
        self.layout_main.addWidget(self.widget_details)
        self.layout_main.addWidget(self.widget_log)

        self.setLayout(self.layout_main)

    def mousePressEvent(self, event):
        """Override mousePressEvent to register clicking the task."""
        super(TaskWidget, self).mousePressEvent(event)

        self.clicked.emit()

        # Important: Accept the event. See below. If we didn't do this then the
        # event would be propagated to its parent which is the JobWidget that
        # this widget resides in which is not what we want.
        event.accept()

    def mouseMoveEvent(self, event):
        """Overwrite mouseMoveEvent to keep track of currently hovered job.

        Args:
            event (QtCore.QEvent): The event that got triggered.

        """
        super(TaskWidget, self).mouseMoveEvent(event)
        self.entered.emit()

        # Important: Accept the event. This will then set the entered
        # job_or_task to the controller of this view. If we didn't do that then
        # the event will be passed to the parent which would be the job
        # widget and consequently the job widget's controller.
        event.accept()


class SimpleTable(QtWidgets.QTableWidget):
    """A simple table."""

    def __init__(self):
        """Initialize the SimpleTable instance."""
        super(SimpleTable, self).__init__()

        self.setColumnCount(1)
        self.horizontalHeader().setStretchLastSection(True)
        self.verticalHeader().hide()
        self.horizontalHeader().hide()
        self.setFocusPolicy(QtCore.Qt.NoFocus)
        self.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        self.setVerticalScrollBarPolicy(
            QtCore.Qt.ScrollBarPolicy.ScrollBarAsNeeded
        )
        self.setShowGrid(False)

        self.resizeRowsToContents()


class StatusBar(QtWidgets.QWidget):
    """Status info with processing wheel."""

    def __init__(self):
        """Initialize the ProgressInfo instance."""
        super(StatusBar, self).__init__()
        self.build_ui()
        self.set_info("")

    def build_ui(self):
        """Build UI."""
        self.create_widgets()
        self.create_layouts()

    def create_widgets(self):
        """Create Widgets."""
        self.label_icon_process = QtWidgets.QLabel()
        self.label_icon_process.setProperty("style", "status")
        movie_process = QtGui.QMovie(_ICONS["processing"])
        self.label_icon_process.setMovie(movie_process)
        movie_process.start()
        self.label_process = QtWidgets.QLabel()
        self.label_process.setProperty("style", "status")

    def create_layouts(self):
        """Create layouts."""
        self.layout_main = QtWidgets.QHBoxLayout()
        self.layout_main.addWidget(self.label_icon_process)
        self.layout_main.addWidget(self.label_process, 100)

        self.setLayout(self.layout_main)
        helper.set_style_sheet(self)

    def set_info(self, text):
        """Set the given text as info.

        Args:
            text: The Text to show.

        """
        self.label_process.setText(text)
        self.label_icon_process.setVisible(self.label_process.text() != "")
        QtWidgets.QApplication.processEvents()

    def flush(self):
        """Set text empty."""
        self.set_info("")


class NukeExeFinder(QtWidgets.QWidget):
    """Widget to enter a root to search for a Nuke executable."""

    def __init__(self):
        """Initialize the NukeExeFinder instance."""
        super(NukeExeFinder, self).__init__()

        self.is_favorite = False

        self.create_widgets()
        self.create_layouts()

    def create_widgets(self):
        """Create widgets."""
        self.button_favorite = QtWidgets.QToolButton()
        self.input = QtWidgets.QLineEdit()
        helper.fix_no_outline(self.input)
        self.button_test = QtWidgets.QPushButton("test")
        self.button_test.setProperty("style", "action_button")
        self.button_add = QtWidgets.QToolButton()
        helper.set_icon(self.button_add, "plus")
        self.button_del = QtWidgets.QToolButton()
        helper.set_icon(self.button_del, "minus")

    def create_layouts(self):
        """Create layouts."""
        self.layout_main = QtWidgets.QHBoxLayout()

        self.layout_main.addWidget(self.button_favorite)
        self.layout_main.addWidget(self.input)
        self.layout_main.addWidget(self.button_test)
        self.layout_main.addWidget(self.button_add)
        self.layout_main.addWidget(self.button_del)

        self.setLayout(self.layout_main)

    def set_path(self, path):
        """Set up the given path in the input.

        Args:
            path (str): Absolute path to show in the input.

        """
        self.input.setText(path)

    def set_favorite(self, is_favorite):
        """Set the favorite button.

        Args:
            is_favorite (bool): If True us favorite active button, otherwise
                use the inactive favorite button.

        """
        if is_favorite:
            icon = "favorite_active"
        else:
            icon = "favorite_inactive"

        self.is_favorite = is_favorite
        helper.set_icon(self.button_favorite, icon)


class PresetWidget(QtWidgets.QWidget):
    """Widget that represent a preset in the preset section."""

    def __init__(self):
        """Initialize the PresetWidget instance.

        """
        super(PresetWidget, self).__init__()

        self.create_widgets()
        self.create_layouts()
        self.setup_style()

    def setup_style(self):
        """Setup ui.

        Set up some style information after having tinted the widget's
        background in order to preserve custom styles.

        """
        self.setAttribute(QtCore.Qt.WA_StyledBackground, True)
        self.setStyleSheet("QWidget {background-color: #282828};")

        # Need to set this stylesheet directly in code -after- we have set
        # WA_StyledBackground for all widgets in order to ensure a tinted
        # background for the whole preset widget but preserving a different
        # color for the save button.
        self.button_save.setStyleSheet(
            "QPushButton { color: #cccccc; background-color: #30363d;outline: none; }"
        )

        self.text_preset_data.setStyleSheet("font-size: 14pt;")
        self.text_preset_data.setMinimumHeight(300)

    def create_widgets(self):
        """Create widgets."""
        self.button_toggle_details = QtWidgets.QToolButton()
        helper.set_icon(self.button_toggle_details, "arrow_right")
        self.label_name = QtWidgets.QLabel()
        self.button_reveal = QtWidgets.QToolButton()
        helper.set_icon(self.button_reveal, "reveal")
        self.button_automatic = QtWidgets.QToolButton()
        helper.set_icon(self.button_automatic, "favorite_inactive")
        self.text_preset_data = QtWidgets.QTextEdit()
        self.button_save = QtWidgets.QPushButton("Save")

    def create_layouts(self):
        """Create layouts."""
        self.layout_main = QtWidgets.QVBoxLayout()
        self.layout_top = QtWidgets.QHBoxLayout()
        self.layout_details = QtWidgets.QVBoxLayout()
        self.group_details = QtWidgets.QWidget()
        self.group_details.setLayout(self.layout_details)
        self.group_details.setVisible(False)

        self.layout_top.addWidget(self.button_toggle_details)
        self.layout_top.addWidget(self.label_name)
        self.layout_top.addStretch()
        self.layout_top.addWidget(self.button_automatic)
        self.layout_top.addWidget(self.button_reveal)

        self.layout_details.addWidget(self.text_preset_data)
        self.layout_details.addWidget(self.button_save)

        self.layout_main.addLayout(self.layout_top)
        self.layout_main.addWidget(self.group_details)

        self.setLayout(self.layout_main)
        helper.set_style_sheet(self)


class PathWidget(QtWidgets.QWidget):
    """Widget to browse to and set a root path.

    This is for example used for preset paths and nodes roots.

    """

    def __init__(self, path):
        """Initialize the PresetPathWidget instance.

        Args:
            path (str): Path to show in the input.

        """
        super(PathWidget, self).__init__()

        self.path = path

        self.create_widgets()
        self.create_layouts()

    def create_widgets(self):
        """Create widgets."""
        self.input = QtWidgets.QLineEdit(self.path)
        helper.fix_no_outline(self.input)
        self.button_add = QtWidgets.QToolButton()
        helper.set_icon(self.button_add, "plus")
        self.button_del = QtWidgets.QToolButton()
        helper.set_icon(self.button_del, "minus")

    def create_layouts(self):
        """Create layouts."""
        self.layout_main = QtWidgets.QHBoxLayout()

        self.layout_main.addWidget(self.input)
        self.layout_main.addWidget(self.button_add)
        self.layout_main.addWidget(self.button_del)

        self.setLayout(self.layout_main)

    def get_path(self):
        """Get the currently set path.

        Returns:
            str: The currently set path.

        """
        return self.input.text()

    def set_path(self, path):
        """Set up the given path in the input.

        Args:
            path (str): Absolute path to show in the input.

        """
        self.input.setText(path)


class WidgetDoubleClickable(QtWidgets.QWidget):
    """A QWidget that emits an event click when it is double clicked."""

    double_clicked = QtCore.Signal()

    def __init__(self):
        """Initialiye the WidgetDoubleClickable instance."""
        super(WidgetDoubleClickable, self).__init__()

        self.setStyleSheet(
            "QWidget { background-color: #333333; padding: 10px; max-height: 20px; }"
        )

    def mouseDoubleClickEvent(self, event):
        """Emit double clicked signal to toggle visibility.

        Args:
            event (QtCore.QEvent): The event that got triggered.

        """
        super(WidgetDoubleClickable, self).mouseDoubleClickEvent(event)
        self.double_clicked.emit()


class EmptyJobsWidget(QtWidgets.QWidget):
    """Widget to show when no jobs are populated in main view."""

    def __init__(self):
        """Initialize the EmptyJobsWidget widget."""
        super(EmptyJobsWidget, self).__init__()

        self.create_widgets()
        self.create_layouts()

    def create_widgets(self):
        """Create widgets."""
        self.label_drop = QtWidgets.QLabel()
        self.label_drop.setPixmap(QtGui.QPixmap(_ICONS["drop"]).scaled(
            QtCore.QSize(75, 75), QtCore.Qt.KeepAspectRatio)
        )
        self.label_drop.setAlignment(QtCore.Qt.AlignCenter)
        self.label_text_drag_file = QtWidgets.QLabel("Drag&Drop media here")
        self.label_text_drag_file.setProperty("style", "h1")
        self.label_text_drag_file.setAlignment(QtCore.Qt.AlignCenter)
        self.label_text_or = QtWidgets.QLabel("or")
        self.label_text_or.setProperty("style", "startup")
        self.label_text_or.setAlignment(QtCore.Qt.AlignCenter)
        self.button_browse = QtWidgets.QPushButton("Browse media")
        self.button_browse.setMaximumWidth(150)

    def create_layouts(self):
        """Create layouts."""
        self.layout_main = QtWidgets.QVBoxLayout()
        self.layout_main.setContentsMargins(0,0,0,0)

        self.group = QtWidgets.QGroupBox()
        self.group.setProperty("style", "EmptyJobsWidget")
        self.layout_group = QtWidgets.QVBoxLayout()
        self.group.setLayout(self.layout_group)

        self.layout_group.addStretch()
        self.layout_group.addWidget(self.label_drop)
        self.layout_group.addWidget(self.label_text_drag_file)
        self.layout_group.addWidget(self.label_text_or)
        self.layout_group.addWidget(
            self.button_browse, alignment=QtCore.Qt.AlignCenter
        )
        self.layout_group.addStretch()

        self.layout_main.addWidget(self.group)

        self.setLayout(self.layout_main)
        helper.set_style_sheet(self)


class TaskDetailVisibilityWidget(QtWidgets.QWidget):
    """Represent one detail to turn its visibility on or off."""

    def __init__(self, key, is_visible, main_ctrl, task_details_ctrl):
        """Initialize the TaskDetailVisibilityWidget instance.

        Args:
            key (str): The key that represents a detail in the task's data.json
                file.
            is_visible (bool): If True make the item visible, otherwise hide
                it.
            main_ctrl (smartMedia.mvc.controller.Controller): The main
                controller to be able to communicate back to the main window.
            task_details_ctrl (
                smartMedia.mvc.controller.TaskDetailsVisibilityViewController):
                The controller for the visibility window in order to
                re-populate it on widget move.

        """
        super(TaskDetailVisibilityWidget, self).__init__()

        self.label = key
        self.is_visible = is_visible
        self.main_ctrl = main_ctrl
        self.task_details_ctrl = task_details_ctrl

        self.create_widgets()
        self.create_layouts()
        self.create_signals()

        self.set_visibility_style()
        self.setMinimumHeight(30)

    def create_widgets(self):
        """Create widgets."""
        self.push_key = QtWidgets.QPushButton(self.label)
        self.push_key.setProperty("style", "mar_bottom")
        self.push_up = QtWidgets.QToolButton()
        helper.set_icon(self.push_up, "arrow_up")
        self.push_down = QtWidgets.QToolButton()
        helper.set_icon(self.push_down, "arrow_down")

    def create_layouts(self):
        """Create layouts."""
        self.layout = QtWidgets.QHBoxLayout()
        self.layout.setContentsMargins(10, 0, 10, 0)

        self.layout.addWidget(self.push_key)
        self.layout.addWidget(self.push_down)
        self.layout.addWidget(self.push_up)

        self.setLayout(self.layout)

    def create_signals(self):
        """Create signals."""
        self.push_key.clicked.connect(lambda: self.toggle_visibility())
        self.push_up.clicked.connect(lambda: self.move("up"))
        self.push_down.clicked.connect(lambda: self.move("down"))

    def move(self, direction):
        """Move the detail into given direction.

        Args:
            direction (str): The direction to move the detail to. Choose
                from ('up', 'down').

        """
        def _update_neighbour(index):
            """Update the neighbour widget and increase/decrease it.

            This swapping is needed to ensure an index gets only used once.
            The upper element gets thus moved below the moved element and
            vice vers.

            Args:
                 index (int): The new index of the moved widget and to search
                    for if any other element has it.

            """
            for key, value in self.main_ctrl.settings["task_details"].items():
                if value["index"] == index and key != self.label:
                    if direction == "up":
                        value["index"] += 1
                    else:
                        value["index"] -= 1

        detail_setting = self.main_ctrl.settings["task_details"][self.label]

        min_ = 0
        max_ = len(self.main_ctrl.settings["task_details"])

        if direction == "up":
            if detail_setting["index"] > min_:
                detail_setting["index"] -= 1
            else:
                pass
        else:
            if detail_setting["index"] < max_:
                detail_setting["index"] += 1

        _update_neighbour(detail_setting["index"])

        self.main_ctrl.settings.save()
        self.task_details_ctrl.populate_list()

        # Update all open task details in main window.
        for job_ctrl in self.main_ctrl.job_controller:
            for task_ctrl in job_ctrl.task_controller:

                # Either the task does not contain such a detail or the details
                # section hasn't been drawn, yet.
                if self.label not in task_ctrl.detail_widgets:
                    continue

                task_ctrl.build_details()

    def set_visibility_style(self):
        """Set the style based on the visibility settings."""
        icon = "eye" if self.is_visible else "eye_disabled"
        helper.set_icon(self.push_key, icon)

        color = "292f35" if self.is_visible else "333333"
        self.push_key.setStyleSheet("QPushButton {{ background-color: #{} }}".format(color))

    def toggle_visibility(self):
        """Toggle the visibility of the detail, update view and setting."""
        self.is_visible = not self.is_visible
        self.set_visibility_style()

        # Update all open task details in main window.
        for job_ctrl in self.main_ctrl.job_controller:
            for task_ctrl in job_ctrl.task_controller:

                # Either the task does not contain such a detail or the details
                # section hasn't been drawn, yet.
                if self.label not in task_ctrl.detail_widgets:
                    continue

                task_ctrl.detail_widgets[self.label].setVisible(self.is_visible)

        self.main_ctrl.settings["task_details"][self.label]["visible"] = self.is_visible
        self.main_ctrl.settings.save()


class ComboBoxWithCompleter(QtWidgets.QComboBox):
    """A combobox which has a completer to set existing values."""

    def __init__(self, parent=None):
        """Initialize the ComboBoxWithCompleter instance."""
        super(ComboBoxWithCompleter, self).__init__(parent)

        self.setFocusPolicy(QtCore.Qt.StrongFocus)
        self.setEditable(True)

        self.pFilterModel = QtCore.QSortFilterProxyModel(self)
        self.pFilterModel.setFilterCaseSensitivity(QtCore.Qt.CaseInsensitive)
        self.pFilterModel.setSourceModel(self.model())

        self.completer = QtWidgets.QCompleter(self)
        self.completer.setModel(self.pFilterModel)
        self.completer.setCompletionMode(
            QtWidgets.QCompleter.UnfilteredPopupCompletion
        )

        self.setCompleter(self.completer)

        def filter(text):
            self.pFilterModel.setFilterFixedString(str(text))

        self.lineEdit().textEdited[unicode].connect(filter)

    def setModelColumn(self, column):
        """Apply the model to the given column.

        Args:
            column (int): The column to apply the model to.

        """
        self.completer.setCompletionColumn(column)
        self.pFilterModel.setFilterKeyColumn(column)
        super(ComboBoxWithCompleter, self).setModelColumn(column)
