"""Path related functionality."""

# Import built-in modules
import os
import platform
import re
import shutil
import subprocess
import time
import uuid

# Import local modules
from smartMedia.constants import FILE_SEQUENCE_EXTENSIONS
from smartMedia.constants import PACKAGE_NAME
from smartMedia.constants import VIDEO_EXTENSIONS

# Absolute path to this directory.
THIS_DIR = os.path.dirname(__file__)


def get_package_dir(private=True):
    """Get absolute path of package directory for this package.

    Args:
        private (bool): If True get the private package directory, otherwise
            get the public one.

    Returns:
        str: Absolute path of package directory.

    """
    cragl_dir = ".cragl" if private else "cragl"
    return os.path.join(os.path.expanduser("~"), cragl_dir, PACKAGE_NAME)


def ensure_directory(path):
    """Create directory if it does not already exist.

    Args:
        path (str): Absolute path of directory to create.

    """
    try:
        os.makedirs(path)
    except OSError:
        # Folder exists already.
        pass


def sanitize(path):
    """Return a sanitized path.

    Args:
        path (str): The path to sanitize.

    Returns:
        str: The sanitized path.

    """
    try:
        return path.replace("\\", "/")
    except AttributeError:
        # A None type has no replace.
        return path


def cli_sanitized(path):
    """Sanitize a path for the cli, wrapping it inside quotes.

    When using a path in the cli we need to ensure to wrap it around double
    quotes and to use forward slashes, otherwise the commands won't execute
    as needed.

    Args:
        path (str): The path to translate to a cli ready path.

    Returns:
        str: The path sanitized to be used as cli argument.

    """
    return '"{}"'.format(sanitize(path))


def reveal(path):
    """Reveal the given path in the explorer.

    Args:
        path (str): Absolute path to reveal in the explorer.

    """
    if platform.system() == "Windows":
        os.startfile(path)
    elif platform.system() == "Darwin":
        subprocess.Popen(["open", path])
    else:
        subprocess.Popen(["xdg-open", path])


def force_extension(path, extension):
    """Return the given path using the given extension.

    Args:
        path (str): Absolute path of the extension to use.
        extension (str): The extension to use. Does not need a fullstop as it
            will be added if not provided.

    Returns:
        str: The given path with the given extension.

    """
    base = os.path.splitext(path)[0]
    extension = extension.replace(".", "")
    return "{}.{}".format(base, extension)


def get_render_task_script_path():
    """Get absolute path to the render script file to render a task.

    Returns:
        str: Absolute path to the render script file to render a task.

    """
    return os.path.join(THIS_DIR, "terminal", "render_task.py")


def get_render_job_data_script_path():
    """Get absolute path to the render script file to render a thumbnail.

    Returns:
        str: Absolute path to the render script file to render a thumbnail.

    """
    return os.path.join(THIS_DIR, "terminal", "render_job_data.py")


def get_create_template_nk_path():
    """Get absolute path to the script file to create a template.nk file.

    Returns:
        str: Absolute path to the script file to create a template.nk file.

    """
    return os.path.join(THIS_DIR, "terminal", "create_template_nk.py")


def get_default_jobs_root():
    """Get the default root directory where all jobs live.

    Returns:
        The default root directory where all jobs live.

    """
    root = os.path.join(get_package_dir(private=False), "jobs")
    ensure_directory(root)
    return root


def get_blueprint_root():
    """Get absolute path to the blueprint root directory.

    Returns:
        str: Absolute path to the blueprint root directory.

    """
    directory = os.path.join(
        THIS_DIR,
        "..",
        "blueprints"
    )
    return os.path.normpath(directory)


def delete(filepath_or_dirpath):
    """Delete the given file or directory.

    Args:
        filepath_or_dirpath (str): Absolute path to the file or directory
            to delete.

    """
    if os.path.isfile(filepath_or_dirpath):
        remove_func = os.remove
    else:
        remove_func = shutil.rmtree

    remove_func(filepath_or_dirpath)


def get_preset_default():
    """Get absolute path to the default preset directory.

    Returns:
        str: Absolute path to the default preset directory.

    """
    return os.path.join(THIS_DIR, "..", "presets", "_default")


def is_dev():
    """Check if we are in dev mode.

    Returns:
        bool: True when we are in dev mode, otherwise False when we are in
            production mode.

    """
    return os.path.exists(os.path.join(THIS_DIR, ".dev"))


def get_version_number():
    """Get the version number from this package.

    Returns:
        str: The version number from this package.

    """
    if is_dev():
        package_root = os.path.join(THIS_DIR, "..")
    else:
        package_root = os.path.join(THIS_DIR, "..", "..")

    package_root = os.path.abspath(package_root)
    package_version_name = os.path.basename(package_root)
    return package_version_name.split("_v")[1]


def reveal(path):
    """Reveal the given path in the explorer.

    Args:
        path (str): Absolute path to reveal in the explorer.

    """
    if platform.system() == "Windows":
        os.startfile(path)
    elif platform.system() == "Darwin":
        subprocess.Popen(["open", path])
    else:
        subprocess.Popen(["xdg-open", path])


def new_job_task_name(job=False, task=False):
    """Get a new folder name using a timestamp and a unique id.

    Args:
        job (optional): If True use a Job identifier.
        task (optional): If True use a Task identifier.

    We use the time stamp in milliseconds to having jobs and tasks sorted
    alphabetically and append another unique identifier to ensure uniqueness,
    even when creating new jobs or tasks within milliseconds.

    Returns:
        str: A new unique name to use for jobs and tasks

    """
    if job:
        char = "J"
    elif task:
        char = "T"
    else:
        raise NotImplemented

    return "{}_{}_{}".format(
        str(time.time()).replace(".", ""),
        char,
        uuid.uuid4()
    )

def is_job_task_name(folder_name):
    """Check if the given folder name is a job or task_name.

    Args:
        folder_name (str): Name of the folder to check if it is a job or a
            task.

    Returns:
        bool, bool: True if the folder name is a job, False otherwise and
            True if the folder name is a task, False otherwise.

    """
    # https://regex101.com/r/kqQ3e1/1
    pattern_job = r"[\d]+_J_[\w]{8}-[\w]{4}-[\w]{4}-[\w]{4}-[\w]{12}"

    # https://regex101.com/
    pattern_tsk = r"[\d]+_T_[\w]{8}-[\w]{4}-[\w]{4}-[\w]{4}-[\w]{12}"

    return (
        bool(re.match(pattern_job, folder_name)),
        bool(re.match(pattern_tsk, folder_name)),
    )


def get_default_template_nk():
    """Get absolute path of the default preset template.nk file.

    In case a preset does not contain a template.nk file we will use the
    default preset's template.nk file for the task.

    Returns:
        str: absolute path of the default preset template.nk.

    """
    return os.path.join(THIS_DIR, "presets", "_default", "template.nk")


def get_nodes_directory():
    """Get absolute path of the nodes directory.

    Returns:
        str: Absolute path of the nodes directory.

    """
    return os.path.normpath(os.path.join(THIS_DIR, "..", "nodes"))


def get_all_nodes_from_paths():
    """Get all .nk and .gizmo files from all nodes roots.

    Returns:
        :obj:`list` of :obj:`str`: Absolute paths of all .nk and .gizmo files
            from all nodes roots.

    """
    from smartMedia.settings import Settings
    settings = Settings()

    nodes = []
    for root in settings["nodes_roots"]:
        if not os.path.isdir(root):
            continue

        nodes.extend([
            os.path.join(root, name)
            for name in os.listdir(root)
            if name.endswith(".nk") or name.endswith(".gizmo")
        ])

    return nodes


def add_remove_filesequence_counter(path):
    """Add or remove file sequence counter based on media extension.

    Args:
        path (str): Absolute path to add or remove file sequence counter.

    Returns:
        str: The updated path that contains a file sequence counter when the
            path handled a file sequence extension or no one when it did not
            handle one.

    """
    path_, ext = os.path.splitext(path)

    if ext in VIDEO_EXTENSIONS or ext not in FILE_SEQUENCE_EXTENSIONS:
        path = path.replace(".####", "")
    elif ext in FILE_SEQUENCE_EXTENSIONS and ".####" not in path:
        path = "{}.####.{}".format(path_, ext.replace(".", ""))

    return path
