"""Important requirements that must be set when smartMedia launches."""

# Import built-in modules
import logging
import re

# Import local modules
from smartMedia import defaults
from smartMedia.colorspace import colorspaces
from smartMedia.path_resolver import PathResolver
from smartMedia.settings import Settings
from smartMedia.mvc.details_widgets import DETAILS_WIDGETS
from smartMedia.mvc.dialogs import message
from smartMedia.mvc.dialogs import path_browser
from smartMedia import paths


def initialize(view):
    """Run all required steps before smartMedia can launch.

    Args:
        view (smartMedia.mvc.view.View): The main window to parent any message
            to.

    """
    colorspaces.copy_to_user_folder()
    add_to_task_details_settings()
    check_nuke_default_executable_set(view)
    ensure_version_related_settings()
    # The following import ensures to register all custom commands. Although
    # this module is unused, i.e. no further needed, we still require to
    # import it here as it imports all custom commands into global scope.
    from smartMedia import media_commands


def add_to_task_details_settings():
    """Add all data elements to the settings visibility list.

    This ensures the items can be shown/hidden in the 'Edit Task Details
    visibility' window.

    """
    settings = Settings()

    added_items = 0
    for key, value in DETAILS_WIDGETS.items():

        if key in settings["task_details"]:
            continue

        next_index = len(settings["task_details"]) + 1
        settings["task_details"][key] = {
            "visible": True,
            "index": next_index,
            "custom_label": value.get("custom_label", "")
        }
        added_items += 1

    if added_items:
        settings.save()


def _store_nuke_executable_from_current_nuke_session():
    """Use nuke executable when running in Nuke environment.

    Returns:
        bool: True when setting Nuke executable from current session as the
            default Nuke executable, otherwise False, i.e. when not running
            in Nuke session.

    """
    try:
        import nuke
    except ImportError:
        return False

    settings = Settings()
    nuke_exe_path = nuke.env["ExecutablePath"]
    settings["nuke_executables"].insert(0, [nuke_exe_path, True])
    settings.save()

    return True


def check_nuke_default_executable_set(view):
    """Check if a Nuke default executable is set and start browser if not.

    Args:
        view (smartMedia.mvc.view.View): The main window to parent any message
            to.

    Returns:
        bool: True when a default Nuke executable is set, otherwise False.

    """
    if defaults.get_default_nuke_version_exe():
        return True

    exe_from_nuke = _store_nuke_executable_from_current_nuke_session()
    if exe_from_nuke:
       return True

    message_ = (
        "No default Nuke executable is set. A Nuke executable is needed "
        "in order to ingest and render data. Please browse to a Nuke "
        "executable to use as a default for any ingestion and rendering "
        "process."
    )
    message.show(view.button_search, message_)

    title = "Browse Nuke executable"
    path = path_browser.show(title, only_directory=True)
    path_resolver = PathResolver(path)

    try:
        path_resolver.run()
    except OSError as error:
        message.show(view.button_search, str(error))
        return False

    settings = Settings()
    settings["nuke_executables"].insert(0, [path_resolver.nuke, True])
    settings.save()

    msg = (
        "Successfully found Nuke executable in:\n"
        "{}".format(path_resolver.nuke)
    )
    message.show(view.button_search, msg)

    return True


def ensure_version_related_settings():
    """Ensure all settings relating to package data use this version.

    There are several settings values that reference version data from
    smartMedia, e.g. the nodes_roots (relating to nodes located in this package)
    as well as preset_paths (presets located in this package). When updating
    the smartMedia version then this would still read package data from the
    previous version. Here we ensure to use the package data of this current
    smartMedia version by replacing these version related settings values.

    """
    settings = Settings()
    logger = logging.getLogger(__file__)

    # Update preset_paths.
    # https://regex101.com/r/8sE9xh/1
    presets_pattern = r"(?P<base>.*smartMedia_v)(?P<version>[0-9]+\.[0-9]+\.[0-9]+)(?P<suffix>\/presets)"

    for index, path in enumerate(settings["preset_paths"]):
        match = re.match(presets_pattern, path)
        if not match:
            continue

        settings["preset_paths"][index] = _update_settings_value_version_number(match)

        if settings["preset_paths"][index] != path:
            logger.info("Updating preset_paths setting to reflect latest version.")
            settings.save()

    # Update nodes_roots.
    # https://regex101.com/r/LD0T9d/1
    presets_pattern = r"(?P<base>.*smartMedia_v)(?P<version>[0-9]+\.[0-9]+\.[0-9]+)(?P<suffix>\/nodes)"

    for index, path in enumerate(settings["nodes_roots"]):
        match = re.match(presets_pattern, path)
        if not match:
            continue

        settings["nodes_roots"][index] = _update_settings_value_version_number(
            match)

        if settings["nodes_roots"][index] != path:
            logger.info("Updating nodes_root setting to reflect latest version.")
            settings.save()


def _update_settings_value_version_number(match):
    """Replace version number of the match with the package's version number.

    Args:
        match (re.Match): The match object to update.

    """
    group_data = match.groupdict()

    return "{}{}{}".format(
        group_data["base"],
        paths.get_version_number(),
        group_data["suffix"],
    )
