"""Render job data, i.e. a source's metadata and a thumbnail."""

# Import built-in modules
import json
import logging
import os
import sys

# Import third-party modules
import nuke

# Import local modules
# In order to call this module from external, we need to add smartMedia's
# root directory to the sys path so that we can import the smartMedia python
# package.
sys.path.append(os.path.join(os.path.dirname(__file__), "..", ".."))
from smartMedia.clogging import LEVEL
from smartMedia.constants import JSON_INDENT
from smartMedia.constants import THUMBNAIL_WIDTH
from smartMedia.constants import VIDEO_EXTENSIONS
from smartMedia.media import Media
from smartMedia.settings import Settings
from smartMedia.naming import Naming


def render_thumbnail(source, dest):
    """Render a thumbnail for the given source to the given dest.

    Args:
        source (str): Absolute path of source to render a thumbnail for.
        dest (str): Absolute path of where to render the thumbnail to.

    """
    logger = logging.getLogger("Thumbnail")
    logging.basicConfig()
    logger.setLevel(LEVEL)

    logger.debug("Setting source to: %s", source)
    read = nuke.createNode("Read")
    read["file"].setValue(source)

    reformat = nuke.createNode("Reformat")
    reformat["type"].setValue("to box")
    reformat["box_width"].setValue(THUMBNAIL_WIDTH)

    logger.debug("Setting dest to: %s", dest)
    write = nuke.createNode("Write")
    write["file"].setValue(dest)

    nuke.execute(write, 1, 1, 1)

    if os.path.isfile(dest):
        logger.debug("Successfully created thumbnail :)")
    else:
        raise RuntimeError("Failed creating thumbnail.")


def collect_metadata(source, meta_path):
    """Get all needed metadata about the given source.

    Args:
        source (str): Absolute path of source to collect metadata from.
        meta_path (str): Absolute path to write the metadata to.

    """
    logger = logging.getLogger("Collect Metadata")
    logging.basicConfig()
    logger.setLevel(LEVEL)

    logger.debug("Collecting metadata for source: %s", source)
    read = nuke.createNode("Read")

    # Video ingestions.
    _, ext = os.path.splitext(source)
    if ext.lower() in VIDEO_EXTENSIONS:
        # Using setValue on video files will not work as Nuke will not
        # detect the source's frame range. We need fromUserText so that
        # Nuke will set up the frame range correctly. For additional
        # information see:
        # https://community.foundry.com/discuss/post/891640
        # Using fromUserText will both set the file knob value and the
        # frame range.
        read["file"].fromUserText(source)
        start = read["first"].value()
        end = read["last"].value()
        source_path = source

    # File sequence ingestions.
    else:
        media = Media(source)
        read["file"].setValue(source)
        source_path = media.path
        start = media.first
        end = media.last

    meta = {
        "source": source_path,
        "specs": {
            "resolution": [read.width(), read.height()],
            "start": start,
            "end": end,
            "length": end - start + 1
        },
        "metadata": read.metadata(),
    }

    with open(meta_path, "w") as file_:
        json.dump(meta, file_, indent=JSON_INDENT)


def extract_naming(source, meta_path):
    """Get all needed metadata about the given source.

    Args:
        source (str): Absolute path of source to collect metadata from.
        meta_path (str): Absolute path to write the metadata to.

    """
    logger = logging.getLogger("Extract Naming")
    logging.basicConfig()
    logger.setLevel(LEVEL)

    logger.debug("Extracting naming value for source: %s", source)

    settings = Settings()
    auto_extract = settings["auto_apply_naming_extraction"]

    naming = Naming(source)

    if auto_extract:
        patterns = naming.get_all_pattern_names()
    else:
        patterns = [settings["current_naming"]]

    patterns = [pattern for pattern in patterns if pattern]
    if not any(patterns):
        return

    logger.info("Trying to extract naming values using patterns: %s", patterns)
    for pattern in patterns:
        try:
            naming.apply(pattern)
            logger.info("Found matching pattern '%s'.", pattern)
            break
        except RuntimeError as error:
            logger.info("Pattern '%s' did not match: %s", pattern, str(error))

    with open(meta_path, "r") as file_:
        content = json.load(file_)

    content["naming_preset"] = pattern
    content["naming_values"] = naming.values

    with open(meta_path, "w") as file_:
        json.dump(content, file_, indent=JSON_INDENT)


def main():
    """Main entry point to generate job data."""
    source = sys.argv[1]
    dest = sys.argv[2]
    meta = sys.argv[3]

    render_thumbnail(source, dest)
    collect_metadata(source, meta)
    extract_naming(source, meta)


if __name__ == "__main__":
    main()
