"""Translate a job's settings value into an action.

Each task contains a data.json file. This file contains key-value pairs.
Here we tell Nuke what to do on the render.nk file for each key. This happens
before we render the render.nk file. Each key corresponds to a method (but that
is not mandatory, there might not be a method needed for each and every key) of
the translator and it will set the given value.

As an example take the following key value pair of a data.json:

    {
        ...
        target_colorspace: 'sRGB'
        ...
    }

The Translator contains a method with the very same name as the key, so
`def target_colorspace(value)`. In here we perform the logic for this key in
order to set our render.nk file up the way we need it. In that example, we set
the colorspace of our Write node to the specified value, which in that case is
'sRGB'.

"""

# Import built-in modules
import logging

# Import third-party modules
import nuke

# Import local modules
from smartMedia.clogging import LEVEL
from smartMedia.media import Media


class Translator(object):
    """Translate a job's settings value into an action."""

    node_name_source = "smartMedia_source"
    node_name_reformat = "smartMedia_reformat"
    node_name_target = "smartMedia_target"

    def __init__(self, job_data):
        """Initialize the Translator instance.

        Args:
            job_data (dict): The job's data that contains key-value pairs for
                all job settings. This is occasionally needed for some steps
                in case they need to refer to other settings.

        """
        self.job_data = job_data
        self.source = nuke.toNode(self.node_name_source)
        self.reformat_node = nuke.toNode(self.node_name_reformat)
        self.target = nuke.toNode(self.node_name_target)

        self.ensure_minimal_nodes_in_template()

        self.logger = logging.getLogger(__name__)
        self.logger.setLevel(LEVEL)

    def ensure_minimal_nodes_in_template(self):
        """Ensure we have a minimal template file.

        Raises:
            RuntimeError: When at least one crucial node does not exist.

        """
        crucial_nodes = (
            self.node_name_source,
            self.node_name_target,
        )

        for name in crucial_nodes:
            if not nuke.toNode(name):
                raise RuntimeError(
                    "Template does not contain node '{}'. Cannot render "
                    "task.".format(name)
                )

    def preset(self, value):
        """Record the preset that was used by writing it into the root.

        Args:
            value (str): The name of the preset that was used which will be
                written into the working file's root (`label` knob).

        """
        self.logger.debug("Using preset %s", value)

        comment_knob = nuke.root()["label"]
        comment_knob.setValue("{}\n{}".format(comment_knob.value(), value))

    def reformat(self, value):
        """Reformat the source by the given value.

        Args:
            value (dict): Information about the reformat mode to use and the
                value to apply. Examples:

                # Same as source: Don't apply any reformat, simply bypass.
                {
                    "type": "same as source",
                    "value": "",
                }

                # Reformat to using a specific format.
                {
                    "type": "to format",
                    "value": "1920 1080 0 0 1920 1080 1 HD_1080",
                }

                # Scale by a certain percent, e.g. 50%, i.e. halfres.
                {
                    "type": "scale",
                    "value": 0.5,
                }

                # Use a specific width, auto adjust the height by the width.
                {
                    "type": "to box",
                    "value": {
                        "width": 1000,
                        "height": 0,
                        "force_shape": False,
                        "pixel_aspect": 1.0,
                    }
                }

                Alternatively, force a specific height by setting a specific
                height and setting force_shape to True

        """
        if not self.reformat_node:
            self.logger.info(
                "No reformat node '{}' in DAG. Will not apply any "
                "reformat.".format(self.node_name_reformat)
            )
            return

        type_ = value["type"]
        value = value["value"]

        self.logger.debug("Setting reformat to %s -> %s", type_, value)

        if type_ == "same as source":
            self.logger.info("Disable reformat to use the source resolution.")
            self.reformat_node["disable"].setValue(True)

        elif type_ in ("format", "to format"):
            self.logger.info("Adding format '%s'", value)
            nuke.addFormat(value)

            self.reformat_node["type"].setValue("to format")
            # The format name is always at the end, so we split it off in
            # order to set the right format.
            self.reformat_node["format"].setValue(value.split(" ")[-1])

        elif type_ == "scale":
            self.reformat_node["type"].setValue("scale")
            self.reformat_node["scale"].setValue(float(value))

        elif type_ == "to box":
            self.reformat_node["type"].setValue("to box")
            self.reformat_node["box_width"].setValue(value["width"])
            self.reformat_node["box_height"].setValue(value["height"])
            self.reformat_node["box_fixed"].setValue(value["force_shape"])
            self.reformat_node["box_pixel_aspect"].setValue(value["pixel_aspect"])

    def source_path(self, value):
        """Set the source path by the given value.

        Args:
            value (str): Absolute path to set as source path.

        """
        self.logger.debug("Setting source to %s", value)

        media = Media(value)

        nuke.frame(media.first)
        self.source["file"].setValue(media.path)

    def source_colorspace(self, value):
        """Set the given colorspace for the Read node.

        Args:
            value (str): The colorspace to apply to the Read node.

        """
        if not value:
            self.logger.info("Will not apply any colorspace as none is set.")
            return

        self.logger.debug("Setting source colorspace to: %s", value)
        self.source["colorspace"].setValue(value)

    def source_frame_mode(self, value):
        """Set the source frame range depending on set mode.

        Notes:
            Depending on the chosen frame mode, this method refers to other
            keys, namely `orig_source_start`, `orig_source_end` and
            `source_start`. These are always set with each task.

        Args:
            value (str): The frame mode to use. Should be one of:
                - source: Use the source's frame range.
                - start at: Start at the given source_start.
                - custom: Use the given source_start and source_end.

        """
        if value in ("source", "same as source"):
            self.source["first"].setValue(self.job_data["orig_source_start"])
            self.source["origfirst"].setValue(self.job_data["orig_source_start"])
            self.source["last"].setValue(self.job_data["orig_source_end"])
            self.source["origlast"].setValue(self.job_data["orig_source_end"])

            self.source["frame_mode"].setValue("expression")
            self.source["frame"].setValue("")

        elif value == "start at":
            self.source["first"].setValue(self.job_data["orig_source_start"])
            self.source["origfirst"].setValue(self.job_data["orig_source_start"])
            self.source["last"].setValue(self.job_data["orig_source_end"])
            self.source["origlast"].setValue(self.job_data["orig_source_end"])

            self.source["frame_mode"].setValue("start at")
            self.source["frame"].setValue(str(self.job_data["source_start"]))

        elif value == "custom":
            self.source["first"].setValue(self.job_data["source_start"])
            self.source["origfirst"].setValue(self.job_data["source_start"])
            self.source["last"].setValue(self.job_data["source_end"])
            self.source["origlast"].setValue(self.job_data["source_end"])

            self.source["frame_mode"].setValue("expression")
            self.source["frame"].setValue("")

    def target_path(self, value):
        """Set the path to render to.

        Args:
            value (str): Absolute path to render to.

        """
        self.logger.debug("Setting Write's output path to %s", value)
        # Setting path using fromUserText as this will set up several other
        # knobs. Nuke will build additional knobs based on the file type of
        # the entered path. Using fromUserText is as close to entering the
        # path in a gui session manually which will then also build additional
        # knobs.
        self.target["file"].fromUserText(value)

    def target_start(self, value):
        """Set the target start point.

        Args:
            value (int): The start frame to use.

        """
        self.logger.debug("Setting target start frame to %s", value)

        self.target["use_limit"].setValue(True)
        self.target["first"].setValue(value)

    def target_end(self, value):
        """Set the target end point.

        Args:
            value (int): The end frame to use.

        """
        self.logger.debug("Setting target last frame to %s", value)

        self.target["use_limit"].setValue(True)
        self.target["last"].setValue(value)

    def target_fps(self, value):
        """Set up the frames per second to use in the output video file.

        Notes:
            The target_fps key is only set (and consequently this method only
            runs) when rendering a video file.

        Args:
            value (str): The frames per second to use in the output video file.

        """
        knobs = {"mov32_fps", "mov64_fps"}
        for knob_name in knobs:
            knob = self.target.knob(knob_name)
            if not knob:
                continue
            self.logger.info("Setting %s to %s", knob_name, value)
            knob.setValue(float(value))

    def target_ext(self, value):
        """Set the given file extension in the Write node.

        Args:
            value (str): The file extension to use.

        """
        self.logger.debug("Setting Write file extension to %s", value)

        if value == "jpg":
            # Nuke knows only jpeg. Forcing this type when trying to set jpg.
            value = "jpeg"

        self.target["file_type"].setValue(str(value))

    def target_codec(self, value):
        """Set the target codec.

        Notes:
            This is a simplistic way of setting up a Write node. Setting up
            a Write node all the way down to every detail is better done by
            creating a new preset and setting up the Write node in that
            preset directly and not using the target_codec step.

        Args:
            value (str): The codec to apply in the Write node.

        """
        self.logger.debug("Setting codec to: '%s'", value)

        codec_knobs_names = (
            # Nuke-11.3 and below.
            "meta_codec",
            # Nuke-12 and above.
            "mov64_codec"
        )
        for codec_knobs_name in codec_knobs_names:
            knob = self.target.knob(codec_knobs_name)
            if not knob:
                continue
            set_value = knob.setValue(value)

            self.logger.debug(
                "-- {} -> {} ({})".format(codec_knobs_name, value, set_value)
            )

    def target_colorspace(self, value):
        """Set the given colorspace for the Write node.

        Args:
            value (str): The colorspace to apply.

        """
        if not value:
            self.logger.info("Will not apply any colorspace as none is set.")
            return

        self.logger.debug("Setting target colorspace to: %s", value)
        self.target["colorspace"].setValue(value)
