"""Create new data from given source media."""

# Import built-in modules
import logging
import os
import subprocess
import threading
import time

# Import local modules
from smartMedia import defaults
from smartMedia import paths
from smartMedia.clogging import LEVEL
from smartMedia.paths import cli_sanitized
from smartMedia.settings import Settings


class JobCreator(object):
    """Create a new job for given source media.

    Running the JobCreator will create the needed data for a job - A thumbnail
    and the metadata for the given source. We do both in one go to speed
    things up. We could do both in separate calls but in order to run both
    things as fast as possible, we combine both. Speed before design here.

    """

    def __init__(self, media):
        """Initialize the JobCreator instance.

        Args:
            media (smartMedia.media.Media): The media to create a new job for.

        """
        self.media = media

        self.settings = Settings()
        self.job_id = paths.new_job_task_name(job=True)
        self.jobs_root = self.settings["jobs_root"]

        self.logger = logging.getLogger(self.__class__.__name__)
        self.logger.setLevel(LEVEL)

    def run(self, wait=False):
        """Create a new job for the media in a separate thread.

        Args:
            wait (bool, optional): If True wait until the thread has finished,
                otherwise proceed immediately. Defaults to False to proceed
                immediately.

        """
        thread = threading.Thread(target=self._run)
        thread.setDaemon(True)
        thread.start()

        if wait:
            thread.join()

    def _run(self):
        """Create a new job for the media."""
        self.job_root = self._create_job_root()
        self.meta_root = self._create_meta_root()

        start = time.time()
        self._create_job_data()
        self.logger.debug("Rendering thumbnail took: %ss", time.time() - start)

    def _create_job_root(self):
        """Create the job root.

        Returns:
            str: Absolute path of the job location.

        """
        job_root = os.path.join(self.jobs_root, self.job_id)
        self.logger.debug("Creating job root in %s", job_root)
        paths.ensure_directory(job_root)

        return job_root

    def _create_meta_root(self):
        """Create the meta root.

        Returns:
            str: Absolute path of the job location.

        """
        meta_root = os.path.join(self.job_root, ".meta")
        self.logger.debug("Creating meta folder in %s", meta_root)
        paths.ensure_directory(meta_root)

        return meta_root

    def _create_job_data(self):
        """Create thumbnail for media and collect meta data."""
        source = self._get_thumb_source_path()
        dest = os.path.join(self.meta_root, "thumb.jpg")

        command = self._assemble_command(source, dest)
        command = " ".join(command)

        self.logger.debug("Creating thumbnail using command: %s", command)
        subprocess.check_call(command, shell=True)

    def _assemble_command(self, source, dest):
        """Assemble command to create thumbnail and collect the metadata.

        Args:
            source (str): Absolute path of the source file to create a
                thumbnail for.
            dest (str): Absolute path to save the thumbnail to.

        Returns:
            :obj:`list` of :obj:`str`: The command to run to create a
                thumbnail.

        """
        return [
            cli_sanitized(defaults.get_default_nuke_version_exe()),
            "-i",
            "-t",
            cli_sanitized(paths.get_render_job_data_script_path()),
            cli_sanitized(source),
            cli_sanitized(dest),
            cli_sanitized(os.path.join(self.meta_root, "meta.json"))
        ]

    def _get_thumb_source_path(self):
        """Get the source path to use for the thumbnail.

        Returns:
            str: Absolute path of the thumbnail to use.

        """
        thumb_to_use = self.settings["thumbnail_index"]

        if thumb_to_use == "first":
            path = self.media.files[0]
        elif thumb_to_use == "middle":
            path = self.media.files[int(len(self.media.files) / 2.0)]
        elif thumb_to_use == "last":
            path = self.media.files[-1]

        return path
