"""Base widget to display a task's setting.

This base class provides the needed structure to represent a setting in a
task's data.json file. This is the interface to be able to communicate with
the data.json file via the main window in order to change a task's data.json
file.

For that we need a base structure that other details widgets can inherit from.
That is what this module is about. Note that usually it is not advised to
mix view and controller, i.e. business logic to drive the widgets. However,
due to the small size of details widgets we decided to keep view and controller
together to make this simple for you. So, expect to find view and controller
functionality in these details widgets together.

"""

# Import third-party modules
try:
    from PySide import QtCore
    from PySide import QtGui as QtWidgets
except ImportError:
    from PySide2 import QtCore
    from PySide2 import QtWidgets


class Base(QtWidgets.QWidget):
    """Base widget to display a task's setting."""

    recalculate_frame_range = QtCore.Signal()

    def __init__(self, preset, keys, custom_label=None):
        """Initialize the Base instance.

        Args:
            preset (smartMedia.oreset.Preset): The preset instance that holds
                all task's settings. This is needed because depending on the
                specific setting that we want to represent we need to have
                access to the other settings of the same task.
            keys (:obj:`list` of :obj:`str` or str): Sequence of setting keys
                or exactly -one- settings key to represent.
            custom_label (str, optional): The label to use for the widget.
                If not given we use the key and replace underscores with white
                space.

        """
        super(Base, self).__init__()

        self.preset = preset

        # Provided several settings to control, concatenated by pipe.
        if "|" in keys:
            self.keys  = keys.split("|")
            label = self.keys[0].replace("_", " ")

        # Provide only one setting to control.
        else:
            self.key = keys
            label = self.key.replace("_", " ")

        self.label_text = custom_label or label

        self.create_widgets()
        self.create_layouts()
        self.create_signals()

    def create_widgets(self):
        """Create widgets."""
        pass

    def create_layouts(self):
        """Create layouts."""
        self.layout = QtWidgets.QHBoxLayout()
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.setLayout(self.layout)

    def create_signals(self):
        """Create signals.

        Usually, it is not a good habit to mix model-view and controller in
        one module. However, due to the fact that detail widgets are basically
        very small and we need lots of them, we decided to keep everything
        contained in one module and one class in order to keep things together.
        Splitting concerns given the fact that we have lots and lots of detail
        widgets would not increase the readability and overview at all.

        """
        pass
