"""Provide a matrix for adding custom attributes."""

# Import third-party modules
try:
    from PySide import QtCore
    from PySide import QtGui as QtWidgets
except ImportError:
    from PySide2 import QtCore
    from PySide2 import QtWidgets

# Import local modules
from smartMedia.mvc.details_widgets.base import Base
from smartMedia.mvc import helper


class CustomAttributes(Base):
    """Provide a matrix for adding custom attributes."""

    def create_widgets(self):
        """Create widgets."""
        super(CustomAttributes, self).create_widgets()

        self.label = QtWidgets.QLabel(self.label_text)
        self.label.setMinimumWidth(120)
        self.label.setAlignment(QtCore.Qt.AlignTop)
        self.button_add_first = QtWidgets.QToolButton()
        helper.set_icon(self.button_add_first, "plus")

        self.properties = []

    def create_layouts(self):
        """Create layouts."""
        super(CustomAttributes, self).create_layouts()

        self.layout.addWidget(self.label)
        self.layout_attributes = QtWidgets.QVBoxLayout()
        self.layout.addLayout(self.layout_attributes)
        self.layout.addWidget(self.button_add_first)

        self.populate()
        self.set_add_first_visibility()

    def create_signals(self):
        """Create signals."""
        super(CustomAttributes, self).create_signals()

        self.button_add_first.clicked.connect(lambda: self.add_property())

    def populate(self):
        """Populate the matrix."""
        custom_data = self.preset["custom"] or {}
        for key, value in custom_data.items():
            self.add_property(key=key, value=value)

    def add_property(self, key="", value=""):
        """Add a new property.

        Args:
            key (str, optional): Value of the property key to show. Defaults
                to empty string.
            value (str, optional): Value of the property value to show.
                Defaults to empty string.

        """
        widget = CustomAttribute(key, value, self)
        self.layout_attributes.addWidget(widget)
        self.properties.append(widget)

        self.set_add_first_visibility()

    def set_add_first_visibility(self):
        """Set visibility of first plus button.

        This button is only needed when there is no custom property. Each
        property adds its own add button, thus once a property is visible we
        don't need to show this widget.

        """
        self.button_add_first.setVisible(not bool(self.properties))

    def update(self):
        """Update the preset and thus the task's data.json"""
        data = {}
        for property in self.properties:
            key, value = property.get_values()
            data[key] = value

        self.preset[self.key] = data
        self.preset.save()

        self.set_add_first_visibility()


class CustomAttribute(QtWidgets.QWidget):
    """A custom attribute to add to the matrix."""

    def __init__(self, key, value, parent):
        """Initialize the CustomAttribute instance.

        Args:
            key (str): Name of the key to display.
            value (str): The value to display.
            parent (
            smartMedia.mvc.details_widgets.custom_attributes.CustomAttributes):
                The CustomAttributes matrix to be able to communicate back to
                it.

        """
        super(CustomAttribute, self).__init__()

        self.key = key
        self.value = value
        self.parent = parent

        self.create_widgets()
        self.create_layouts()
        self.create_signals()

    def create_widgets(self):
        """Create widgets."""
        self.input_property_name = QtWidgets.QLineEdit(self.key)
        self.input_property_name.setPlaceholderText("enter property name")
        helper.fix_no_outline(self.input_property_name)

        self.input_property_value = QtWidgets.QLineEdit(self.value)
        self.input_property_value.setPlaceholderText("enter property value")
        helper.fix_no_outline(self.input_property_value)

        self.button_add = QtWidgets.QToolButton()
        helper.set_icon(self.button_add, "plus")

        self.button_del = QtWidgets.QToolButton()
        helper.set_icon(self.button_del, "minus")

    def create_layouts(self):
        """Create layouts."""
        self.layout = QtWidgets.QHBoxLayout()
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.layout.addWidget(self.input_property_name)
        self.layout.addWidget(self.input_property_value)
        self.layout.addWidget(self.button_add)
        self.layout.addWidget(self.button_del)

        self.setLayout(self.layout)

    def create_signals(self):
        """Create signals."""
        self.button_add.clicked.connect(lambda: self.add_property())
        self.button_del.clicked.connect(lambda: self.delete_property())
        self.input_property_name.textChanged.connect(lambda: self.parent.update())
        self.input_property_value.textChanged.connect(lambda: self.parent.update())

    def get_values(self):
        """Get the key and value from widget.

        Returns:
            str, str: The key and value from the widget.

        """
        return (
            self.input_property_name.text(),
            self.input_property_value.text()
        )

    def add_property(self):
        """Add a new property."""
        self.parent.add_property()
        self.parent.update()

    def delete_property(self):
        """Delete current property and remove from data.json."""
        self.parent.properties.remove(self)

        self.setVisible(False)
        self.setParent(None)
        self.deleteLater()

        self.parent.update()
