"""Ask for output specific settings."""

# Import third-party modules
try:
    from PySide import QtCore
    from PySide import QtGui
    from PySide import QtGui as QtWidgets
except ImportError:
    from PySide2 import QtCore
    from PySide2 import QtGui
    from PySide2 import QtWidgets

# Import local modules
from smartMedia.colorspace import colorspaces
from smartMedia.constants import PATTERN_FLOATS
from smartMedia.mvc import helper


class PageOutput(QtWidgets.QWizardPage):
    """Ask for output specific settings."""

    # All popular video codecs to offer. ATTENTION: Foundry deprecated all
    # mov32 codecs with Nuke-12 and only a simplified list of mov64 codecs are
    # available which are listed here. That means consequently that these
    # codecs won't be available in Nuke-11 and older. This should however not
    # be too much of an issue because there are not too many users anymore
    # using Nuke-11 and below and in case there are one can always go manually
    # into the generated template and set up the Write node exactly like they
    # need. So they don't need to rely on the offered codec choice here.
    # For each item the first element is the label as it appears in the codec
    # dropdown, the second element is the actual underlying value to apply in
    # the Write node.
    VIDEO_CODECS = (
        ("Apple ProRes", "appr"),
        ("Avid DNxHD", "AVdn"),
        ("Avid DNxHR", "AVdh"),
        ("H264", "h264"),
        ("Animation", "rle"),
        ("MPEG-4", "mp4v"),
        ("Uncompressed", "v210"),
    )

    # File type extensions to expose in the wizard page.
    FILE_TYPES = (
        "exr",
        "jpeg",
        "png",
        "dpx",
        "mov",
        "cin",
        "hdr",
        "mxf",
        "pic",
        "sgi",
        "targa",
        "tiff",
        "xpm",
        "yuv"
    )

    def __init__(self, page_index, count_all_pages):
        """Ask for output specific settings.

        Args:
            page_index (int): The index of the current page shown.
            count_all_pages (int): The number of all pages in the wizard.

        """
        super(PageOutput, self).__init__()

        self.page_index = page_index
        self.count_all_pages = count_all_pages

        self.create_widgets()
        self.create_layouts()
        self.create_signals()

        self.register_fields()

        self.populate_codec_options()
        helper.set_tooltips(self, "wizard_page_output")

    def create_widgets(self):
        """Create the page."""
        self.label_title = QtWidgets.QLabel(
            "Output options (Page: {}/{})".format(
                self.page_index, self.count_all_pages)
            )
        self.label_title.setObjectName("page_title")

        self.label_page_description = QtWidgets.QLabel(
            "Tell me a bit about the output you want to create."
        )

        label_min_width = 70

        self.label_file_ext = QtWidgets.QLabel("File type")
        self.label_file_ext.setMinimumWidth(label_min_width)
        self.combo_file_ext = QtWidgets.QComboBox()
        helper.fix_combo_solid_bg(self.combo_file_ext)
        self.combo_file_ext.addItems(self.FILE_TYPES)
        self.input_file_ext_mapper = QtWidgets.QLineEdit()
        self.input_file_ext_mapper.setText(self.combo_file_ext.currentText())
        self.input_file_ext_mapper.setVisible(False)
        self.label_fps = QtWidgets.QLabel("fps")
        self.label_fps.setMinimumWidth(label_min_width)
        self.input_fps = QtWidgets.QLineEdit()
        helper.fix_no_outline(self.input_fps)
        self.input_fps.setValidator(
            QtGui.QRegExpValidator(QtCore.QRegExp(PATTERN_FLOATS))
        )

        self.label_codec = QtWidgets.QLabel("Codec")
        self.label_codec.setMinimumWidth(label_min_width)
        self.combo_codec = QtWidgets.QComboBox()
        helper.fix_combo_solid_bg(self.combo_codec)
        self.input_codec_mapper = QtWidgets.QLineEdit()
        self.input_codec_mapper.setVisible(False)

        self.label_colorspace = QtWidgets.QLabel("Colorspace")
        self.label_colorspace.setMinimumWidth(label_min_width)
        self.input_colorspace = QtWidgets.QLineEdit()
        helper.fix_no_outline(self.input_colorspace)
        self.set_colorspace_completer()

        self.button_hint = QtWidgets.QPushButton("Need specific output?")
        self.button_hint.setProperty("style", "link_button")

        self.label_note = QtWidgets.QLabel(
            "Note: If you need to set up any specific knob values for "
            "your output (e.g. channels, codec profiles, custom callbacks, "
            "etc.) we recommend adjusting the Write node in your "
            "generated template after having created it. Simply open the "
            "generated preset's template.nk file in Nuke and adjust the "
            "smartRender_target Write node to your needs."
        )
        self.label_note.setWordWrap(True)
        self.label_note.setProperty("style", "note")

    def set_colorspace_completer(self):
        """Set the completer for the colorspace input."""
        user_colorspaces = colorspaces.load_colorspaces()

        self.completer = QtWidgets.QCompleter(user_colorspaces, self)
        self.completer.setCaseSensitivity(QtCore.Qt.CaseInsensitive)

        self.completer.popup().setStyleSheet(
            """
            background-color: #232323;
            color: #999999;
            font-size: 14pt;
            """
        )

        self.input_colorspace.setCompleter(self.completer)

    def create_layouts(self):
        """Create layouts."""
        self.layout_main = QtWidgets.QVBoxLayout()

        self.layout_main.addWidget(self.label_title)
        self.layout_main.addWidget(self.label_page_description)

        self.layout_file_type = QtWidgets.QHBoxLayout()
        self.layout_file_type.addWidget(self.label_file_ext)
        self.layout_file_type.addWidget(self.combo_file_ext)
        self.layout_file_type.addWidget(self.input_file_ext_mapper)
        self.layout_file_type.addStretch()

        self.group_video_options = QtWidgets.QGroupBox("Video Options")
        self.group_video_options.setVisible(False)
        self.group_video_options.setProperty("style", "with_header")
        self.layout_video_options = QtWidgets.QVBoxLayout()
        self.group_video_options.setLayout(self.layout_video_options)

        self.layout_fps = QtWidgets.QHBoxLayout()
        self.layout_fps.addWidget(self.label_fps)
        self.layout_fps.addWidget(self.input_fps)
        self.layout_fps.addStretch()

        self.layout_codec = QtWidgets.QHBoxLayout()
        self.layout_codec.addWidget(self.label_codec)
        self.layout_codec.addWidget(self.combo_codec)
        self.layout_codec.addStretch()

        self.layout_video_options.addLayout(self.layout_codec)
        self.layout_video_options.addLayout(self.layout_fps)
        self.layout_video_options.addWidget(self.input_codec_mapper)

        self.layout_colorspace = QtWidgets.QHBoxLayout()
        self.layout_colorspace.addWidget(self.label_colorspace)
        self.layout_colorspace.addWidget(self.input_colorspace)
        self.layout_colorspace.addStretch()

        self.layout_main.addLayout(self.layout_file_type)
        self.layout_main.addLayout(self.layout_colorspace)
        self.layout_main.addWidget(self.group_video_options)
        self.layout_main.addStretch()
        self.layout_main.addWidget(self.label_note)

        self.setLayout(self.layout_main)

    def create_signals(self):
        """Create signals."""
        self.combo_file_ext.currentIndexChanged.connect(
            lambda: self.set_video_options_visibility()
        )
        self.combo_file_ext.currentIndexChanged.connect(
            lambda: self.update_file_ext_mapper()
        )
        self.combo_codec.currentIndexChanged.connect(
            lambda: self.update_codec_mapper()
        )

    def update_file_ext_mapper(self):
        """Update the value for the file ext mapper."""
        self.input_file_ext_mapper.setText(self.combo_file_ext.currentText())

    def update_codec_mapper(self):
        """Update the code mapper on any codec change."""
        self.input_codec_mapper.setText(
            self.combo_codec.itemData(self.combo_codec.currentIndex())
        )

    def set_video_options_visibility(self):
        """Set the visibility of all video widgets.

        This group is only relevant when the user selects 'mov' as output
        so we show it only then.

        """
        self.group_video_options.setVisible(
            self.combo_file_ext.currentText() == "mov"
        )

    def register_fields(self):
        """Register fields."""
        self.registerField("target_ext", self.input_file_ext_mapper)
        self.registerField("target_fps", self.input_fps)
        self.registerField("target_colorspace", self.input_colorspace)
        self.registerField("target_codec", self.input_codec_mapper)

    def populate_codec_options(self):
        """Populate the codec drop down menu with all code options."""
        for codec in self.VIDEO_CODECS:
            self.combo_codec.addItem(codec[0], userData=codec[1])
