"""Provide information about a reformat to apply in the preset."""

# Import built-in modules
import json

# Import third-party modules
try:
    from PySide import QtCore
    from PySide import QtGui
    from PySide import QtGui as QtWidgets
except ImportError:
    from PySide2 import QtCore
    from PySide2 import QtGui
    from PySide2 import QtWidgets

# Import local modules
from smartMedia.constants import PATTERN_ALL_NUMBERS
from smartMedia.constants import PATTERN_FLOATS
from smartMedia.mvc import helper
from smartMedia.mvc.dialogs.new_format import NewFormatDialog
from smartMedia.settings import Settings


class PageReformat(QtWidgets.QWizardPage):
    """Create the reformat page."""

    def __init__(self, page_index, count_all_pages):
        """Initialize the PageReformat instance.

        Args:
            page_index (int): The index of the current page shown.
            count_all_pages (int): The number of all pages in the wizard.

        """
        super(PageReformat, self).__init__()

        self.page_index = page_index
        self.count_all_pages = count_all_pages

        self.settings = Settings()

        self.create_widgets()
        self.create_layouts()
        self.create_signals()
        self.register_fields()

        helper.set_tooltips(self, "wizard_page_reformat")

    def create_widgets(self):
        """Create widgets."""
        self.label_title = QtWidgets.QLabel(
            "Apply reformat? (Page: {}/{})".format(
                self.page_index, self.count_all_pages)
        )
        self.label_title.setObjectName("page_title")

        self.label_page_description = QtWidgets.QLabel(
            "Do you need to apply any reformat to the output?"
        )
        self.radio_same_as_source = QtWidgets.QRadioButton("Same as source")
        self.radio_same_as_source.setChecked(True)
        self.radio_scale = QtWidgets.QRadioButton("Scale")
        self.radio_format = QtWidgets.QRadioButton("Format")
        self.radio_box = QtWidgets.QRadioButton("Box")
        self.reformat_value_mapper = QtWidgets.QLineEdit()
        self.reformat_value_mapper.setVisible(False)

        # Scale
        self.spin_scale = QtWidgets.QDoubleSpinBox()
        self.spin_scale.setRange(0, 3)
        self.spin_scale.setValue(1)
        self.spin_scale.setSingleStep(0.01)

        # Format
        self.combo_format = QtWidgets.QComboBox()
        helper.fix_combo_solid_bg(self.combo_format)
        self.populate_root_formats()
        helper.fix_combo_menu_width(self.combo_format)

        # Box
        self.label_box_width = QtWidgets.QLabel("Width")
        self.input_box_width = QtWidgets.QLineEdit()
        helper.fix_no_outline(self.input_box_width)
        self.label_box_height = QtWidgets.QLabel("Height")
        self.input_box_height = QtWidgets.QLineEdit()
        helper.fix_no_outline(self.input_box_height)
        self.input_box_height.setEnabled(False)
        self.check_box_force_shape = QtWidgets.QCheckBox("Force shape")
        self.label_box_pixel_aspect = QtWidgets.QLabel("Aspect ratio")
        self.input_box_pixel_aspect = QtWidgets.QLineEdit("1.0")
        helper.fix_no_outline(self.input_box_pixel_aspect)
        self.set_validators()

    def register_fields(self):
        """Register fields."""
        self.registerField("reformat_same_as_source", self.radio_same_as_source)
        self.registerField("reformat_scale", self.radio_scale)
        self.registerField("reformat_value", self.reformat_value_mapper)
        self.registerField("reformat_format", self.radio_format)
        self.registerField("reformat_box", self.radio_box)

    def create_layouts(self):
        """Create layouts."""
        # Scale
        self.group_scale = QtWidgets.QGroupBox()
        self.group_scale.setVisible(False)
        self.layout_scale = QtWidgets.QHBoxLayout()
        self.group_scale.setLayout(self.layout_scale)
        self.layout_scale.addWidget(self.spin_scale)
        self.layout_scale.addStretch()

        # Format
        self.group_format = QtWidgets.QGroupBox()
        self.group_format.setVisible(False)
        self.layout_format = QtWidgets.QHBoxLayout()
        self.group_format.setLayout(self.layout_format)
        self.layout_format.addWidget(self.combo_format)
        self.layout_format.addStretch()

        # Box
        self.layout_box = QtWidgets.QHBoxLayout()
        self.layout_box.addWidget(self.label_box_width)
        self.layout_box.addWidget(self.input_box_width)
        self.layout_box.addWidget(self.label_box_height)
        self.layout_box.addWidget(self.input_box_height)
        self.layout_box.addWidget(self.check_box_force_shape)
        self.layout_box.addWidget(self.label_box_pixel_aspect)
        self.layout_box.addWidget(self.input_box_pixel_aspect)

        self.group_box = QtWidgets.QGroupBox()
        self.group_box.setVisible(False)
        self.group_box.setLayout(self.layout_box)

        # Reformat widgets
        self.layout_reformat_widgets = QtWidgets.QVBoxLayout()
        self.layout_reformat_widgets.addWidget(self.group_scale)
        self.layout_reformat_widgets.addWidget(self.group_format)
        self.layout_reformat_widgets.addWidget(self.group_box)
        self.layout_reformat_widgets.addWidget(self.reformat_value_mapper)

        # Main
        self.layout_main = QtWidgets.QVBoxLayout()
        self.layout_main.addWidget(self.label_title)
        self.layout_main.addWidget(self.label_page_description)
        self.layout_main.addWidget(self.radio_same_as_source)
        self.layout_main.addWidget(self.radio_scale)
        self.layout_main.addWidget(self.radio_format)
        self.layout_main.addWidget(self.radio_box)
        self.layout_main.addLayout(self.layout_reformat_widgets)

        self.setLayout(self.layout_main)

    def create_signals(self):
        """Create signals."""
        self.combo_format.currentIndexChanged.connect(
            self.check_create_new_format
        )

        self.radio_same_as_source.clicked.connect(
            lambda: self.show_group(None)
        )

        self.radio_scale.clicked.connect(
            lambda: self.show_group(self.group_scale)
        )

        self.radio_format.clicked.connect(
            lambda: self.show_group(self.group_format)
        )
        self.radio_box.clicked.connect(
            lambda: self.show_group(self.group_box))

        self.check_box_force_shape.stateChanged.connect(
            lambda: self.set_box_height_editable())

        self.radio_same_as_source.clicked.connect(
            lambda: self.map_reformat_value(self.radio_same_as_source)
        )

        self.radio_scale.clicked.connect(
            lambda: self.map_reformat_value(self.radio_scale)
        )

        self.radio_format.clicked.connect(
            lambda : self.map_reformat_value(self.radio_format)
        )

        self.radio_box.clicked.connect(
            lambda: self.map_reformat_value(self.radio_box)
        )

        self.spin_scale.valueChanged.connect(
            lambda: self.map_reformat_value(self.radio_scale)
        )
        self.combo_format.currentIndexChanged.connect(
            lambda: self.map_reformat_value(self.radio_format)
        )

        self.input_box_width.textChanged.connect(
            lambda: self.map_reformat_value(self.radio_box)
        )

        self.input_box_height.textChanged.connect(
            lambda: self.map_reformat_value(self.radio_box)
        )

        self.check_box_force_shape.clicked.connect(
            lambda: self.map_reformat_value(self.radio_box)
        )

        self.input_box_pixel_aspect.textChanged.connect(
            lambda: self.map_reformat_value(self.radio_box)
        )

    def set_box_height_editable(self):
        """Set the editable state of the box height."""
        self.input_box_height.setEnabled(self.check_box_force_shape.isChecked())

    def map_reformat_value(self, widget):
        """Map the current reformat value to use based on chosen checkbox.

        Args:
            widget (QtWidgets.QRadioButton): The radio button that is currently
                selected by the user.

        """
        if widget == self.radio_same_as_source:
            self.reformat_value_mapper.setText("")

        elif widget == self.radio_scale:
            self.reformat_value_mapper.setText(str(self.spin_scale.value()))

        elif widget == self.radio_format:
            self.reformat_value_mapper.setText(
                self.combo_format.itemData(self.combo_format.currentIndex())
            )

        elif widget == self.radio_box:
            value = {}

            try:
                value["width"] = int(self.input_box_width.text())
            except ValueError:
                # No text entered, fall back to initial value of 0. Do the
                # same below for all other text inputs.
                value["width"] = 0

            try:
                value["height"] = int(self.input_box_height.text())
            except ValueError:
                value["height"] = 0

            value["force_shape"] = self.check_box_force_shape.isChecked()

            try:
                value["pixel_aspect"] = float(self.input_box_pixel_aspect.text())
            except ValueError:
                value["pixel_aspect"] = 1.0

            self.reformat_value_mapper.setText(
                json.dumps(value)
            )

    def show_group(self, which):
        groups = (
            self.group_scale,
            self.group_format,
            self.group_box,
        )
        for group in groups:
            group.setVisible(False)

        if which:
            which.setVisible(True)

    def populate_root_formats(self):
        self.combo_format.clear()
        formats = sorted(
            self.settings["formats"],
            key=lambda item: item[0].lower()
        )

        for format_ in formats:
            self.combo_format.addItem(format_[0], userData=format_[1])
        self.combo_format.addItem("new...")

    def check_create_new_format(self):
        if self.combo_format.currentText() == "new...":
            format_data = NewFormatDialog.show()
            if not format_data:
                return

            format_label = "{name} {width}x{height}".format(**format_data)
            format_value = \
                "{width} {height} 0 0 {width} {height} {pixel_aspect} {name}".format(
                    **format_data
                )
            self.settings["formats"].append(
                (
                    format_label,
                    format_value
                ),
            )
            self.settings.save()
            self.populate_root_formats()
            self.combo_format.setCurrentText(format_label)

    def set_validators(self):
        """Set validators for widgets."""
        # Allow all numbers for width and height widget.
        for widget in (self.input_box_width, self.input_box_height):
            widget.setValidator(
                QtGui.QRegExpValidator(QtCore.QRegExp(PATTERN_ALL_NUMBERS))
            )

        # Allow numbers and floating numbers for pixel aspect.
        self.input_box_pixel_aspect.setValidator(
            QtGui.QRegExpValidator(QtCore.QRegExp(PATTERN_FLOATS))
        )
