"""Ask for using slates and burnins."""

# Import built-in modules
import os

# Import third-party modules
try:
    from PySide import QtCore
    from PySide import QtGui
    from PySide import QtGui as QtWidgets
except ImportError:
    from PySide2 import QtCore
    from PySide2 import QtGui
    from PySide2 import QtWidgets


# Import local modules
from smartMedia import paths
from smartMedia.mvc import helper


class PageSlateBurnins(QtWidgets.QWizardPage):
    """Ask for using slates and burnins."""

    def __init__(self, page_index, count_all_pages):
        """Ask for using slates and burnins.

        Args:
            page_index (int): The index of the current page shown.
            count_all_pages (int): The number of all pages in the wizard.

        """
        super(PageSlateBurnins, self).__init__()

        self.page_index = page_index
        self.count_all_pages = count_all_pages

        self.all_nodes = paths.get_all_nodes_from_paths()

        self.create_widgets()
        self.create_layouts()
        self.create_signals()
        self.register_fields()

        self.populate_combo(self.combo_burnins, "burnins")
        self.populate_combo(self.combo_slate, "slate")

        helper.set_tooltips(self, "wizard_page_slate_burnins")

    def create_widgets(self):
        """Create the page."""
        self.label_title = QtWidgets.QLabel(
            "Slates and burnins (Page: {}/{})".format(
                self.page_index, self.count_all_pages)
        )
        self.label_title.setObjectName("page_title")

        self.label_page_description = QtWidgets.QLabel(
            "Do you want to apply slates and burnins?"
        )

        min_size = QtCore.QSize(120, 35)

        self.check_slate = QtWidgets.QCheckBox("Apply slate")
        self.check_slate.setMinimumSize(min_size)
        self.combo_slate = QtWidgets.QComboBox()
        helper.fix_combo_solid_bg(self.combo_slate)
        self.combo_slate.setVisible(False)
        self.input_path_slate = QtWidgets.QLineEdit()
        self.input_path_slate.setVisible(False)

        self.check_burnins = QtWidgets.QCheckBox("Apply burnins")
        self.check_burnins.setMinimumSize(min_size)
        self.combo_burnins = QtWidgets.QComboBox()
        helper.fix_combo_solid_bg(self.combo_burnins)
        self.combo_burnins.setVisible(False)
        self.input_path_burnins = QtWidgets.QLineEdit()
        self.input_path_burnins.setVisible(False)

    def create_layouts(self):
        """Create layouts."""
        self.layout_main = QtWidgets.QVBoxLayout()

        self.layout_slate = QtWidgets.QHBoxLayout()
        self.layout_slate.addWidget(self.check_slate)
        self.layout_slate.addWidget(self.combo_slate)
        self.layout_slate.addWidget(self.input_path_slate)

        self.layout_burnins = QtWidgets.QHBoxLayout()
        self.layout_burnins.addWidget(self.check_burnins)
        self.layout_burnins.addWidget(self.combo_burnins)
        self.layout_burnins.addWidget(self.input_path_burnins)

        self.layout_main.addWidget(self.label_title)
        self.layout_main.addWidget(self.label_page_description)
        self.layout_main.addLayout(self.layout_slate)
        self.layout_main.addLayout(self.layout_burnins)
        self.layout_main.addStretch()

        self.setLayout(self.layout_main)

    def create_signals(self):
        """Create signals."""
        self.check_burnins.stateChanged.connect(
            lambda: self.toggle_visibility(
                self.check_burnins,
                self.combo_burnins,
            )
        )
        self.combo_burnins.currentIndexChanged.connect(
            lambda: self.update_input_path("burnins")
        )

        self.check_slate.stateChanged.connect(
            lambda: self.toggle_visibility(
                self.check_slate,
                self.combo_slate,
            )
        )
        self.combo_slate.currentIndexChanged.connect(
            lambda: self.update_input_path("slate")
        )

    def update_input_path(self, which):
        """Update the input path of the node to use.

        Args:
            which (str): Name of the node to update information for.

        """
        if which == "burnins":
            self.input_path_burnins.setText(self.combo_burnins.itemData(self.combo_burnins.currentIndex()))

        elif which == "slate":
            self.input_path_slate.setText(self.combo_slate.itemData(self.combo_slate.currentIndex()))

    def toggle_visibility(self, checkbox, combo):
        """Toggle the visibility of the combo.

        Args:
            checkbox (QtWidgets.QCheckBox): The checkbox to ask for its state.
            combo (QtWidgets.QComboBox): The dropdown to update in visibility.

        """
        combo.setVisible(checkbox.isChecked())

    def register_fields(self):
        """Register fields."""
        self.registerField("use_slate", self.check_slate)
        self.registerField("slate_path", self.input_path_slate)

        self.registerField("use_burnins", self.check_burnins)
        self.registerField("burnins_path", self.input_path_burnins)

    def populate_combo(self, combo, keyword):
        """Populate the combo with all nodes.

        Args:
            combo (QtWidgets.QComboBox): The combo to populate
            keyword (str): Name of the nodes to search for.

        """
        combo.clear()

        all_nodes_paths = [
            path for path in paths.get_all_nodes_from_paths()
            if keyword in os.path.basename(path)
        ]

        for path in all_nodes_paths:
            name = os.path.splitext(os.path.basename(path))[0]
            combo.addItem(name, userData=path)
