"""Translate the preset wizard data into a preset.

Example data:
    {
        "preset_name": "delivery_fullres_dpx",
        "preset_root": "/Volumes/Secomba/cragl/Boxcryptor/Dropbox/crypto/_GLOBALS/NUKE/python/cragl/_cragl_tools/smartMedia_v0.1.0/presets",
        "preset_info": "Create a dpx sequence for final delivery with slate and burnins",
        "reformat":{
            "type": "same as source",
            "value": ""
        },
        "use_slate": true,
        "slate_path": "/Volumes/Secomba/cragl/Boxcryptor/Dropbox/crypto/_GLOBALS/NUKE/python/cragl/_cragl_tools/smartMedia_v0.1.0/nodes/smartMedia_slate_v001.nk",
        "use_burnins": true,
        "burnins_path": "/Volumes/Secomba/cragl/Boxcryptor/Dropbox/crypto/_GLOBALS/NUKE/python/cragl/_cragl_tools/smartMedia_v0.1.0/nodes/smartMedia_burnins_v001.nk",
        "source_frame_mode": "same as source",
        "source_start": "1001",
        "target_ext": "dpx",
        "target_fps": "",
        "target_colorspace": "AlexaV3LogC"
    }

"""

# Import built-in modules
import json
import logging
import os
import shutil
import subprocess
import tempfile

# Import local modules
from smartMedia import defaults
from smartMedia import paths
from smartMedia.clogging import LEVEL
from smartMedia.constants import JSON_INDENT
from smartMedia.paths import cli_sanitized


class PresetWizardTranslator(object):
    """Translate the preset wizard data into a preset."""

    def __init__(self, data):
        """Initialize the PresetWizardTranslator.

        Args:
            data (dict): Key-value pairs of data used for preset creation.

        """
        self.data = data

        self.logger = logging.getLogger(self.__class__.__name__)
        self.logger.setLevel(LEVEL)
        self.preset_root = None

    def run(self):
        """Run all translations and create the preset."""
        self._create_preset_path()
        self._create_data_json()
        if self.data["general"].get("generate_template") == "Generate template":
            self._create_template_nk()
        self._copy_blueprints()

    def _create_preset_path(self):
        """Create the path for the preset."""
        self.preset_root = os.path.join(
            self.data["paths"]["preset_root"],
            self.data["paths"]["preset_name"],
        )

        self.logger.info("Creating preset root in: %s", self.preset_root)
        try:
            os.makedirs(self.preset_root)
        except OSError:
            # Folder already exists.
            pass
        except IOError as error:
            # User permission denied for folder creation.
            self.logger.error("Cannot create preset root: %s", str(error))

    def _create_data_json(self):
        """Create the preset's data.json file."""
        path = os.path.join(self.preset_root, "data.json")
        self.logger.info("Writing data.json into: %s", path)

        with open(path, "w") as file_:
            json.dump(self.data["data"], file_, indent=JSON_INDENT)

    def _write_data_to_temp(self):
        """Write data to tempfile and return path.

        This temp file gets passed to a Nuke terminal script to create the
        template .nk file.

        Returns:
            str: Absolute path to the temp file that was created.

        """
        temp_root = tempfile.mkdtemp(suffix="com.cragl.smartMedia.preset_")
        path = os.path.join(temp_root, "data.json")
        with open(path, "w") as file_:
            json.dump(self.data, file_)
        return path

    def _create_template_nk(self):
        """Create template.nk file."""
        command = [
            cli_sanitized(defaults.get_default_nuke_version_exe()),
            "-i",
            "-t",
            cli_sanitized(paths.get_create_template_nk_path()),
            cli_sanitized(self.preset_root),
            cli_sanitized(self._write_data_to_temp()),
        ]
        command = " ".join(command)
        self.logger.info(
            "Creating preset's template.nk using command: %s",
            command
        )
        subprocess.check_call(command, shell=True)

    def _copy_blueprints(self):
        """Copy blueprint files for any injection enabled."""
        root = paths.get_blueprint_root()

        for name in ("data", "pre_render", "post_render"):

            if self.data["general"]["code_inject-{}".format(name)]:
                module_name = "{}.py".format(name)
                src = os.path.join(root, module_name)
                dst = os.path.join(self.preset_root, module_name)
                shutil.copy(src, dst)


if __name__ == "__main__":
    # Example to run directly in IDE.
    data = {
        "paths": {
            "preset_name": "delivery_fullres_dpx",
            "preset_root": "/Volumes/Secomba/cragl/Boxcryptor/Dropbox/crypto/_GLOBALS/NUKE/python/cragl/_cragl_tools/smartMedia_v0.1.0/presets",
            "slate_path": "/Volumes/Secomba/cragl/Boxcryptor/Dropbox/crypto/_GLOBALS/NUKE/python/cragl/_cragl_tools/smartMedia_v0.1.0/nodes/smartMedia_slate_v001.nk",
            "burnins_path": "/Volumes/Secomba/cragl/Boxcryptor/Dropbox/crypto/_GLOBALS/NUKE/python/cragl/_cragl_tools/smartMedia_v0.1.0/nodes/smartMedia_burnins_v001.nk",

        },
        "data": {
            "preset_info": "Create a dpx sequence for final delivery with slate and burnins",
                "reformat":{
                    "type": "same as source",
                    "value": ""
            },
            "use_slate": True,
            "use_burnins": True,
            "source_frame_mode": "same as source",
            "source_start": "1001",
            "target_ext": "dpx",
            "target_fps": "",
            "target_colorspace": "AlexaV3LogC"
        },
    }

    translator = PresetWizardTranslator(data)
    translator.run()
