"""The windows of this package."""

# Import third-party modules
try:
    from PySide import QtCore
    from PySide import QtGui as QtWidgets
except ImportError:
    from PySide2 import QtCore
    from PySide2 import QtGui
    from PySide2 import QtWidgets

# Import local modules
from smartMedia.mvc import helper
from smartMedia.mvc import settings
from smartMedia.mvc import widgets


class View(QtWidgets.QMainWindow):
    """Main window that lets us convert media."""

    close_window = QtCore.Signal()
    section_switched = QtCore.Signal(str)
    dropped = QtCore.Signal(list)
    load_job_by_root = QtCore.Signal(str)
    create_auto_tasks = QtCore.Signal(str)
    set_info = QtCore.Signal(str)
    ctrl_hold = QtCore.Signal(bool)
    alt_hold = QtCore.Signal(bool)
    key_pressed = QtCore.Signal(int)
    requested_info = QtCore.Signal(bool)

    def __init__(self):
        """Initialize the MainView instance."""
        super(View, self).__init__()

        self.icons = helper.load_icons()
        self.setup_ui()
        self.create_menubar()
        self.create_widgets()
        self.create_layouts()

    def setup_ui(self):
        """Setup UI."""
        self.setMinimumSize(QtCore.QSize(1100, 1000))
        self.setAcceptDrops(True)

        from smartMedia import settings
        if settings.Settings.load_settings()["always_on_top"]:
            self.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint)

        self.setWindowIcon(QtGui.QIcon(helper._ICONS["logo"]))

    def create_widgets(self):
        """Create widgets."""
        self.main = QtWidgets.QWidget()

        button_size = 20

        self.button_jobs = QtWidgets.QToolButton()
        self.button_jobs.setIconSize(QtCore.QSize(button_size, button_size))
        helper.set_icon(self.button_jobs, "jobs_n")

        self.button_presets = QtWidgets.QToolButton()
        self.button_presets.setIconSize(QtCore.QSize(button_size, button_size))
        helper.set_icon(self.button_presets, "presets_n")

        self.button_settings = QtWidgets.QToolButton()
        self.button_settings.setIconSize(QtCore.QSize(button_size, button_size))
        helper.set_icon(self.button_settings, "settings_n")

        self.combo_output_root = QtWidgets.QComboBox()
        helper.fix_combo_solid_bg(self.combo_output_root)
        self.combo_output_root.setProperty("style", "plain")

        self.input_search = QtWidgets.QLineEdit()
        helper.fix_no_outline(self.input_search)
        self.input_search.setVisible(False)
        self.button_search_close = QtWidgets.QToolButton()
        helper.set_icon(self.button_search_close, "close")
        self.button_search_close.setVisible(False)

        self.button_search = QtWidgets.QToolButton()
        helper.set_icon(self.button_search, "search")

        self.button_select = QtWidgets.QToolButton()
        helper.set_icon(self.button_select, "select")

        self.button_new = QtWidgets.QToolButton()
        helper.set_icon(self.button_new, "plus")

        self.slider_size = QtWidgets.QSlider()
        self.slider_size.setOrientation(QtCore.Qt.Horizontal)
        self.slider_size.setRange(0, 600)
        self.slider_size.setFixedWidth(110)

        self.empty_jobs_widget = widgets.EmptyJobsWidget()
        self.empty_jobs_widget.setVisible(False)

        self.section_jobs_widget = QtWidgets.QWidget()
        self.section_jobs = QtWidgets.QScrollArea()
        self.section_jobs.setVerticalScrollBarPolicy(
            QtCore.Qt.ScrollBarAsNeeded)
        self.section_jobs.setHorizontalScrollBarPolicy(
            QtCore.Qt.ScrollBarAlwaysOff)
        self.section_jobs.setWidgetResizable(True)
        self.section_jobs.setWidget(self.section_jobs_widget)

        self.section_presets_widget = QtWidgets.QWidget()
        self.section_presets = QtWidgets.QScrollArea()
        self.section_presets.setVerticalScrollBarPolicy(
            QtCore.Qt.ScrollBarAsNeeded)
        self.section_presets.setHorizontalScrollBarPolicy(
            QtCore.Qt.ScrollBarAlwaysOff)
        self.section_presets.setWidgetResizable(True)
        self.section_presets.setWidget(self.section_presets_widget)

        self.section_settings = QtWidgets.QWidget()
        self.section_settings.setVisible(False)

        self.settings_widget = settings.SettingsWidget()

        self.statusbar = widgets.StatusBar()

    def create_layouts(self):
        """Create layouts."""
        self.layout_main = QtWidgets.QVBoxLayout()
        self.layout_main.setContentsMargins(10,10,10,10)

        self.layout_top = QtWidgets.QHBoxLayout()
        self.group_top = QtWidgets.QGroupBox()
        self.group_top.setFixedHeight(60)
        self.group_top.setLayout(self.layout_top)
        self.layout_toolbar = QtWidgets.QHBoxLayout()

        self.layout_top.addLayout(self.layout_toolbar)

        self.layout_top_right = QtWidgets.QHBoxLayout()
        self.layout_top_right.setContentsMargins(0,0,0,0)
        self.group_top_right = QtWidgets.QGroupBox()
        self.group_top_right.setProperty("style", "plain")
        self.group_top_right.setLayout(self.layout_top_right)

        self.layout_toolbar.addWidget(self.button_jobs)
        self.layout_toolbar.addWidget(self.button_presets)
        self.layout_toolbar.addWidget(self.button_settings)
        self.layout_toolbar.addStretch()
        self.layout_toolbar.addWidget(self.group_top_right)

        self.layout_top_right.addStretch()
        self.layout_top_right.addWidget(self.combo_output_root)
        self.layout_top_right.addWidget(self.input_search)
        self.layout_top_right.addWidget(self.button_search)
        self.layout_top_right.addWidget(self.button_search_close)
        self.layout_top_right.addWidget(self.button_select)
        self.layout_top_right.addWidget(self.button_new)
        self.layout_top_right.addWidget(self.slider_size)

        self.layout_jobs = QtWidgets.QVBoxLayout()
        self.layout_jobs.setContentsMargins(0,0,0,0)
        self.section_jobs_widget.setLayout(self.layout_jobs)
        self.layout_jobs.setAlignment(QtCore.Qt.AlignTop)
        self.layout_jobs.addWidget(self.empty_jobs_widget)

        self.layout_sections = QtWidgets.QVBoxLayout()
        self.layout_sections.addWidget(self.section_jobs)
        self.layout_sections.addWidget(self.section_presets)
        self.layout_sections.addWidget(self.section_settings)

        self.layout_presets = QtWidgets.QVBoxLayout()
        self.layout_presets.setAlignment(QtCore.Qt.AlignTop)
        self.section_presets_widget.setLayout(self.layout_presets)

        self.layout_settings = QtWidgets.QVBoxLayout()
        self.layout_settings.addWidget(self.settings_widget)
        self.section_settings.setLayout(self.layout_settings)

        self.layout_main.addWidget(self.group_top)
        self.layout_main.addLayout(self.layout_sections, 100)
        self.layout_main.addWidget(self.statusbar)

        self.main.setLayout(self.layout_main)

        self.setCentralWidget(self.main)
        helper.set_style_sheet(self.main)

    def create_menubar(self):
        """Create menubar."""
        icons = helper.load_icons()

        self.menu_main = self.menuBar()
        self.menu_main.setNativeMenuBar(False)

        self.menu_main.setStyleSheet(helper.MENUBAR_STYLES)

        # File Menu section
        self.menu_file = self.menu_main.addMenu("File")
        helper.darken_context_menu(self.menu_file)

        self.action_open = QtWidgets.QAction(
            QtGui.QIcon(self.icons["folder"]), "Open", self
        )

        self.action_open.setShortcut("Ctrl+O")

        self.menu_recent_files = QtWidgets.QMenu("Open Recent")
        helper.darken_context_menu(self.menu_recent_files)

        self.action_new = QtWidgets.QAction(
            QtGui.QIcon(self.icons["document"]), "New", self
        )
        self.action_new.setShortcut("Ctrl+N")

        self.action_save = QtWidgets.QAction(
            QtGui.QIcon(self.icons["save"]), "Save", self
        )
        self.action_save.setShortcut("Ctrl+S")

        self.action_save_as = QtWidgets.QAction(
            QtGui.QIcon(self.icons["save"]), "Save as...", self
        )
        self.action_save_as.setShortcut("Ctrl+Shift+S")

        self.action_exit = QtWidgets.QAction(
            QtGui.QIcon(self.icons["exit"]), "Exit", self
        )
        self.action_exit.setShortcut("Ctrl+Q")

        self.menu_file.addAction(self.action_new)
        self.menu_file.addAction(self.action_open)
        self.menu_file.addMenu(self.menu_recent_files)
        helper.set_icon(self.menu_recent_files, "time")

        self.menu_file.addSeparator()

        self.menu_file.addAction(self.action_save)
        self.menu_file.addAction(self.action_save_as)

        self.menu_file.addSeparator()

        self.action_export_selected_tasks = QtWidgets.QAction(
            "Export selected tasks...", self)
        helper.set_icon(self.action_export_selected_tasks, "export")
        self.action_import_tasks = QtWidgets.QAction("Import tasks...", self)
        helper.set_icon(self.action_import_tasks, "import")

        self.menu_file.addAction(self.action_export_selected_tasks)
        self.menu_file.addAction(self.action_import_tasks)

        self.menu_file.addSeparator()

        self.menu_file.addAction(self.action_exit)

        # View menu section
        self.menu_view = self.menu_main.addMenu("View")
        helper.darken_context_menu(self.menu_view)

        self.menu_section = QtWidgets.QMenu("Section")
        helper.darken_context_menu(self.menu_section)
        self.menu_section.setIcon(QtGui.QIcon(self.icons["eye"]))

        self.action_section_jobs = QtWidgets.QAction(
            QtGui.QIcon(self.icons["jobs_n"]), "Jobs", self)

        self.action_section_jobs.setShortcut("Ctrl+1")
        self.action_section_presets = QtWidgets.QAction(
            QtGui.QIcon(self.icons["presets_n"]), "Presets", self
        )
        self.action_section_presets.setShortcut("Ctrl+2")
        self.action_section_settings = QtWidgets.QAction(
            QtGui.QIcon(self.icons["settings_n"]), "Settings", self
        )
        self.action_section_settings.setShortcut("Ctrl+3")

        self.menu_section.addAction(self.action_section_jobs)
        self.menu_section.addAction(self.action_section_presets)
        self.menu_section.addAction(self.action_section_settings)

        self.action_view_expand_all_jobs = QtWidgets.QAction(
            QtGui.QIcon(icons["expand"]), "Expand all jobs", self
        )

        self.action_view_contract_all_jobs = QtWidgets.QAction(
            QtGui.QIcon(icons["contract"]), "Contract all jobs", self
        )

        self.menu_view.addMenu(self.menu_section)
        self.menu_view.addAction(self.action_view_expand_all_jobs)
        self.menu_view.addAction(self.action_view_contract_all_jobs)

        # Selection menu section
        self.menu_selection = self.menu_main.addMenu("Selection")
        helper.darken_context_menu(self.menu_selection)

        self.action_toggle_selection_mode = QtWidgets.QAction("Show/hide select", self)
        helper.set_icon(self.action_toggle_selection_mode, "select")
        self.action_select_all = QtWidgets.QAction("Select all", self)
        helper.set_icon(self.action_select_all, "select_all")
        self.action_deselect_all = QtWidgets.QAction("Deselect all", self)
        helper.set_icon(self.action_deselect_all, "deselect")
        self.action_invert_selection = QtWidgets.QAction("Invert selection", self)
        helper.set_icon(self.action_invert_selection, "invert_selection")
        self.action_render_selected = QtWidgets.QAction("Render selected", self)
        helper.set_icon(self.action_render_selected, "play_many")
        self.action_setup_selected = QtWidgets.QAction("Setup selected", self)
        helper.set_icon(self.action_setup_selected, "setup")
        self.action_reload_preset_selected = QtWidgets.QAction("Reload preset for selected", self)
        helper.set_icon(self.action_reload_preset_selected, "reload")
        self.action_reveal_output = QtWidgets.QAction("Reveal outputs", self)
        helper.set_icon(self.action_reveal_output, "reveal")
        self.action_reveal_task = QtWidgets.QAction("Reveal selected tasks", self)
        helper.set_icon(self.action_reveal_task, "reveal")
        self.action_lock_tasks = QtWidgets.QAction("Lock selected", self)
        helper.set_icon(self.action_lock_tasks, "locked")
        self.action_unlock_tasks = QtWidgets.QAction("Unlock selected", self)
        helper.set_icon(self.action_unlock_tasks, "unlocked")
        self.action_delete_tasks = QtWidgets.QAction("Delete selected", self)
        helper.set_icon(self.action_delete_tasks, "delete")

        self.menu_selection.addAction(self.action_toggle_selection_mode)
        self.menu_selection.addSeparator()
        self.menu_selection.addAction(self.action_select_all)
        self.menu_selection.addAction(self.action_deselect_all)
        self.menu_selection.addAction(self.action_invert_selection)
        self.menu_selection.addSeparator()
        self.menu_selection.addAction(self.action_render_selected)
        self.menu_selection.addAction(self.action_setup_selected)
        self.menu_selection.addAction(self.action_reload_preset_selected)
        self.menu_selection.addAction(self.action_reveal_output)
        self.menu_selection.addAction(self.action_reveal_task)
        self.menu_selection.addAction(self.action_lock_tasks)
        self.menu_selection.addAction(self.action_unlock_tasks)
        self.menu_selection.addAction(self.action_delete_tasks)

        # Task menu section
        self.menu_task = self.menu_main.addMenu("Task")
        helper.darken_context_menu(self.menu_task)

        self.action_add_task_to_all_sources = QtWidgets.QAction(
            "New preset for all sources...", self)

        self.menu_task.addAction(self.action_add_task_to_all_sources)

        # Help menu section
        self.menu_help = self.menu_main.addMenu("Help")
        helper.darken_context_menu(self.menu_help)

        self.action_getting_started = QtWidgets.QAction("Getting Started", self)
        self.action_documentation = QtWidgets.QAction("Documentation", self)
        self.action_changelog = QtWidgets.QAction("Changelog", self)
        self.action_support = QtWidgets.QAction("Support", self)
        self.action_about = QtWidgets.QAction("About", self)

        self.menu_help.addAction(self.action_getting_started)
        self.menu_help.addAction(self.action_documentation)
        self.menu_help.addAction(self.action_changelog)
        self.menu_help.addAction(self.action_support)
        self.menu_help.addAction(self.action_about)

    def keyPressEvent(self, event, *args):
        """Overwrite keyPressEvent to trigger various events."""
        key = event.key()
        modifiers = event.modifiers()
        is_ctrl_pressed = modifiers & QtCore.Qt.ControlModifier
        is_alt_pressed = modifiers & QtCore.Qt.AltModifier

        self.key_pressed.emit(key)

        if is_ctrl_pressed:
            if event.key() == QtCore.Qt.Key_1:
                self.section_switched.emit("jobs")
            if event.key() == QtCore.Qt.Key_2:
                self.section_switched.emit("presets")
            if event.key() == QtCore.Qt.Key_3:
                self.section_switched.emit("settings")

            self.ctrl_hold.emit(True)

        elif is_alt_pressed:
            self.alt_hold.emit(True)

            if key == QtCore.Qt.Key_I:
                self.requested_info.emit(True)

        elif key == QtCore.Qt.Key_Escape:
            self.close_window.emit()
            return True

        super(View, self).keyPressEvent(event)

    def keyReleaseEvent(self, event, *args):
        """Overwrite KeyReleaseEvent to trigger various events."""
        modifiers = event.modifiers()
        is_alt_pressed = modifiers & QtCore.Qt.AltModifier

        if event.key() == QtCore.Qt.Key_Control:
            self.ctrl_hold.emit(False)

        elif event.key() == QtCore.Qt.Key_Alt:
            self.alt_hold.emit(False)

        elif is_alt_pressed:
            if event.key() == QtCore.Qt.Key_I:
                self.requested_info.emit(False)
        
        super(View, self).keyReleaseEvent(event)

    def closeEvent(self, event):
        """Override close event to check for saving unsaved changes."""
        super(View, self).closeEvent(event)
        self.close_window.emit()

    def dragEnterEvent(self, event):
        """Override dragEnterEvent to drag element into table.

        Args:
            event (QtCore.Event): The event that got triggered.

        """
        if event.mimeData().hasUrls:
            event.accept()
        else:
            event.ignore()

    def dragMoveEvent(self, event):
        """Override dragMoveEvent to drag element into table.

        Args:
            event (QtCore.Event): The event that got triggered.

        """
        if event.mimeData().hasUrls:
            event.accept()
        else:
            event.ignore()

    def dropEvent(self, event):
        """Override dropEvent to drag element into table.

        Args:
            event (QtCore.Event): The event that got triggered.

        """
        if event.mimeData().hasUrls:
            event.setDropAction(QtCore.Qt.CopyAction)
            event.accept()

            urls = [
                url.toLocalFile()
                for url in event.mimeData().urls()
            ]
            event.mimeData().setText("")
            self.dropped.emit(urls)

        else:
            event.ignore()


class TaskDetailsVisibilityView(QtWidgets.QDialog):
    """Dialog to adjust the visibility of details for all tasks."""

    def __init__(self):
        """Initialize the TaskDetailsVisibilityView instance."""
        super(TaskDetailsVisibilityView, self).__init__()

        self.setup()
        self.create_widgets()
        self.create_layouts()

    def setup(self):
        """Setup UI."""
        self.setWindowTitle("Change details layout")
        self.setMinimumSize(QtCore.QSize(500, 700))
        self.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint)

    def create_widgets(self):
        """Create widgets."""
        self.table_details = widgets.SimpleTable()

        self.button_close = QtWidgets.QPushButton("Close")

    def create_layouts(self):
        """Create layouts."""
        self.layout_main = QtWidgets.QVBoxLayout()

        self.layout_main.addWidget(self.table_details)
        self.layout_main.addWidget(self.button_close)

        self.setLayout(self.layout_main)
        helper.set_style_sheet(self)


class JobInfoView(QtWidgets.QWidget):
    """Window to show all information for a job."""

    def __init__(self):
        """Initialize the JobInfoView instance."""
        super(JobInfoView, self).__init__()

        self.setup()
        self.create_widgets()
        self.create_layouts()

    def setup(self):
        """Set up UI."""
        self.setWindowFlags(
            QtCore.Qt.WindowStaysOnTopHint | QtCore.Qt.FramelessWindowHint
        )
        self.setMinimumSize(QtCore.QSize(800, 800))

    def create_widgets(self):
        """Create widgets."""
        self.label_title = QtWidgets.QLabel()
        self.label_title.setProperty("style", "title_info")
        self.button_reveal = QtWidgets.QToolButton()
        helper.set_icon(self.button_reveal, "reveal")
        self.tree = QtWidgets.QTreeWidget()
        self.header = QtWidgets.QTreeWidgetItem(["Property", "Value"])
        self.tree.setHeaderItem(self.header)
        self.tree.setColumnWidth(0, 250)
        self.tree.setAlternatingRowColors(True)

    def create_layouts(self):
        """Create layouts."""
        self.layout_main = QtWidgets.QVBoxLayout()
        self.layout_top = QtWidgets.QHBoxLayout()

        self.layout_top.addWidget(self.label_title)
        self.layout_top.addWidget(self.button_reveal)

        self.layout_main.addLayout(self.layout_top)
        self.layout_main.addWidget(self.tree)

        helper.set_style_sheet(self)
        self.setLayout(self.layout_main)

    def keyReleaseEvent(self, event):
        super(JobInfoView, self).keyReleaseEvent(event)

        if event.key() in (QtCore.Qt.Key_I, QtCore.Qt.Key_Escape):
            self.close()


class OutputRootSelectorView(QtWidgets.QWidget):
    """Window to choose absolute and relative output paths."""

    def __init__(self):
        """Initialize the OutputRootSelector instance."""
        super(OutputRootSelectorView, self).__init__()

        self.setup()
        self.create_widgets()
        self.create_layouts()

    def setup(self):
        """Setup UI."""
        self.setWindowTitle("Choose new output root directory")
        self.setMinimumWidth(700)
        self.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint)

    def create_widgets(self):
        """Create widgets."""
        self.radio_absolute = QtWidgets.QRadioButton(" Absolute")
        self.radio_relative = QtWidgets.QRadioButton(" Relative")

        self.input_path = QtWidgets.QLineEdit()
        helper.fix_no_outline(self.input_path)
        self.button_browse = QtWidgets.QToolButton()
        helper.set_icon(self.button_browse, "folder")

        self.button_create = QtWidgets.QPushButton("Create")
        self.button_create.setProperty("style", "action_button")
        self.button_cancel = QtWidgets.QPushButton("Cancel")

    def create_layouts(self):
        """Create layouts."""
        self.layout_main = QtWidgets.QVBoxLayout()
        self.layout_radio = QtWidgets.QHBoxLayout()
        self.layout_path = QtWidgets.QHBoxLayout()
        self.layout_button = QtWidgets.QHBoxLayout()

        self.layout_radio.addWidget(self.radio_absolute)
        self.layout_radio.addWidget(self.radio_relative)

        self.layout_path.addWidget(self.input_path)
        self.layout_path.addWidget(self.button_browse)

        self.layout_button.addWidget(self.button_create)
        self.layout_button.addWidget(self.button_cancel)

        self.layout_main.addLayout(self.layout_radio)
        self.layout_main.addLayout(self.layout_path)
        self.layout_main.addLayout(self.layout_button)

        self.setLayout(self.layout_main)
        helper.set_style_sheet(self)

    def keyPressEvent(self, event):
        """Override keyPressEvent to trigger custom actions."""
        super(OutputRootSelectorView, self).keyPressEvent(event)

        if event.key() == QtCore.Qt.Key_Escape:
            self.close()


class AddPresetForSources(QtWidgets.QDialog):
    """Window to assign a preset to all sources."""

    def __init__(self):
        """Initialize the AddPresetForSources instance."""
        super(AddPresetForSources, self).__init__()

        self.setup_ui()
        self.create_widgets()
        self.create_layouts()

    def setup_ui(self):
        """Setup UI."""
        self.setWindowTitle("Assign preset for all sources")
        self.setModal(True)
        self.setMinimumWidth(400)

    def create_widgets(self):
        """Create widgets."""
        self.combo_preset = QtWidgets.QComboBox()
        helper.fix_combo_solid_bg(self.combo_preset)
        self.button_add = QtWidgets.QPushButton("Add")
        self.button_add.setProperty("style", "action_button")
        self.button_cancel = QtWidgets.QPushButton("Cancel")

    def create_layouts(self):
        """Create layouts."""
        self.layout_main = QtWidgets.QVBoxLayout()
        self.layout_button = QtWidgets.QHBoxLayout()

        self.layout_button.addWidget(self.button_add)
        self.layout_button.addWidget(self.button_cancel)

        self.layout_main.addWidget(self.combo_preset)
        self.layout_main.addLayout(self.layout_button)

        self.setLayout(self.layout_main)
        helper.set_style_sheet(self)
