"""Render job data, i.e. a source's metadata and a thumbnail."""

# Import built-in modules
import json
import logging
import os
import sys

# Import third-party modules
import nuke

# Import local modules
# In order to call this module from external, we need to add smartMedia's
# root directory to the sys path so that we can import the smartMedia python
# package.
sys.path.append(os.path.join(os.path.dirname(__file__), "..", ".."))
from smartMedia.clogging import LEVEL
from smartMedia import nuke_funcs


def create_template(preset_root, preset_data):
    """Generate a preset's template.nk.

    This generates the following node tree:

    source
      |
    reformat
      |
    slate
      |
    burnins
      |
    write

    Particular knob setups take place in smartMedia's translator before
    rendering a task. Here we create only the required minimal node connection.

    Args:
        preset_root (str): Absolute path of the preset to create a template.nk
            for.
        preset_data (dict): Mapping containing information about the
            information to use in the template.

    """
    logger = logging.getLogger("create template.nk")
    logging.basicConfig()
    logger.setLevel(LEVEL)

    # The smartMedia nodes require a globals `data` in various places.
    # While building the template we initialize it here to an empty dict so
    # that they don't raise any errors.
    globals()["data"] = {}

    template_path = os.path.join(preset_root, "template.nk")

    logger.debug("Creating template in: %s", template_path)

    # Create source Read.
    read = nuke.createNode("Read")
    read.setName("smartMedia_source")

    # Create reformat.
    reformat = nuke_funcs.insert_node("smartMedia_reformat_v001")
    reformat.setXYpos(int(read.xpos()), int(read.ypos() + 200))
    reformat.setInput(0, read)

    latest_node = reformat

    # Create slate.
    if preset_data["data"].get("use_slate"):
        slate_path = preset_data["data"].get("slate_path")
        if slate_path and os.path.isfile(slate_path):
            slate_node = nuke_funcs.insert_node(slate_path)
        else:
            slate_node = nuke_funcs.insert_node("smartMedia_slate_v001")

        slate_node.setInput(0, reformat)
        slate_node.setXYpos(
            int(reformat.xpos()),
            int(reformat.ypos() + 200)
        )

        latest_node = slate_node

    # Create burnins.
    if preset_data["data"].get("use_burnins"):
        burnins_path = preset_data["data"].get("burnins_path")
        if burnins_path and os.path.isfile(burnins_path):
            burnins_node = nuke_funcs.insert_node(burnins_path)
        else:
            burnins_node = nuke_funcs.insert_node("smartMedia_burnins_v001")

        burnins_node.setInput(0, latest_node)
        burnins_node.setXYpos(
            int(latest_node.xpos()),
            int(latest_node.ypos() + 200)
        )

        latest_node = burnins_node

    # Create Write.
    write = nuke.nodes.Write()
    write.setName("smartMedia_target")
    write.setInput(0, latest_node)
    write.setXYpos(
        int(latest_node.xpos()),
        int(latest_node.ypos() + 200),
    )

    # Save template.nk.
    nuke.scriptSave(template_path)


def main():
    """Main entry point to generate a preset's template.nk."""
    preset_root = sys.argv[1]
    preset_data_path = sys.argv[2]

    with open(preset_data_path, "r") as file_:
        preset_data = json.load(file_)

    create_template(preset_root, preset_data)


if __name__ == "__main__":
    main()
