"""Render a given file."""

# Import built-in modules
import logging
import os
from pydoc import importfile
import sys


# Import third-party modules
import nuke

# Import local modules
# In order to call this module from external, we need to add smartMedia's
# root directory to the sys path so that we can import the smartMedia python
# package.
sys.path.append(os.path.join(os.path.dirname(__file__), "..", ".."))
from smartMedia import callbacks
from smartMedia.job import Job
from smartMedia.clogging import LEVEL
from smartMedia.constants import PY2
from smartMedia.translator import Translator


def render(root, render):
    """Setup the working file and render it.

    Args:
        root (str): Absolute path of job root to process.
        render (bool, optional): If False set up the job only without
            actually rendering it. This is useful for debugging without
            the need to constantly re-render frames when all what is needed
            is to be able to view the render.nk file that was set up.

    """
    logger = logging.getLogger("Render")
    logging.basicConfig()
    logger.setLevel(LEVEL)

    job = Job(root)

    logger.info("Opening working file: %s", job.path_job_template)
    nuke.scriptOpen(job.path_job_template)

    execute_custom_callbacks("pre_render", job, logger)

    # Apply translator on all possible job data values.
    translator = Translator(job.data)

    logger.info("--- Starting translating job data to actions ---")

    # Due to the fact that Nuke's Write node is build in the way of updating
    # the existence of knobs based on the entered file_type, setting the
    # file_type must get priority over all other translations, otherwise we
    # might miss setting some important knobs.
    translator.target_path(job.data["target_path"])
    translator.target_ext(job.data["target_ext"])

    # Extend task rendering by one frame at the beginning in case 'use_slate'
    # is set. This is a special case that we need to handle here.
    if job.data.get("use_slate"):
        job.data["target_start"] -= 1
        logger.info(
            "Updating frame start to %s to render slate",
            job.data["target_start"]
        )

    for key, value in job.data.items():

        # Skip all keys that were already set previously.
        if key in ("target_path", "target_ext"):
            continue

        try:
            func = getattr(translator, key)
        except AttributeError:
            # No translate function for the current key. This is fine as
            # not all keys have a translate function. There might be other
            # key-value pairs that are referenced in the render.nk file,
            # e.g. slate and burnin values, status, etc.
            continue

        logger.info("Executing '%s'", key)

        # Nuke handles strings and not unicode. Convert any unicode value
        # to string type to not risk any non valid type here.
        if PY2:
            if isinstance(value, unicode):
                value = value.encode("utf-8")

        try:
            func(value)
        except Exception as error:
            logger.exception(
                "Error executing setting for '%s' (Function %s | value: %s): %s: ",
                key,
                func,
                value,
                str(error),
                exc_info=sys.exc_info()
            )
            raise

    logger.info("--- Finished translating job data to actions ---")

    # Save the updated file with a new name to disk. This is useful for
    # debugging purposes to see what would get rendered. This file could
    # also be rendered somewhere else.
    path = os.path.join(root, "render.nk")
    logger.info("Saving to: %s", path)
    nuke.scriptSave(path)

    if not render:
        return

    # Add after frame render callbacks to keep track of render progress.
    nuke.addAfterFrameRender(
        lambda: write_progress(job)
    )

    # Render all Write nodes.
    for write in nuke.allNodes("Write"):

        knob = write.knob("create_directories")
        if knob:
            knob.setValue(True)

        logger.info(
            "Rendering '%s' (%s-%s)",
            write.name(),
            job.data["target_start"],
            job.data["target_end"]
        )

        nuke.execute(
            write,
            job.data["target_start"],
            job.data["target_end"],
            1,
        )

    execute_custom_callbacks("post_render", job, logger)


def write_progress(job):
    """Write the render progress in percentage for the current frame.

    Write into a text file inside the task root. The GUI will then pick up
    the file and read the content from it so that it updates the percentage
    label in the UI.

    Args:
        job (smartMedia.job.Job): The job instance that holds data about
            frame range for progress calculation and the path to write to.

    """
    duration = job.data["target_end"] - job.data["target_start"]
    progress = (100.0 / duration) * (nuke.frame() - job.data["target_start"])

    with open(job.path_render_progress, "w") as file_:
        file_.write(str(int(progress)))


def execute_custom_callbacks(name, job, logger):
    """Execute custom callback python module.

    Args:
        name (str): Name of the python module that resides in the job's root
            directory to execute.
        job (smartMedia.job.Job): The job to execute.
        logger (logging.RootLogger): The logger to use for logging to the
            console.

    """
    module_path = os.path.join(job.root, "{}.py".format(name))
    if not os.path.isfile(module_path):
        logger.debug("No custom %s.py module in used preset.", name)
        return

    module = importfile(module_path)

    try:
        logger.info("Executing {}.py...".format(name))
        module.main(job.data, logger)
    # Explicitly catch anything that can go wrong here and log the error
    # upstream because this framework has no chance in controlling external
    # code.
    except Exception as error:
        raise RuntimeError(
            "Error executing {}:\n{}".format(
                module_path,
                str(error)
            )
        )


def main():
    """Main entry point to start the rendering."""
    job_root = sys.argv[1]
    setup_only = True if sys.argv[2] == "True" else False

    callbacks.add_callbacks(globals())
    render(job_root, setup_only)


if __name__ == "__main__":
    main()
