"""Actions that are run from context menus."""

# Import built-in modules
import os
import shutil
import webbrowser

# Import local modules
from smartMedia import paths
from smartMedia.mvc.dialogs import confirm
from smartMedia.mvc.dialogs import message
from smartMedia.mvc.dialogs import path_browser

# Main and controller view to keep as global variable to avoid garbage
# collection.
DETAILS_VISIBILITY_VIEW = None
DETAILS_VISIBILITY_CONTROLLER = None


def run_task(task_controller, setup_only=False):
    """Render the task.

    Args:
        task_controller (smartMedia.mvc.controller.TaskWidgetController):
            The task widget controller that contains all needed information
            about the task.
        setup_only (bool, optional): If True set up the task only but don't
            process any rendering of media. Defaults to False so that we
            render media by default.

    """
    task_controller.run(setup_only=setup_only)


def reveal_task_root(task_controller):
    """Reveal the selected task's job root directory.

    Args:
        task_controller (smartMedia.mvc.controller.TaskWidgetController):
            The task widget controller that contains all needed information
            about the task.

    """
    paths.reveal(task_controller.task_root)


def reveal_task_data_json(task_controller):
    """Reveal the selected task's job data.json file.

    Args:
        task_controller (smartMedia.mvc.controller.TaskWidgetController):
            The task widget controller that contains all needed information
            about the task.

    """
    path = os.path.join(task_controller.task_root, "data.json")
    paths.reveal(path)


def reveal_task_output(task_controller):
    """Reveal the selected task's render output.

    Args:
        task_controller (smartMedia.mvc.controller.TaskWidgetController):
            The task widget controller that contains all needed information
            about the task.

    """
    paths.reveal(os.path.dirname(task_controller.preset.data["target_path"]))


def delete_task(task_controller, bypass_warning=False):
    """Delete the selected task.

    Args:
        task_controller (smartMedia.mvc.controller.TaskWidgetController):
            The task widget controller that contains all needed information
            about the task and that can also be used to communicate back
            to the main window.
        bypass_warning (bool, optional): If True then bypass the warning to
            confirm the deletion. Otherwise delete the task right away without
            asking for confirmation. Defaults to False so that we ask the user
            for confirmation before deleting a task. Bypassing this is however
            useful when batch deleting multiple tasks that that a user does
            not get asked with each and every task.

    """
    if not bypass_warning:
        title = "Confirm deleting task"
        message = (
            "Do you want to delete this task? This will delete all task files "
            "that produced the rendering. But this will not delete the rendering."
        )
        ask, _ = confirm.Confirm.ask(
            title, message, accept_label="DELETE", accept_color="red"
        )
        if not ask:
            return

    task_controller.view.deleteLater()
    task_controller.view.hide()
    del(task_controller.view)

    shutil.rmtree(task_controller.task_root)


def reveal_job_source(job_widget_ctrl):
    """Reveal the job's source path.

    Args:
        job_widget_ctrl (smartMedia.mvc.controller.JobWidgetController): The
            controller that contains information about the source to reveal.

    """
    paths.reveal(os.path.dirname(job_widget_ctrl.source_path))


def remove_job(job_widget_ctrl):
    """Remove the job from the main view.

    Args:
        job_widget_ctrl (smartMedia.mvc.controller.JobWidgetController): The
            controller that contains a link to the view to remove from the
            main view.

    """
    title = "Confirm removing Source"
    message = (
        "Do you want to remove the source from the view? Note that this "
        "will not delete the source or any of its tasks. It will just remove "
        "it from this window."
    )
    ask, _ = confirm.Confirm.ask(
        title, message, accept_label="REMOVE", accept_color="red"
    )
    if not ask:
        return

    view = job_widget_ctrl.view

    view.hide()
    view.setParent(None)
    view.deleteLater()

    job_widget_ctrl.main_ctrl.job_controller.remove(job_widget_ctrl)

    if not job_widget_ctrl.main_ctrl.job_controller:
        job_widget_ctrl.main_ctrl.view.empty_jobs_widget.setVisible(True)

    job_widget_ctrl.main_ctrl.modified()


def show_details_visibility(main_ctrl):
    """Show the details visibility edit window.

    Args:
        main_ctrl (smartMedia.mvc.controller.Controller): The main window
            controller to be able to communicate back to the main window.

    """
    from smartMedia.mvc.view import TaskDetailsVisibilityView
    from smartMedia.mvc import controller

    global DETAILS_VISIBILITY_VIEW

    if not DETAILS_VISIBILITY_VIEW:
        DETAILS_VISIBILITY_VIEW = TaskDetailsVisibilityView()

    DETAILS_VISIBILITY_VIEW.raise_()
    DETAILS_VISIBILITY_VIEW.show()

    global DETAILS_VISIBILITY_CONTROLLER
    if not DETAILS_VISIBILITY_CONTROLLER:
        DETAILS_VISIBILITY_CONTROLLER = controller.TaskDetailsVisibilityViewController(
            DETAILS_VISIBILITY_VIEW, main_ctrl
        )


def process_media_command(instance, selected_media, parent):
    """Execute the callable for the selected media and show any error.

    Args:
        instance (smartMedia.media_commands.BaseMediaCommand): The custom
            media command to trigger.
        media (:obj:`list` of :obj:`smartMedia.media.Media`): Sequence of
            selected Media instances to process.
        parent (QtWidgets.QWidget): The window to parent the notification
            dialog to in case we need to show one in case of any error.

    """
    try:
        instance.process(selected_media)
    # Catching any exception here on purpose to show any error that came up
    # while trying to execute any media command. This makes the media commands
    # more flexible as we can raise any suitable error inside the media command
    # classes and will always show any error that came up.
    except Exception as error:  # pylint: disable-broad-except
        message_ = (
            "Error processing {}:\n\n{}".format(
                instance.label(), str(error))
        )

        try:
            message.show(parent.button_search, message_)
        except:
            message.show(parent, message_)


def browse_docs_creating_custom_commands():
    """Browse documentation for creating custom commands."""
    webbrowser.open("https://www.cragl.com/smartMedia_CustomCommands")


def setup_all_selected_tasks(ctrl):
    """Set up all selected tasks of all jobs.

    Args:
        ctrl (smartMedia.mvc.controller.Controller): The main view controller
            to call for all its task controllers.

    """
    _batch_all_selected_tasks(ctrl, "run", setup_only=True)


def reload_preset_for_selected(ctrl):
    """Reload presets for all selected tasks.

    Args:
        ctrl (smartMedia.mvc.controller.Controller): The main view controller
            to call for all its task controllers.

    """
    _batch_all_selected_tasks(ctrl, "apply_preset")


def render_all_selected_tasks(ctrl):
    """Render all selected tasks of all jobs.

    Args:
        ctrl (smartMedia.mvc.controller.Controller): The main view controller
            to call for all its task controllers.

    """
    _batch_all_selected_tasks(ctrl, "run")


def reveal_all_selected_tasks_outputs(ctrl):
    """Reveal all selected tasks outputs of all jobs.

    Args:
        ctrl (smartMedia.mvc.controller.Controller): The main view controller
            to call for all its task controllers.

    """
    _batch_all_selected_tasks(ctrl, "reveal_task_output")


def reveal_all_selected_task_dirs(ctrl):
    """Reveal all selected task directories of all jobs.

    Args:
        ctrl (smartMedia.mvc.controller.Controller): The main view controller
            to call for all its task controllers.

    """
    for job_ctrl in ctrl.job_controller:
        for task_ctrl in job_ctrl.task_controller:
            if task_ctrl.selected:
                reveal_task_root(task_ctrl)


def lock_all_selected_task(ctrl, is_locked):
    """Lock/unlock all selected tasks of all jobs.

    Args:
        ctrl (smartMedia.mvc.controller.Controller): The main view controller
            to call for all its task controllers.
        is_locked (bool): If True lock all selected tasks, otherwise unlock
            them.

    """
    _batch_all_selected_tasks(ctrl, "set_task_locked", is_locked)


def delete_all_selected_tasks(ctrl):
    """Delete all unlocked tasks of all jobs.

    Args:
        ctrl (smartMedia.mvc.controller.Controller): The main view controller
            to call for all its task controllers.

    """
    title = "Confirm deleting all selected unlocked tasks"
    message = (
        "Do you want to delete ALL selected task? This will delete "
        "ALL task files of all selected tasks that produced the renderings. "
        "This will not delete the renderings themselves. Note that locked "
        "tasks will be skipped.\n\n"
        "Click DELETE if you are sure to delete all unlocked selected "
        "tasks."
    )
    ask, _ = confirm.Confirm.ask(
        title, message, accept_label="DELETE", accept_color="red"
    )
    if not ask:
        return

    for job_ctrl in ctrl.job_controller:
        for task_ctrl in job_ctrl.task_controller:
            if not task_ctrl.selected:
                continue

            if task_ctrl.preset["locked"]:
                continue

            delete_task(task_ctrl, bypass_warning=True)


def _batch_all_selected_tasks(ctrl, method_name, *args, **kwargs):
    """Batch process all selected tasks of all jobs with given method.

    Args:
        ctrl (smartMedia.mvc.controller.Controller): The main view controller
            to call for all its task controllers.
        method_name (str): Name of the method on each task controller to call.

    """
    for job_ctrl in ctrl.job_controller:
        for task_ctrl in job_ctrl.task_controller:
            if not task_ctrl.selected:
                continue

            func = getattr(task_ctrl, method_name)
            func(*args, **kwargs)


def export_selected_tasks(ctrl):
    """Export all the tasks that are currently selected in the main window.

    Args:
        ctrl (smartMedia.mvc.controller.Controller): The main controller that
            holds reference to all selected tasks.

    """
    selected_tasks = []
    for job_ctrl in ctrl.job_controller:
        for task_ctrl in job_ctrl.task_controller:
            if not task_ctrl.selected:
                continue
            selected_tasks.append(task_ctrl.task_root)

    if not selected_tasks:
        message.show(ctrl.view.button_search, "Please select some tasks to export.")
        return

    title = "Select export root"
    export_root = path_browser.show(title, only_directory=True)
    if not export_root:
        return

    _export_tasks(selected_tasks, export_root)

    message.show(ctrl.view.button_search, "Successfully exported tasks.")


def _export_tasks(task_roots, output_root):
    """Export the selected tasks.

    Args:
        task_roots (:obj:`list` of :obj:`str`): Sequence of absolute paths of
            to export.
        output_root (str): Absolute path of root directory to output tasks
            to.

    """
    for task_src in task_roots:
        # Copy task root into output root.
        job_root = os.path.dirname(task_src)
        job_name = os.path.basename(job_root)
        task_name = os.path.basename(task_src)
        task_dst = os.path.join(output_root, job_name, task_name)

        if os.path.isdir(task_dst):
            shutil.rmtree(task_dst)
        shutil.copytree(task_src, task_dst)

        # Copy job meta directory into job job export root as it is required
        # for displaying needed meta data. A task can only exist within a
        # media source.
        job_meta_src = os.path.join(job_root, ".meta")
        job_meta_dst = os.path.join(output_root, job_name, ".meta")
        if not os.path.isdir(job_meta_dst):
            shutil.copytree(job_meta_src, job_meta_dst)


def import_tasks(ctrl):
    """Import tasks into main window.

    Args:
        ctrl (smartMedia.mvc.controller.Controller): The main controller to
            add imported tasks to.

    """
    title = "Select job, root of jobs to import"
    import_root = path_browser.show(title, only_directory=True)
    if not import_root:
        return

    ctrl.drop([import_root])
