"""Callbacks that get executed in various situations.

These functions require the Nuke module and we inject data into Nuke.

"""

# Import built-in modules
import json
import os

# Import third-party modules
import nuke


def _load_job_data(globals):
    """Parse the job data and make it available in Nuke.

    This will pass all content of the data.json to the globals 'data' to make
    it available inside Nuke. This data can then be used in knob expressions,
    slates, burnins, etc.

    globals (dict): The globals dictionary that we add to.

    """
    workfile_path = nuke.root().name()
    data_path = os.path.join(
        os.path.dirname(workfile_path),
        "data.json"
    )
    if not os.path.isfile(data_path):
        globals["data"] = {}
        return

    with open(data_path, "r") as file_:
        try:
            content = json.load(file_)
        # Explicitly catch everything that could possibly go wrong here as this
        # is external data. Most possibly, the json file can simply not be
        # parsed.
        except Exception:
            content = {}

    globals["data"] = content


def _unload_job_data(globals):
    """Unload the job data on closing so that it is no longer available.

    Once we close the working file there is no need in keeping the data any
    longer in our session which is why we remove it from the globals.

    Args:
        globals (dict): The globals dictionary to remove the data from.

    """
    globals.pop("data", None)


def _load_datetime():
    """Get Python's datetime class to be used in the slates.

    Returns:
        datetime.datetime: Python's datetime class.

    """
    from datetime import datetime
    return datetime


def add_callbacks(globals):
    """Add all necessary callbacks to Nuke.

    Args:
        globals (dict): The sessions's globals to add data to.

    Notes:
        Please keep these callbacks, otherwise smartMedia will not fully
        function.

    """
    # This callback will parse the data.json file living next to a working
    # file to load (if that data.json file exists) and pass it into the globals
    # to make it available in the variable `data`. You can then call any
    # key for its value. This data can be used in any expression knobs, slates,
    # burnins, etc.
    nuke.addOnScriptLoad(_load_job_data, args=(globals,))

    # Once we close a working file we remove the global data again.
    nuke.addOnScriptClose(_unload_job_data, args=(globals,))

    # Add Python's datetime class to be used in the slates. Slates access
    # the datetime class directly which is why we need them available in
    # the globals.
    globals["datetime"] = _load_datetime()
