"""Render a job by given root directory."""

# Import built-in modules
import json
import logging
import os
import subprocess
import time

# Import local modules
from smartMedia import defaults
from smartMedia import paths
from smartMedia.clogging import LEVEL
from smartMedia.paths import cli_sanitized
from smartMedia.settings import Settings


class Job(object):
    """Represent a job to process."""

    def __init__(self, root, ctrl=None, render=True):
        """Initialize the Job instance.

        Args:
            root (str): The absolute path of the job's root directory.
            ctrl (smartMedia.mvc.controller.TaskWidgetController, optional): A
                task widget controller to talk back to when the job has
                finished so that the status can be updated.
            render (bool, optional): If False set up the job only without
                actually rendering it. This is useful for debugging without
                the need to constantly re-render frames when all what is needed
                is to be able to view the render.nk file that was set up.

        """
        self.root = root
        self.ctrl = ctrl
        self.render = "True" if render else "False"

        self.settings = Settings()
        self.data = self._parse()
        self.logger = logging.getLogger(
            "{} ({})".format(
                self.__class__.__name__,
                os.path.basename(root)
            )
        )
        self.logger.setLevel(LEVEL)

    @classmethod
    def from_root(cls, root):
        """Factory to create a job from the given root.

        job_root (str): The absolute path of the job's root directory.

        Returns:
            smartMedia.job.Job: The job instance that got created.

        """
        return cls(root)

    def run(self):
        """Process the given source with the given preset.

        Returns:
            bool: True when the task has finished successfully, otherwise
                False.

        """
        success = self._run()

        if self.ctrl:
            self.ctrl.view.finished.emit(success)

        return success

    def _run(self):
        """Process the given source with the given preset.

        Returns:
            bool: True when the task has finished successfully, otherwise
                False.

        """
        start = time.time()

        command = self._assemble_command()
        self.logger.info("Running job using command: %s ", " ".join(command))

        with open(self.path_std_out, "wb") as log_file:
            try:
                subprocess.check_call(
                    " ".join(command),
                    stdout=log_file,
                    stderr=log_file,
                    shell=True
                )
            except subprocess.CalledProcessError:
                self.logger.error(
                    "Failed rendering job :( Have a look at %s for more "
                    "information.", self.path_std_out
                )
                return False

        self.logger.info(
            "Finished job :) Render time: %ss", time.time() - start
        )

        return True

    @property
    def path_job_data(self):
        """Return absolute path of the job data file.

        Returns:
            str: Absolute path of the job data file.

        """
        return os.path.join(self.root, "data.json")

    @property
    def path_job_template(self):
        """Return absolute path of the job template file.

        Returns:
            str: Absolute path of the job template file.

        """
        return os.path.join(self.root, "template.nk")

    @property
    def path_std_out(self):
        """Return absolute path of the job's std_out log.

        Returns:
            str: Absolute path of the job's std_out log.

        """
        return os.path.join(self.root, "render.log")

    @property
    def path_render_progress(self):
        """Return absolute path of the job's render progress file.

        Returns:
            str: Absolute path of the job's render progress file.

        """
        return os.path.join(self.root, "progress")

    def _parse(self):
        """Parse the job data for the given job."""
        with open(self.path_job_data, "r") as file_:
            return json.load(file_)

    def _get_nuke_exe(self):
        """Get the absolute path of the Nuke executable to use.

        We prefer the job data's Nuke version override and if it is not set
        fall back to the default Nuke version that is defined in the settings.

        Returns:
            str: Absolute path of the Nuke executable to run the job with.

        """
        # Use job specific override.
        job_nuke_version_override = self.data.get("nuke_version")

        if job_nuke_version_override:
            for path, is_favorite in self.settings["nuke_executables"]:
                if job_nuke_version_override.lower() in path.lower():
                    return path

        # Fall back to globally defined Nuke version.
        return defaults.get_default_nuke_version_exe()

    def _assemble_command(self):
        """Assemble the command to run the job with.

        Returns:
            :obj:`list` of :obj:`str`: The command to run the job with.

        """
        cmd = [
            cli_sanitized(self._get_nuke_exe()),
            "-i",
            "-t",
            cli_sanitized(paths.get_render_task_script_path()),
            cli_sanitized(self.root),
            self.render
        ]

        return cmd
