"""Drive views."""

# Import built-in modules
from collections import OrderedDict
import functools
import json
import logging
import os
from pydoc import importfile
import shutil
import threading
import webbrowser

# Import third-party modules
try:
    from PySide import QtCore
    from PySide import QtGui as QtWidgets
except ImportError:
    from PySide2 import QtCore
    from PySide2 import QtGui
    from PySide2 import QtWidgets

# Import local modules
from smartMedia import actions
from smartMedia import constants
from smartMedia import media
from smartMedia import naming
from smartMedia import preset
from smartMedia import paths
from smartMedia import settings
from smartMedia import startup
from smartMedia import job
from smartMedia.path_resolver import PathResolver
from smartMedia.clogging import LEVEL
from smartMedia.constants import INGESTABLE_EXTENSIONS_FOOTAGE
from smartMedia.creator import JobCreator
from smartMedia.mvc import helper
from smartMedia.mvc import widgets
from smartMedia.mvc.dialogs import confirm
from smartMedia.mvc.dialogs import input
from smartMedia.mvc.dialogs import message
from smartMedia.mvc.dialogs.preset_wizard.main import get_preset_user_data
from smartMedia.mvc.dialogs.preset_wizard import preset_wizard_translator
from smartMedia.mvc.dialogs import path_browser
from smartMedia.mvc.details_widgets import DETAILS_WIDGETS
from smartMedia.mvc.view import JobInfoView
from smartMedia.mvc.view import OutputRootSelectorView
from smartMedia.mvc.view import AddPresetForSources
from smartMedia.processor import Processor

# Global views to prevent from garbage collection
INFO_VIEW = None
OUTPUT_ROOT_SELECTOR_VIEW = None
ADD_PRESET_FOR_SOURCES_VIEW = None

# Ensure all settings key-value pairs exist.
# ATTENTION: Run this only once and not in threads.
settings.Settings.check_settings()


class Controller(object):
    """Drive the main view."""

    def __init__(self, view):
        """Initialize the Controller instance.

        Args:
            view (smartMedia.mvc.view.View): The view to drive.

        """
        self.view = view

        self.logger = logging.getLogger(__name__)
        self.logger.setLevel(LEVEL)
        self.settings = settings.Settings()
        self.processor = Processor(
            controller=self,
            num_threads=self.settings["ingest_processor_threads"]
        )
        self.job_controller = []
        self.preset_controllers = []
        self.document = None
        self._modified = ""
        self.version_number = paths.get_version_number()
        self.ctrl_hold = False
        self.alt_hold = False
        self._build_presets = False
        self.current_section = ""
        self.naming = naming.Naming("")
        self.entered_job_or_task_ctrl = None
        self.show_selection_mode = False

        self.logger.setLevel(
            constants.LOGGING_LEVELS[self.settings["logging_level"]]
        )

        self.presets = preset.load_all_presets()
        self.set_widget_states()

        if self.settings["auto_load_last_workfile"] and self.settings["recent_files"]:
            self.load_jobs_from_file(self.settings["recent_files"][0])
        else:
            self.set_window_title()
            self.view.empty_jobs_widget.setVisible(True)

        self.create_signals()

        # This ensures we have no status bar at the beginning.
        self.status("")

        if self.settings["tooltips"]:
            helper.set_tooltips(self.view.settings_widget, "settings_widget")
            helper.set_tooltips(self.view, "main")

        self.set_section("jobs")

    def get_all_task_ctrl(self):
        """Get all task controllers.

        Returns:
            :obj:`list` of smartMedia.mvc.controller.TaskWidgetController:
                Sequence of all current TaskWidgetController.

        """
        task_controller = []

        for job_ctrl in self.job_controller:
            for task_ctrl in job_ctrl.task_controller:
                task_controller.append(task_ctrl)

        return task_controller

    @property
    def current_output_root_and_mode(self):
        """Get the currently active output root.

        Returns:
            str, str: The path of the currently selected output root and mode.

        """
        for item in self.settings["output_roots"]:
            if item["active"]:
                return item["path"], item["mode"]
        return "", ""

    def create_signals(self):
        """Create signals."""
        self.view.key_pressed.connect(lambda key: self.on_key_pressed(key))
        self.view.close_window.connect(lambda: self.close())
        self.view.action_new.triggered.connect(lambda: self.new_document())
        self.view.action_exit.triggered.connect(lambda: self.close())
        self.view.button_jobs.clicked.connect(lambda: self.set_section("jobs"))
        self.view.button_presets.clicked.connect(lambda: self.set_section("presets"))
        self.view.button_settings.clicked.connect(lambda: self.set_section("settings"))
        self.view.slider_size.valueChanged.connect(lambda: self.update_thumb_size())
        self.view.dropped.connect(lambda paths: self.drop(paths))
        self.view.load_job_by_root.connect(lambda job_root: self.load_job(job_root))
        self.view.create_auto_tasks.connect(lambda job_root: self.create_auto_tasks(job_root))
        self.view.set_info.connect(lambda text: self._set_status(text))
        self.view.settings_widget.button_save.clicked.connect(
            lambda: self.save_settings()
        )
        self.view.action_view_expand_all_jobs.triggered.connect(
            lambda: self.toggle_view_all(True)
        )
        self.view.action_view_contract_all_jobs.triggered.connect(
            lambda: self.toggle_view_all(False)
        )
        self.view.section_switched.connect(
            lambda section: self.set_section(section)
        )
        self.view.empty_jobs_widget.button_browse.clicked.connect(
            lambda: self.new_item()
        )
        self.view.button_new.clicked.connect(lambda: self.new_item())
        self.view.action_section_jobs.triggered.connect(lambda: self.set_section("jobs"))
        self.view.action_section_presets.triggered.connect(lambda: self.set_section("presets"))
        self.view.action_section_settings.triggered.connect(lambda: self.set_section("settings"))
        self.view.action_open.triggered.connect(lambda: self.browse_to_load())
        self.view.action_save_as.triggered.connect(lambda: self.save_jobs_to_file())
        self.view.action_save.triggered.connect(lambda: self.save_jobs_to_file(show_browser=False))
        self.view.ctrl_hold.connect(lambda is_hold: self.update_ctrl_hold(is_hold))
        self.view.alt_hold.connect(lambda is_hold: self.update_alt_hold(is_hold))
        self.view.button_search.clicked.connect(lambda: self.show_search())
        self.view.button_search_close.clicked.connect(lambda: self.close_search())
        self.view.input_search.textChanged.connect(lambda: self.search())
        self.view.settings_widget.combo_naming_preset.currentIndexChanged.connect(
            lambda: self.show_naming()
        )
        self.view.settings_widget.browse_naming_root.clicked.connect(
            lambda: self.browse_naming_root()
        )
        self.view.requested_info.connect(
            lambda: self.show_entered_job_or_task_widget_info()
        )
        self.view.combo_output_root.currentIndexChanged.connect(
            lambda: self.set_output_root()
        )
        self.view.action_about.triggered.connect(lambda: self.show_about())
        self.view.button_select.clicked.connect(lambda: self.toggle_selection_mode())
        self.view.action_toggle_selection_mode.triggered.connect(
            lambda: self.toggle_selection_mode()
        )
        self.view.action_select_all.triggered.connect(lambda: self.select_all_tasks("select"))
        self.view.action_deselect_all.triggered.connect(lambda: self.select_all_tasks("deselect"))
        self.view.action_invert_selection.triggered.connect(lambda: self.select_all_tasks("invert"))
        self.view.action_render_selected.triggered.connect(lambda: actions.render_all_selected_tasks(self))
        self.view.action_setup_selected.triggered.connect(lambda: actions.setup_all_selected_tasks(self))
        self.view.action_reload_preset_selected.triggered.connect(lambda: actions.reload_preset_for_selected(self))
        self.view.action_reveal_output.triggered.connect(lambda: actions.reveal_all_selected_tasks_outputs(self))
        self.view.action_reveal_task.triggered.connect(lambda: actions.reveal_all_selected_task_dirs(self))
        self.view.action_lock_tasks.triggered.connect(lambda: actions.lock_all_selected_task(self, True))
        self.view.action_unlock_tasks.triggered.connect(lambda: actions.lock_all_selected_task(self, False))
        self.view.action_delete_tasks.triggered.connect(lambda: actions.delete_all_selected_tasks(self))
        self.view.action_export_selected_tasks.triggered.connect(lambda: actions.export_selected_tasks(self))
        self.view.action_import_tasks.triggered.connect(lambda: actions.import_tasks(self))
        self.view.action_add_task_to_all_sources.triggered.connect(lambda: self.add_task_to_all_sources())
        self.view.action_getting_started.triggered.connect(lambda: self.browse_getting_started())
        self.view.action_documentation.triggered.connect(lambda: self.browse_documentation())
        self.view.action_support.triggered.connect(lambda: self.browse_support())
        self.view.action_changelog.triggered.connect(lambda: self.show_changelog())

    def browse_getting_started(self):
        """Browse to the doc's getting started section."""
        webbrowser.open("https://www.cragl.com/smartMediaGettingStarted")

    def browse_documentation(self):
        """Browse to the documentation."""
        webbrowser.open("https://www.cragl.com/smartMediaTut")

    def browse_support(self):
        """Browse to the online support."""
        webbrowser.open("https://www.cragl.com/support")

    def show_changelog(self):
        """Show the about window."""
        import platform
        import smartMedia

        # Development => py files
        if os.path.isfile(os.path.join(smartMedia.root(), ".dev")):
            from smartMedia import license_d as license
        # Release => cython files
        else:
            _OS = platform.system()

            if _OS == "Darwin":
                from smartMedia import license_m as license
            elif _OS == "Linux":
                from smartMedia import license_l as license
            elif _OS == "Windows":
                from smartMedia import license_w as license

        license.show_changelog()

    def on_key_pressed(self, key):
        """Execute events on key pressed for currently hovered job or task.

        Args:
            key (int): The key code of the pressed key.

        """
        ctrl = self.entered_job_or_task_ctrl

        if isinstance(ctrl, JobWidgetController):
            if self.alt_hold:
                if key == QtCore.Qt.Key_O:
                    actions.reveal_job_source(ctrl)
                elif key == QtCore.Qt.Key_D:
                    actions.remove_job(ctrl)
            if key == QtCore.Qt.Key_Plus:
                ctrl.new_default_task()

        elif isinstance(ctrl, TaskWidgetController):
            if self.ctrl_hold:
                if key == QtCore.Qt.Key_R:
                    ctrl.run()

            if self.alt_hold:
                if key == QtCore.Qt.Key_R:
                    ctrl.run(setup_only=True)
                elif key == QtCore.Qt.Key_L:
                    ctrl.set_lock_state(True)
                elif key == QtCore.Qt.Key_D:
                    actions.delete_task(ctrl)
                elif key == QtCore.Qt.Key_T:
                    actions.reveal_task_root(ctrl)
                elif key == QtCore.Qt.Key_O:
                    actions.reveal_task_output(ctrl)

    def select_all_tasks(self, mode):
        """Select, unselect or invert the selection of all tasks.

        Args:
            mode (str): The mode to apply for selection. Choose between:
                - select: Select all tasks.
                - deselect: Deselect all tasks.
                - invert: Invert selection for all tasks.

        """
        # Show selection checkboxes in case they are currently not visible.
        if not self.show_selection_mode:
            self.toggle_selection_mode()

        for job_ctrl in self.job_controller:
            for task_ctrl in job_ctrl.task_controller:
                if mode == "select":
                    task_ctrl.select_task()
                elif mode == "deselect":
                    task_ctrl.unselect_task()
                elif mode == "invert":
                    task_ctrl.toggle_selection()

    def show_about(self):
        """Show about window."""
        import platform
        import smartMedia

        _OS = platform.system()

        if os.path.isfile(os.path.join(smartMedia.root(), ".dev")):
            from smartMedia import license_d as license
        # Release => cython files
        else:
            if _OS == "Darwin":
                from smartMedia import license_m as license
            elif _OS == "Linux":
                from smartMedia import license_l as license
            elif _OS == "Windows":
                from smartMedia import license_w as license

        license.show_about_window()

    def toggle_selection_mode(self):
        """Toggle show/hide to select tasks."""
        self.show_selection_mode = not self.show_selection_mode

        for job_ctrl in self.job_controller:
            for task_ctrl in job_ctrl.task_controller:
                # New task was created after being in select mode. At that
                # stage there is no such view for the task.
                if not hasattr(task_ctrl, "view"):
                    continue

                task_ctrl.view.button_select.setVisible(self.show_selection_mode)

    def show_entered_job_or_task_widget_info(self):
        """Show info window for currently hovered job."""
        # Return early in case no entered_job_ctrl is set, yet.
        if not self.entered_job_or_task_ctrl:
            return

        # Use global to avoid window from garbage collection.
        global INFO_VIEW

        INFO_VIEW = JobInfoView()
        helper.move_widget_under_cursor(INFO_VIEW)
        JobInfoViewController(INFO_VIEW, self.entered_job_or_task_ctrl)

        INFO_VIEW.show()
        INFO_VIEW.raise_()

        helper.force_move_on_top(INFO_VIEW)
        # Set the focus to any widget other than the tree, otherwise holding
        # the 'I' key to keep the info view open will cycle throuh all tree
        # items.
        INFO_VIEW.label_title.setFocus(QtCore.Qt.MouseFocusReason)

    def browse_naming_root(self):
        """Browse to a root directory to use as naming root."""
        title = "Browse naming root"
        root = path_browser.show(title, only_directory=True)

        self.view.settings_widget.input_naming_root.setText(root)
        self.settings["naming_root"] = root
        self.settings.save()

    def show_naming(self):
        """Set the naming pattern based on selected value."""
        selected_naming_preset = \
            self.view.settings_widget.combo_naming_preset.currentText()
        if selected_naming_preset in ("", "new..."):
            content = ""
        else:
            content = self.naming.get_content(selected_naming_preset)

        if selected_naming_preset == "new...":
            self.create_new_naming_preset()


        self.view.settings_widget.text_naming_values.setText(content)

    def add_task_to_all_sources(self):
        """Show dialog to add a new task to all sources."""
        global ADD_PRESET_FOR_SOURCES_VIEW
        ADD_PRESET_FOR_SOURCES_VIEW = AddPresetForSources()

        ADD_PRESET_FOR_SOURCES_VIEW.raise_()
        ADD_PRESET_FOR_SOURCES_VIEW.show()

        AddPresetForSourcesController(ADD_PRESET_FOR_SOURCES_VIEW, self)

    def create_new_naming_preset(self):
        """Create a new naming preset by user input."""
        name = input.show(
            self.view,
            "Set name",
            "Give your naming preset a name",
            ""
        )

        if not name:
            return

        # Make sure to remove any file extension and use just the plain
        # base name.
        name = os.path.splitext(name)[0]

        path = os.path.join(
            self.settings["naming_root"],
            "{}.txt".format(name)
        )

        if os.path.isfile(path):
            message.show(
                self.view.button_search,
                "The naming preset '{}' already exists in the naming root."
                "please use a different name".format(name)
            )
            return

        with open(path, "w") as file_:
            file_.write("")

        self.view.settings_widget.combo_naming_preset.addItem(name)

        self.view.settings_widget.combo_naming_preset.setCurrentIndex(
            self.view.settings_widget.combo_naming_preset.findText(name)
        )

    def show_search(self):
        """Show the search input."""
        self.view.input_search.setVisible(True)
        self.view.button_search_close.setVisible(True)
        self.view.button_search.setVisible(False)

        self.view.input_search.setFocus()

    def close_search(self):
        """Close the search input."""
        self.view.input_search.setVisible(False)
        self.view.button_search_close.setVisible(False)
        self.view.button_search.setVisible(True)

        for ctrl in self.job_controller:
            ctrl.view.setVisible(True)

        for ctrl in self.preset_controllers:
            ctrl.view.setVisible(True)

    def search(self):
        """Filter jobs to show only relevant searched ones."""
        search = self.view.input_search.text()

        if self.current_section == "jobs":
            for ctrl in self.job_controller:
                ctrl.view.setVisible(
                    search in os.path.basename(ctrl.source_path)
                )

        elif self.current_section == "presets":
            for ctrl in self.preset_controllers:
                ctrl.view.setVisible(
                    search in ctrl.preset.name
                )

    def update_ctrl_hold(self, is_hold):
        """Update internal state of ctrl (cmd on mac) is currently hold."""
        self.ctrl_hold = is_hold

    def update_alt_hold(self, is_hold):
        """Update internal state of alt is currently hold."""
        self.alt_hold = is_hold

    def set_window_title(self):
        """Set the window title.

        Args:
            modified (bool, True): If True set modified flag in window,
                otherwise don't set it.

        """
        if self.document:
            name = os.path.basename(self.document)
        else:
            name = "Untitled"

        title = "smartMedia {} - {}{}".format(
            self.version_number,
            name,
            self._modified
        )
        self.view.setWindowTitle(title)

    def modified(self):
        """Set window modified."""
        self._modified = "*"
        self.set_window_title()

    def unmodified(self):
        """Set window unmodified."""
        self._modified = ""
        self.set_window_title()

    def browse_to_load(self):
        """Browse .media file to load."""
        self.ask_save_unsaved_changes()

        name_filters = ["media files (*.media)"]
        path = path_browser.show("Load .media file", name_filters=name_filters)

        if not path:
            return

        self.new_document()
        self.load_jobs_from_file(path)

    def new_item(self):
        """Create a new item depending on the currently selected section.

        When being in jobs section then show browser to browse to a new source
        to ingest. When being in presets mode then show wizard to create a
        new preset.

        Job section:
            Open browser to browse for new source(s).

        Presets section:
            Create new preset.

        """
        if self.current_section == "jobs":
            path = path_browser.show("browse to media", only_directory=True)
            if not path:
                return
            self.drop([path])

        elif self.current_section == "presets":
            preset_user_data = get_preset_user_data()
            if not preset_user_data:
                # User aborted the wizard. Exit early
                return

            translator = preset_wizard_translator.PresetWizardTranslator(
                preset_user_data
            )
            translator.run()

            self.presets = preset.load_all_presets()
            preset_ = preset.Preset(translator.preset_root)
            self.add_preset_widget(preset_)

            message.show(
                self.view.button_search,
                "Successfully created preset '{}'".format(
                    preset_user_data["paths"]["preset_name"]
                )
            )

            for job_ctrl in self.job_controller:
                for task_ctrl in job_ctrl.task_controller:
                    task_ctrl.view.combo_presets.addItem(
                        preset_.name,
                        userData=preset_
                    )

    def ask_save_unsaved_changes(self):
        """Ask user if they want to save unsaved changes to current file."""
        if not self._modified:
            return

        title = "Save changes?"
        message = "Do you want to save unsaved changes?"
        accept_label = "save"
        accept_color = "blue"
        reject_label = "no"
        remember_checkbox = False

        ask, _ = confirm.Confirm.ask(
            title, message, accept_label, accept_color, reject_label,
            remember_checkbox
        )

        if ask:
            self.save_jobs_to_file(show_browser=False)

    def new_document(self):
        """Remove all jobs and start with a blank main window."""
        self.ask_save_unsaved_changes()

        self.document = None
        self.unmodified()
        self.set_section("jobs")

        for ctrl in self.job_controller:
            ctrl.view.hide()
            ctrl.view.deleteLater()

        self.job_controller = []

        self.view.empty_jobs_widget.setVisible(True)

    def toggle_view_all(self, expand):
        """Expand or contract all jobs.

        Args:
            expand (bool): If True expand all jobs, otherwise contract all
                jobs.

        """
        for ctrl in self.job_controller:
            # Force the expanded state on all controller so that we can toggle
            # the afterwards.
            if expand:
                ctrl.is_expanded = False
            else:
                ctrl.is_expanded = True

            ctrl.toggle_expand_collapse()

    def populate_output_roots(self):
        """Populate all output roots."""
        self.view.combo_output_root.clear()
        self.view.combo_output_root.addItems(["", "new...", "delete"])
        self.view.combo_output_root.insertSeparator(3)

        longest_item = 0
        # Add all relative paths.
        for item in self.settings["output_roots"]:
            if item["mode"] == "rel":
                self.view.combo_output_root.addItem(item["path"])
            if len(item["path"]) > longest_item:
                longest_item = len(item["path"])
        self.view.combo_output_root.insertSeparator(
            self.view.combo_output_root.count()
        )

        # Add all absolute paths.
        for item in self.settings["output_roots"]:
            if item["mode"] == "abs":
                self.view.combo_output_root.addItem(item["path"])
                if len(item["path"]) > longest_item:
                    longest_item = len(item["path"])

        # Make active element the selected element
        for item in self.settings["output_roots"]:
            if item["active"]:
                self.view.combo_output_root.setCurrentIndex(
                    self.view.combo_output_root.findText(
                        item["path"]
                    )
                )

        self.view.combo_output_root.setMinimumContentsLength(longest_item)

    def set_output_root(self):
        """Set, create or delete an output root."""
        previous_selection, _ = self.current_output_root_and_mode
        current_selection = self.view.combo_output_root.currentText()

        if current_selection == "":
            for item in self.settings["output_roots"]:
                item["active"] = False
            self.settings.save()

        elif current_selection == "new...":
            global OUTPUT_ROOT_SELECTOR_VIEW
            OUTPUT_ROOT_SELECTOR_VIEW = OutputRootSelectorView()
            OUTPUT_ROOT_SELECTOR_VIEW.raise_()
            OUTPUT_ROOT_SELECTOR_VIEW.show()

            OutputRootSelectorViewController(OUTPUT_ROOT_SELECTOR_VIEW, self)
            
        elif current_selection == "delete":
            self.delete_current_output_root()

        else:
            # Check if there are any tasks and ask for updating their output
            # paths.
            if self.get_all_task_ctrl():

                title = "Change output root for all current tasks?"
                message = (
                    "Do you want to change the output root for all current "
                    "tasks?"
                )

                ask, _ = confirm.Confirm.ask(title, message)
                if not ask:
                    self.view.combo_output_root.blockSignals(True)
                    self.view.combo_output_root.setCurrentText(previous_selection)
                    self.view.combo_output_root.blockSignals(False)

                    return

            for item in self.settings["output_roots"]:
                item["active"] = item["path"] == current_selection

            self.update_all_task_output_paths()
            self.settings.save()

    def update_all_task_output_paths(self):
        """Update the output paths of all tasks.

        Skip all tasks that are locked.

        """
        current_output_root, mode = self.current_output_root_and_mode
        for task_ctrl in self.get_all_task_ctrl():
            if task_ctrl.preset.data.get("locked", False):
                continue

            if mode == "abs":
                path = task_ctrl.preset["target_path"]
                task_path = os.path.join(
                    current_output_root,
                    os.path.basename(path)
                )

            elif mode == "rel":
                path = task_ctrl.preset["source_path"]
                root, basename = os.path.split(path)

                # Prefer the basename from target path in case it is already
                # set up. That ensures we keep changes that have already been
                # made and pipe through the already set up basename.
                if task_ctrl.preset["target_path"]:
                    basename = os.path.basename(task_ctrl.preset["target_path"])
                task_path = os.path.join(
                    root,
                    current_output_root,
                    basename
                )
                task_path = os.path.normpath(task_path)

            task_ctrl.preset["target_path"] = task_path
            task_ctrl.preset.save()
            task_ctrl.update_target_path_details_widget()

    def delete_current_output_root(self):
        """Delete the current output root from settings."""
        for index, item in enumerate(self.settings["output_roots"]):
            if not item["active"]:
                continue

            self.settings["output_roots"].remove(item)

        self.settings.save()
        self.populate_output_roots()

    def add_preset_widget(self, preset_):
        """Add a preset as preset widget."""
        widget = widgets.PresetWidget()
        ctrl = PresetWidgetController(widget, preset_, self)
        self.preset_controllers.append(ctrl)

        self.view.layout_presets.addWidget(widget)

    def populate_presets_section(self):
        """Populate the presets section with all presets."""
        if self._build_presets:
            return

        for preset in self.presets.values():
            self.add_preset_widget(preset)

        self._build_presets = True

    def populate_settings_widget(self):
        """Populate the settings widget."""
        self.view.settings_widget.check_show_tooltips.setChecked(
            self.settings["tooltips"]
        )

        self.view.settings_widget.check_always_on_top.setChecked(
            self.settings["always_on_top"]
        )

        self.view.settings_widget.check_auto_load_last_workfile.setChecked(
            self.settings["auto_load_last_workfile"]
        )

        self.view.settings_widget.check_auto_extract_naming.setChecked(
            self.settings["auto_apply_naming_extraction"]
        )

        self.view.settings_widget.spin_number_recent_files.setValue(
            self.settings["number_recent_files"]
        )

        self.view.settings_widget.spin_ingest_threads.setValue(
            self.settings["ingest_processor_threads"]
        )

        self.view.settings_widget.combo_logging_level.setCurrentText(
            self.settings["logging_level"]
        )

        self.view.settings_widget.combo_new_jobs_behavior.setCurrentText(
            self.settings["new_jobs_order"]
        )

        self.view.settings_widget.combo_thumbnail_index.setCurrentText(
            self.settings["thumbnail_index"]
        )

        self.view.settings_widget.combo_preset_dropdown_type.setCurrentText(
            self.settings["preset_dropdown_type"]
        )

        self.view.settings_widget.input_naming_root.setText(
            self.settings["naming_root"]
        )

        # Block signals on dropdown because we trigger a signal on index
        # change and since we populate the dropdown here this would trigger
        # one which we don't want here.
        self.view.settings_widget.combo_naming_preset.blockSignals(True)

        self.view.settings_widget.combo_naming_preset.clear()
        self.view.settings_widget.combo_naming_preset.addItems(["", "new..."])
        self.view.settings_widget.combo_naming_preset.insertSeparator(2)
        self.view.settings_widget.combo_naming_preset.addItems(
            self.naming.get_all_pattern_names()
        )

        self.view.settings_widget.combo_naming_preset.setCurrentText(
            self.settings["current_naming"]
        )
        self.view.settings_widget.combo_naming_preset.blockSignals(False)
        self.show_naming()

        # Populate the Nuke executables table.
        table = self.view.settings_widget.table_nuke_executables
        table.clear()
        table.setRowCount(len(self.settings["nuke_executables"]))

        self.exe_finder_widgets = []
        for row, executable in enumerate(self.settings["nuke_executables"]):
            widget = widgets.NukeExeFinder()
            widget.set_path(executable[0])
            widget.set_favorite(executable[1])
            NukeExeFinderController(widget, self)
            table.setCellWidget(row, 0, widget)
            self.exe_finder_widgets.append(widget)

            if self.settings["tooltips"]:
                helper.set_tooltips(widget, "nuke_exe_finder")

        table.resizeRowsToContents()

        # Populate the presets table.
        table = self.view.settings_widget.table_presets
        table.clear()
        table.setRowCount(len(self.settings["preset_paths"]))

        self.preset_path_widgets = []
        for row, path in enumerate(self.settings["preset_paths"]):
            widget = widgets.PathWidget(path)
            PathWidgetController(widget, self, "preset_paths")
            table.setCellWidget(row, 0, widget)
            self.preset_path_widgets.append(widget)

            if self.settings["tooltips"]:
                helper.set_tooltips(widget, "preset_path_widget")

        table.resizeRowsToContents()

        # Populate the nodes roots table.
        table = self.view.settings_widget.table_nodes
        table.clear()
        table.setRowCount(len(self.settings["nodes_roots"]))

        self.nodes_roots_path_widgets = []
        for row, path in enumerate(self.settings["nodes_roots"]):
            widget = widgets.PathWidget(path)
            PathWidgetController(widget, self, "nodes_roots")
            table.setCellWidget(row, 0, widget)
            self.nodes_roots_path_widgets.append(widget)

            if self.settings["tooltips"]:
                helper.set_tooltips(widget, "nodes_roots_widget")

        table.resizeRowsToContents()

        self.view.settings_widget.input_jobs_root.setText(
            self.settings["jobs_root"]
        )

    def populate_recent_files(self):
        """Populate the recent files menu."""
        self.view.menu_recent_files.clear()

        for index, path in enumerate(self.settings["recent_files"], 1):
            action = QtWidgets.QAction(path, self.view)
            action.setShortcut("Shift+Alt+{}".format(index))
            self.view.menu_recent_files.addAction(action)
            action.triggered.connect(
                lambda checked=True, path_=path: self.load_jobs_from_file(path_)
            )

    def save_settings(self):
        """Save the current settings to the setting file."""
        widget = self.view.settings_widget
        self.settings["tooltips"] = \
            widget.check_show_tooltips.isChecked()

        self.settings["always_on_top"] = \
            widget.check_always_on_top.isChecked()

        self.settings["auto_load_last_workfile"] = \
            widget.check_auto_load_last_workfile.isChecked()

        self.settings["auto_apply_naming_extraction"] = \
            widget.check_auto_extract_naming.isChecked()

        self.settings["logging_level"] = \
            widget.combo_logging_level.currentText()

        self.settings["new_jobs_order"] = \
            widget.combo_new_jobs_behavior.currentText()

        self.settings["thumbnail_index"] = \
            widget.combo_thumbnail_index.currentText()

        self.settings["preset_dropdown_type"] = \
            widget.combo_preset_dropdown_type.currentText()

        self.settings["number_recent_files"] = \
            widget.spin_number_recent_files.value()

        self.settings["ingest_processor_threads"] = \
            widget.spin_ingest_threads.value()

        executables_data = []
        for exe_finder_widget in self.exe_finder_widgets:
            executables_data.append([
                exe_finder_widget.input.text(),
                exe_finder_widget.is_favorite,
            ])

        self.settings["nuke_executables"] = executables_data

        preset_paths = [
            widget.get_path()
            for widget in self.preset_path_widgets
        ]
        self.settings["preset_paths"] = preset_paths

        nodes_roots = [
            widget.get_path()
            for widget in self.nodes_roots_path_widgets
        ]
        self.settings["nodes_roots"] = nodes_roots

        self.settings["naming_root"] = self.view.settings_widget.input_naming_root.text()

        # Save the currently selected naming file.
        if self.view.settings_widget.combo_naming_preset.currentText():
            path = os.path.join(
                self.settings["naming_root"],
                "{}.txt".format(self.view.settings_widget.combo_naming_preset.currentText())
            )
            if os.path.isfile(path):
                with open(path, "w") as file_:
                    file_.write(self.view.settings_widget.text_naming_values.toPlainText())

        selected_naming_preset = \
            self.view.settings_widget.combo_naming_preset.currentText()

        if selected_naming_preset != "new...":
            self.settings["current_naming"] = selected_naming_preset

        self.settings.save()

        message.show(self.view.button_search, "Successfully saved settings.")
        self.set_section("jobs")

    def status(self, text):
        """Show the given status text.

        Args:
            text (str): The text to show. Leave empty to make status invisible.

        """
        self.view.set_info.emit(text)
        self.view.statusbar.setVisible(bool(text))

    def _set_status(self, text):
        """Set the given text on the status bar label.

        This is the private method that emits a signal. This is needed to
        ensure updating the UI while running in separate threads. Use 'status'
        method for convenience calls.

        Args:
            text (str): The text to show on the label.

        """
        self.view.statusbar.set_info(text)

    def drop(self, paths):
        """Process dropped sources.

        This will crawl all dropped folders for media and add jobs to the UI.
        In case of dropping a .media file we will load that file. In case of
        dropping an already existing job or one of its tasks we will load the
        job with all its tasks

        Args:
            paths (:obj:`list` of :obj:`str`): Absolute paths of dropped
                files and folders to create as jobs in the UI.

        """
        if not startup.check_nuke_default_executable_set(self.view):
            return

        self.view.empty_jobs_widget.setVisible(False)

        dropped_media = self.get_dropped_media(paths)

        self.modified()

        for media_ in dropped_media.values():
            creator = JobCreator(media_)
            self.processor.add_job(creator)

    def get_dropped_media(self, dropped_paths):
        """Scan for media from dropped paths.

        Args:
            dropped_paths (:obj:`list` of :obj:`str`): Sequence of dropped
                paths to scan.

        Returns:
            dict: All nested media from the dropped path in the format:
                {
                    <path to media1 relative from root>:
                        <Media instance of media1 relative from root>,
                    <path to media2 relative from root>:
                        <Media instance of media2 relative from root>,

                }

        """

        def _append_to_media_data(path):
            """Check to append the media of the given path to media_data.

            Notes:
                We check the given path on its file extension to see if we
                can ingest the data. If the path is not already in our
                media_data contained then we create a Media instance and
                will use the data for ingestion.

            Args:
                path (str): Absolute path of the media to check adding to
                    media_data.

            """
            ext = os.path.splitext(path)[1]

            if ext not in INGESTABLE_EXTENSIONS_FOOTAGE:
                self.logger.debug(
                    "Skip ingesting: '%s'. Non allowed file extension ",
                    path)
                return

            media_ = media.Media(path, autoscan=False)
            if media_.sequence_path in media_data:
                return

            media_.scan()
            media_data[os.path.basename(media_.path)] = media_

        media_data = {}

        for path in dropped_paths:

            # Remove any leading slash in case the path contains one. We
            # need to make sure to remove these, otherwise we cannot determine
            # the folder basename to check if the user dropped an existing
            # job or taks into the GUI.
            if path.endswith("/"):
                path = path[:-1]

            if os.path.isdir(path):
                folder_name = os.path.split(path)[1]
                is_job, is_task = paths.is_job_task_name(folder_name)

                # The dropped path is a job from the jobs root. Load the
                # job and all its tasks.
                if is_job:
                    self.load_job(path)
                    # Return empty dict so that no further media gets
                    # processed, but only the dropped .media file.
                    return {}

                # The dropped path is a task from the jobs root. Load the
                # job and all its tasks.
                elif is_task:
                    parent_folder = os.path.dirname(path)
                    self.load_job(parent_folder)
                    # Return empty dict so that no further media gets
                    # processed, but only the dropped .media file.
                    return {}

                for root, dirs, files in os.walk(path, topdown=False):

                    # Ignore all .meta folders.
                    if ".meta" in root:
                        continue

                    for dir_ in dirs:
                        is_job, is_task = paths.is_job_task_name(dir_)
                        dir_path = os.path.join(root, dir_)

                        # The dropped path is a job from the jobs root. Load the
                        # job and all its tasks.
                        if is_job:
                            self.load_job(dir_path)
                            # Return empty dict so that no further media gets
                            # processed, but only the dropped .media file.
                            continue

                        # The dropped path is a task from the jobs root. Load the
                        # job and all its tasks.
                        elif is_task:
                            parent_folder = os.path.dirname(dir_path)
                            self.load_job(parent_folder)
                            # Return empty dict so that no further media gets
                            # processed, but only the dropped .media file.
                            continue

                    for name in files:
                        path = os.path.join(root, name)
                        # Don't ingest any file that already resides in a
                        # task folder because they have already been handled
                        # by crawling dirs.
                        parent_folder_basename = os.path.basename(os.path.dirname(path))
                        is_job, is_task = paths.is_job_task_name(parent_folder_basename)
                        if is_job or is_task:
                            continue

                        if name in (
                        "preview.mov", "preview.jpg", "meta.json"):
                            continue

                        _append_to_media_data(os.path.join(root, name))

            # When dropping a .media file then load it and don't process
            # any other.
            elif path.endswith(".media"):
                self.load_jobs_from_file(path)
                # Return empty dict so that no further media gets
                # processed, but only the dropped .media file.
                return {}

                _append_to_media_data(path)

            # When dropping a .txt file into smartMedia that contains
            # one source per line.
            elif path.endswith(".txt"):
                with open(path, "r") as file_:
                    lines = [
                        line.replace("\n", "")
                        for line in file_.readlines()
                        if line
                    ]

                for line in lines:
                    media_ = media.Media(line)
                    if not media_.files:
                        continue

                    _append_to_media_data(media_.path)

            else:
                media_ = media.Media(path)
                _append_to_media_data(media_.path)

        return media_data

    def new_job_path(self):
        """Return a new unique job path.

        Returns:
            str: Absolute path of the new task.

        """
        return os.path.join(
            self.settings["jobs_root"],
            paths.new_job_task_name(job=True)
        )

    def set_widget_states(self):
        """Set up the widget states."""
        self.view.slider_size.setValue(self.settings["slider_size"])
        self.populate_recent_files()
        self.populate_output_roots()

    def update_thumb_size(self):
        """Update the thumbnail size on slider changed."""
        width = self.view.slider_size.value()
        self.settings["slider_size"] = width
        self.settings.save()

        for ctrl in self.job_controller:
            ctrl.pixmap = QtGui.QPixmap(ctrl.thumb_path).scaled(
                width, width/2, aspectMode=QtCore.Qt.KeepAspectRatio
            )
            ctrl.view.thumbnail.setPixmap(ctrl.pixmap)

    def set_section(self, section_name):
        """Set the currently active section.

        Args:
            section_name (str): The section to switch to. Choose between
                ["jobs", "presets", "settings"].

        """
        self.current_section = section_name

        self.view.section_jobs_widget.setVisible(section_name == "jobs")
        self.view.section_jobs.setVisible(section_name == "jobs")
        self.view.section_presets.setVisible(section_name == "presets")
        self.view.section_settings.setVisible(section_name == "settings")

        icon_jobs = "jobs_h" if section_name == "jobs" else "jobs_n"
        icon_presets = "presets_h" if section_name == "presets" else "presets_n"
        icon_settings = "settings_h" if section_name == "settings" else "settings_n"

        helper.set_icon(self.view.button_jobs, icon_jobs)
        helper.set_icon(self.view.button_presets, icon_presets)
        helper.set_icon(self.view.button_settings, icon_settings)

        if section_name == "presets":
            self.populate_presets_section()

        elif section_name == "settings":
            self.populate_settings_widget()

        # Drive the visibility of certain widgets based on current section.
        self.view.group_top_right.setVisible(section_name in ("jobs", "presets"))
        self.view.slider_size.setVisible(section_name == "jobs")
        self.view.combo_output_root.setVisible(section_name == "jobs")
        self.view.button_select.setVisible(section_name == "jobs")

    def load_jobs_from_file(self, path):
        """Load all jobs of the given path.

        Args:
            path (str): Absolute path of the media file to load all jobs from.

        """
        self.ask_save_unsaved_changes()

        if not os.path.isfile(path):
            error = "No such job file {}".format(path)
            message.show(self.view.button_search, error)
            return

        with open(path, "r") as file_:
            try:
                job_roots = json.load(file_)["jobs"]
            except ValueError as error:
                self.logger.error("Invalid job file: %s: %s", path, str(error))
                return

        self.new_document()

        for job_root in job_roots:
            self.load_job(job_root)

        self.document = path
        self.unmodified()
        self.add_to_recent_files()

    def save_jobs_to_file(self, show_browser=True):
        """Save current job ids to .media file.

        Args:
            show_browser (bool, optional): If True show a path browser to
                specify a path to save to, otherwise use the instance's
                document. Always show browser if the instance's document is
                not set.

        """
        if not self.document:
            show_browser = True

        if show_browser:
            name_filters = ["media file (*.media)"]
            path = path_browser.show(
                "Save .media file",
                save=True,
                name_filters=name_filters
            )

            if not path:
                return

            self.document = paths.force_extension(path, ".media")

        data = {
            "jobs": [
                ctrl.job_root
                for ctrl in self.job_controller
            ]
        }

        with open(self.document, "w") as file_:
            json.dump(data, file_, indent=constants.JSON_INDENT)

        self.add_to_recent_files()
        self.unmodified()

    def add_to_recent_files(self):
        """Add the current document to the recent files, keep size."""
        # Remove all occurrences of the current document and add the document
        # to the first index so that no duplicate is contained.
        self.settings["recent_files"] = [
            path for path in self.settings["recent_files"]
            if path != self.document
        ]
        self.settings["recent_files"].insert(0, self.document)

        self.settings["recent_files"] = \
            self.settings["recent_files"][:self.settings["number_recent_files"]]

        self.settings.save()
        self.populate_recent_files()

    def load_job(self, job_root):
        """Load job of the given ID and create job_widget for it.

        Args:
            job_root (str): The absolute path of the job to load.

        """
        # Don't load a job if it is already loaded.
        all_job_roots = [job_ctrl.job_root for job_ctrl in self.job_controller]
        if job_root in all_job_roots:
            self.logger.info(
                "Job '%s' is already contained in the view.",
                os.path.basename(job_root)
            )
            return

        job_widget = widgets.JobWidget()

        if self.settings["new_jobs_order"] == "top":
            self.view.layout_jobs.insertWidget(0, job_widget)
        else:
            self.view.layout_jobs.addWidget(job_widget)

        self.view.layout_jobs.setAlignment(job_widget, QtCore.Qt.AlignTop)

        ctrl = JobWidgetController(job_widget, job_root, self)
        if not ctrl.job_ok:
            job_widget.setParent(None)
            job_widget.deleteLater()
            job_widget.hide()
            return

        self.job_controller.append(ctrl)

        self.view.empty_jobs_widget.setVisible(False)

        # Force using the current thumbnail size as all other jobs when adding
        # a new job to the UI. This seemed to be an issue when increasing
        # and decreasing thumbnails and then ingesting new media that triggered
        # this function.
        self.update_thumb_size()

    def create_auto_tasks(self, job_root):
        """Create auto task for given job root.

        Args:
            job_root (str): Absolute path of the job root to create auto tasks
                for.

        """
        for ctrl in self.job_controller:
            if ctrl.job_root != job_root:
                continue

        for preset_name in self.settings["automatic_presets"]:
            task_ctrl = ctrl.new_default_task()
            task_ctrl.apply_preset_by_name(preset_name)

    def close(self):
        """Ask for saving unsaved changes and close the view."""
        self.view.close()


class JobWidgetController(object):
    """Drive a JobWidget."""

    def __init__(self, view, job_root, main_ctrl):
        """Initialize the JobWidgetController instance.

        Args:
            view (smartMedia.mvc.widgets.JobWidget): The JobWidget to drive.
            job_root (str): Absolute path of the job root directory.
            main_ctrl (smartMedia.mvc.controller.Controller): The main view
                controller to be able to communicate back to it.

        """
        self.view = view
        self.job_root = job_root
        self.job_id = os.path.basename(job_root)
        self.main_ctrl = main_ctrl

        self.task_controller = []
        self.job_ok = True

        try:
            self.meta = self._load_meta()
        except Exception as error:
            self.meta = {}
            self.job_ok = False
            main_ctrl.logger.error(str(error))

        if self.meta.get("_is_expanded") is None:
            self.meta["_is_expanded"] = True
            if self.job_ok:
                self.save_meta()

        self.is_expanded = self.meta["_is_expanded"]

        self.naming = naming.Naming(self.meta.get("source"))

        if self.job_ok:
            self.populate_source()
            self.load_tasks()

        self.create_signals()

        if self.main_ctrl.settings["tooltips"]:
            helper.set_tooltips(self.view, "job")

        if self.job_ok:
            self.check_source_offline()

        self.expand_collapse()

    @property
    def meta_root(self):
        """Get absolute path of the meta root directory.

        Returns:
            str: Absolute path of the meta root directory.

        """
        return os.path.join(self.job_root, ".meta")

    @property
    def meta_path(self):
        """Get absolute path of the meta file.

        Returns:
            str: Absolute path of the meta file.

        """
        return os.path.join(self.meta_root, "meta.json")

    def save_meta(self):
        """Save the current state of the meta file."""
        with open(self.meta_path, "w") as file_:
            json.dump(self.meta, file_, indent=constants.JSON_INDENT)

    @property
    def thumb_path(self):
        """Get absolute path of the thumbnail file.

        Returns:
            str: Absolute path of the thumbnail file.

        """
        return os.path.join(self.meta_root, "thumb.jpg")

    @property
    def source_path(self):
        """Get absolute path of the source file.

        Returns:
            str: Absolute path of the source file.

        """
        return self.meta["source"]

    @property
    def source_name(self):
        """Get the source's name.

        Returns:
            str: The source's name.

        """
        return os.path.basename(self.meta["source"]).split(".")[0]

    def create_signals(self):
        """Create signals."""
        self.view.top.double_clicked.connect(
            lambda: self.toggle_expand_collapse())
        self.view.button_add_task.clicked.connect(
            lambda: self.new_default_task()
        )
        self.view.button_reveal.clicked.connect(
            lambda: self.reveal_source()
        )
        self.view.button_expand_collapse.clicked.connect(
            lambda: self.toggle_expand_collapse()
        )
        self.view.customContextMenuRequested.connect(
            lambda: self.show_menu())

        self.view.button_naming.clicked.connect(lambda: self.toggle_naming())

        self.view.combo_naming.currentIndexChanged.connect(
            lambda: self.apply_naming()
        )
        self.view.button_reload_naming.clicked.connect(
            lambda: self.apply_naming()
        )
        self.view.entered.connect(lambda: self.on_entered())

    def on_entered(self):
        """Set the controller as entered job controller in main controller."""
        self.main_ctrl.entered_job_or_task_ctrl = self

    def show_menu(self):
        """Show menu."""
        self.menu = QtWidgets.QMenu()

        helper.darken_context_menu(self.menu)

        # Add actions.
        self.action_new_task = QtWidgets.QAction(
            "New task                           +", self.menu
        )
        helper.set_icon(self.action_new_task, "plus")
        self.action_reveal_source = QtWidgets.QAction(
            "Reveal source                   Alt+O", self.menu
        )
        helper.set_icon(self.action_reveal_source, "reveal")
        self.action_remove = QtWidgets.QAction(
            "Remove from view           Alt+D", self.menu
        )
        helper.set_icon(self.action_remove, "delete")

        self.menu.addAction(self.action_new_task)
        self.menu.addAction(self.action_reveal_source)
        self.menu.addAction(self.action_remove)

        # Create Signals.
        self.action_new_task.triggered.connect(lambda: self.new_default_task())
        self.action_reveal_source.triggered.connect(
            lambda: actions.reveal_job_source(self))
        self.action_remove.triggered.connect(
            lambda: actions.remove_job(self))

        # Show menu.
        current_loc = QtCore.QPoint((QtGui.QCursor.pos()))
        self.menu.popup(current_loc)

    def check_source_offline(self):
        """Check if source is offline and if so mark it in UI."""
        media_ = media.Media(self.meta["source"])
        if media_.files:
            return

        self.view.thumbnail.setProperty("style", "offline_source")
        self.view.label_source_offline.setVisible(True)

    def toggle_expand_collapse(self):
        """Toggle expand collapsing."""
        self.is_expanded = not self.is_expanded

        if self.is_expanded:
            icon = "expand"
        else:
            icon = "contract"

        helper.set_icon(self.view.button_expand_collapse, icon)
        self.expand_collapse()

        self.meta["_is_expanded"] = self.is_expanded
        self.save_meta()

    def expand_collapse(self):
        """Expand / collapse JobWidget."""
        self.view.thumbnail.setVisible(self.is_expanded)
        self.view.table_specs.setVisible(self.is_expanded)

        for task_ctrl in self.task_controller:
            task_ctrl.view.setVisible(self.is_expanded)

    def toggle_naming(self):
        """Toggle the visibility of the naming group."""
        self.view.group_naming.setVisible(
            not self.view.group_naming.isVisible()
        )

    def apply_naming(self):
        """Apply the naming using the given named file from dropdown."""
        selected_naming_preset = self.view.combo_naming.currentText()
        color = "333333"
        match = False
        naming_values = {}

        if selected_naming_preset:
            content_input_naming = self.naming.get_content(selected_naming_preset)

            error = ""
            try:
                self.naming.apply(selected_naming_preset)
                applied = True
            except Exception as naming_error:
                # Path does not match pattern.
                applied = False
                error = str(naming_error)

            if applied:
                naming_values = self.naming.values
                text_naming_values = json.dumps(naming_values, indent=4)
                match = True
            else:
                text_naming_values = error
                color = "ff0000"

        else:
            content_input_naming = ""
            text_naming_values = ""

        self.view.input_naming_pattern.setText(content_input_naming)
        self.view.text_naming_values.setText(text_naming_values)

        self.view.text_naming_values.setStyleSheet(
            "QTextEdit {{border: 1px solid #{}}}".format(color)
        )

        # If we were able to extract naming values then we save the applied
        # naming into the job and also the actual naming values. Also save
        # when we reset the naming values as we then no longer want to store
        # them.
        if match or selected_naming_preset == "":
            self.meta["naming_preset"] = selected_naming_preset
            self.meta["naming_values"] = naming_values
            self.save_meta()

    def reveal_source(self):
        """Reveal the source in explorer."""
        paths.reveal(os.path.dirname(self.meta["source"]))

    def apply_default_preset(self, task_root, target_path=""):
        """Apply the default preset values.

        Args:
            task_root (str): Absolute path of the task root directory to
                apply the default preset to.
            target_path (str, optional): The target path to set in the task
                data.

        """
        # Copy default preset into task.
        src_preset = paths.get_preset_default()

        for name in os.listdir(src_preset):
            # Skip copying over the default preset's README.md as it just
            # describes the default preset but won't be needed any further
            # when applying it.
            if name == "README.md":
                continue

            src = os.path.join(src_preset, name)
            dst = os.path.join(task_root, name)
            shutil.copy(src, dst)

        # Update data.json with data from job.
        preset_ = preset.Preset(task_root)
        job_data = {
            "preset": os.path.basename(task_root),
            "source_path": self.meta["source"],
            "source_frame_mode": "source",
            "source_start": self.meta["specs"]["start"],
            "source_end": self.meta["specs"]["end"],
            # Original source's frame range is needed to keep stored so that
            # we can refer back to when using 'source' as a frame mode.
            "orig_source_start": self.meta["specs"]["start"],
            "orig_source_end": self.meta["specs"]["end"],
            "target_frame_mode": "same as source",
            "target_path": target_path,
            "target_ext": os.path.splitext(target_path)[1].replace(".", ""),
            "target_start": self.meta["specs"]["start"],
            "target_end": self.meta["specs"]["end"],
        }
        preset_.apply_preset(job_data)
        preset_.save()

    def new_default_task(self):
        """Create a new default task that does nothing and return ctrl.

        Returns:
            smartMedia.mvc.controller.TaskWidgetController: The
                TaskWidgetController that was created.

        """
        # Create task root.
        task_root = self.new_task_path()
        if not os.path.isdir(task_root):
            os.makedirs(task_root)

        # Create task widget.
        task_widget = widgets.TaskWidget()
        ctrl = TaskWidgetController(task_widget, task_root, self)
        self.task_controller.append(ctrl)
        self.view.layout_tasks.addWidget(task_widget)

        output_root, mode = self.main_ctrl.current_output_root_and_mode
        if output_root and mode == "abs":
            ctrl.preset["target_path"] = os.path.join(
                output_root,
                os.path.basename(self.meta["source"])
            )
            ctrl.preset.save()

        elif output_root and mode == "rel":
            source_root, basename = os.path.split(self.meta["source"])
            path = os.path.join(
                source_root,
                output_root,
                basename
            )
            ctrl.preset["target_path"] = os.path.normpath(path)
            ctrl.preset.save()

        return ctrl

    def new_task_path(self):
        """Return a new job path using the current timestamp.

        We use the time stamp in milliseconds to avoid creating new tasks
        with the same id. This could for instance happen when just using a
        timestamp using seconds when a user clicks quickly the 'new task'
        button (plus button) in the UI. By using milliseconds we ensure
        uniqueness.

        Returns:
            str: Absolute path of the new task.

        """
        return os.path.join(self.job_root, paths.new_job_task_name(task=True))

    def populate_source(self):
        """Populate the job widget and add all its tasks."""
        self.view.title.setText(os.path.basename(self.meta["source"]))
        self.view.title.setToolTip(self.meta["source"])

        slider_size = self.main_ctrl.settings["slider_size"]

        self.pixmap = QtGui.QPixmap(self.thumb_path).scaled(
            slider_size,
            slider_size / 2,
            aspectMode = QtCore.Qt.KeepAspectRatio
        )
        self.view.thumbnail.setPixmap(self.pixmap)

        specs = [
            "<strong>{}:</strong> {}".format(key, value)
            for key, value in self._load_source_specs().items()
        ]
        for column in range(3):
            for row in range(3):
                try:
                    label = QtWidgets.QLabel(specs[column*3 + row])
                    self.view.table_specs.setCellWidget(row, column, label)
                except IndexError:
                    continue

        self.view.combo_naming.addItems(
            [""] + self.naming.get_all_pattern_names()
        )

        naming_preset = self.meta.get("naming_preset", "")
        if naming_preset:
            self.view.combo_naming.setCurrentText(naming_preset)

            content_input_naming = self.naming.get_content(naming_preset)
            self.view.input_naming_pattern.setText(content_input_naming)
            self.view.text_naming_values.setText(
                json.dumps(
                    self.meta.get("naming_values") or {},
                    indent=constants.JSON_INDENT
                )
            )

    def load_tasks(self):
        """Populate the job with all currently assigned tasks."""
        task_paths = sorted([
            os.path.join(self.job_root, name)
            for name in os.listdir(self.job_root)
            if os.path.isdir(os.path.join(self.job_root, name))
            and name != ".meta"
        ])

        for task_path in task_paths:
            task_widget = widgets.TaskWidget()
            ctrl = TaskWidgetController(task_widget, task_path, self)
            self.task_controller.append(ctrl)
            self.view.layout_tasks.addWidget(task_widget)

    def _load_meta(self):
        """Try the job's metadata and raise error if that is not possible.

        Returns:
            dict: The parsed job's meta file.

        Raises:
            IOError: When the job has no meta.json file.
            ValueError: When the job's meta.json file cannot be parsed.

        """
        if not os.path.isfile(self.meta_path):
            raise IOError(
                "Failed loading job. No such job meta.json file for job "
                "in {}".format(self.meta_path)
            )

        with open(self.meta_path, "r") as file_:
            try:
                return json.load(file_)
            except ValueError:
                raise ValueError(
                    "Skipping loading job. Non well formed meta.json file for "
                    "job in {}".format(self.meta_path))

    def _load_source_specs(self):
        meta = OrderedDict()
        for key in self.main_ctrl.settings["source_fields"]:
            if key in self.meta["specs"]:
                meta[key] = self.meta["specs"][key]

        return meta


class TaskWidgetController(object):
    """Drive a TaskWidget."""

    STATUS_FINISHED = "finished"
    STATUS_FAILED = "failed"

    STATUS_COLORS = {
        STATUS_FINISHED: "status_green",
        STATUS_FAILED: "status_red",
    }

    def __init__(self, view, task_root, job_widget_ctrl):
        """Initialize the TaskWidgetController instance.

        Args:
            view (smartMedia.mvc.widgets.TaskWidget): The view to drive.
            task_root (str): Absolute path of the task root directory.
            main_ctrl (smartMedia.mvc.controller.Controller): The main view
                controller to be able to communicate back to it.
            job_widget_ctrl (smartMedia.mvc.controller.JobWidgetController):
                The JobWidget's controller to be able to communicate back to
                the JobWidget.

        """
        self.view = view
        self.task_root = task_root
        self.job_widget_ctrl = job_widget_ctrl
        self.selected = False
        self.timer_render_progress = QtCore.QTimer()
        self.timer_render_progress.setInterval(2000)

        self.main_ctrl = self.job_widget_ctrl.main_ctrl
        self.detail_widgets = {}
        self._details_build = False
        self._show_details = False

        if os.path.isfile(self.task_data_path):
            self.preset = preset.Preset(self.task_root)
        else:
            self.job_widget_ctrl.apply_default_preset(self.task_root)
            self.preset = preset.Preset(self.task_root)

        self.populate_widgets()
        self.set_status_label()
        self.set_render_log_button_visibility()
        self.set_reveal_button_visibility()
        self.create_signals()

        if self.main_ctrl.settings["tooltips"]:
            helper.set_tooltips(self.view, "task")

    @property
    def render_log(self):
        """Get absolute path to the task's render log.

        Returns:
            str: Absolute path to the task's render log.

        """
        return os.path.join(self.task_root, "render.log")

    def create_signals(self):
        """Create signals."""
        self.view.button_output_path.clicked.connect(self.browse_output_path)
        self.view.combo_presets.currentIndexChanged.connect(
            lambda: self.apply_preset()
        )
        self.view.button_start.clicked.connect(lambda: self.run())
        self.view.customContextMenuRequested.connect(lambda: self.show_menu())
        self.view.button_render_log.clicked.connect(
            lambda: self.open_render_log()
        )
        self.view.button_reveal.clicked.connect(lambda: self.reveal_task_output())
        self.view.button_details.clicked.connect(lambda: self.toggle_details())
        self.view.finished.connect(lambda success: self.on_task_finished(success))
        self.view.button_lock.clicked.connect(lambda: self.set_task_locked(False))
        self.view.button_reload.clicked.connect(lambda: self.apply_preset())
        self.view.entered.connect(lambda: self.on_entered())
        self.view.button_select.clicked.connect(lambda: self.toggle_selection())
        self.timer_render_progress.timeout.connect(lambda: self.update_render_progress())

    def update_render_progress(self):
        """Update render progress."""
        path = os.path.join(self.task_root, "progress")
        if not os.path.isfile(path):
            return

        with open(path, "r") as file_:
            progress = file_.read()

        # In the very rare condition that it gets written to the file while
        # reading the file will be empty. Seen this once. In that case don't
        # update the progress and wait on the next timeout to update the
        # progress then.
        if not progress:
            return

        self.view.label_status.setText("{}%".format(progress))

    def toggle_selection(self):
        """Toggle the task being selected/unselected."""
        self.selected = not self.selected

        icon = "selected" if self.selected else "unselected"
        helper.set_icon(self.view.button_select, icon)

    def select_task(self):
        """Select the task."""
        self.selected = True
        helper.set_icon(self.view.button_select, "selected")

    def unselect_task(self):
        """Unselect the task."""
        self.selected = False
        helper.set_icon(self.view.button_select, "unselected")

    def on_entered(self):
        """Set the controller as entered job controller in main controller."""
        self.main_ctrl.entered_job_or_task_ctrl = self

    def toggle_details(self):
        """Toggle showing/hiding the task's details."""
        # Lazily build details on demand. Otherwise we would always need to
        # build all details when it might not be necessary as the user might
        # not view them for each task.
        if not self._details_build:
            self.build_details()
            self._details_build = True

        self._show_details = not self._show_details

        icon = "arrow_down" if self._show_details else "arrow_right"
        helper.set_icon(self.view.button_details, icon)

        self.view.widget_details.setVisible(self._show_details)
        if not self._show_details:
            self.view.widget_log.setVisible(False)

        if not self._show_details:
            self.view.layout_main.setContentsMargins(10, 10, 10, 10)

    def build_details(self):
        """Build details section.

        Notes:
            build lazily and only on demand as we don't need to end up building
            widgets that we don't see at the beginning because the details
            section in contracted at the beginning.

        """
        helper.clear_layout(self.view.layout_details)

        task_details = self.main_ctrl.settings["task_details"]

        keys_sorted_by_index = sorted(
            task_details.keys(), key=lambda key: task_details[key]["index"]
        )

        for detail_key in keys_sorted_by_index:
            try:
                widget_class = DETAILS_WIDGETS[detail_key]["widget"]
            except KeyError:
                # DetailsWidget with that given name from configuration does
                # not exist. Skip it as there is nothing to build here.
                continue

            widget = widget_class(
                self.preset,
                detail_key,
                custom_label=task_details[detail_key].get("custom_label")
            )

            # Drive the visibility of the details widget by configuration when
            # existing for that detail widget.
            visible = True

            if detail_key in task_details:
                visible = task_details[detail_key]["visible"]

            widget.setVisible(visible)

            self.view.layout_details.addWidget(widget)
            self.detail_widgets[detail_key] = widget

        spacer = QtWidgets.QWidget()
        spacer.setMinimumHeight(10)
        self.view.layout_details.addWidget(spacer)

    def rebuild_details(self):
        """Re-build the details section."""
        self._details_build = False
        if self._show_details:
            self.build_details()

    def reveal_task_output(self):
        """Reveal the task's output when its status is finished."""
        if self.preset["status"] == "finished":
            actions.reveal_task_output(self)

    def show_menu(self):
        """Show menu."""
        self.menu = QtWidgets.QMenu()

        helper.darken_context_menu(self.menu)

        selected_media = [media.Media(self.preset["target_path"])]

        # Add all custom commands.
        # Perform late import to make sure all the custom commands have been
        # registered up to this point.
        self.menu_media_commands = QtWidgets.QMenu("Media Commands")
        helper.darken_context_menu(self.menu_media_commands)

        helper.set_icon(self.menu_media_commands, "command")
        from smartMedia.osl import CRAGL_SMARTMEDIA_MEDIA_COMMANDS
        for class_ in CRAGL_SMARTMEDIA_MEDIA_COMMANDS:
            instance = class_()
            command_action = QtWidgets.QAction(
                instance.label(),
                self.menu_media_commands
            )
            self.menu_media_commands.addAction(command_action)
            command_action.triggered.connect(functools.partial(
                actions.process_media_command, instance,
                selected_media, self.main_ctrl.view
            ))

        self.menu_media_commands.addSeparator()

        self.action_docs_custom_command = QtWidgets.QAction(
            "How to create custom media commands...",
            self.menu_media_commands
        )
        self.menu_media_commands.addAction(self.action_docs_custom_command)

        self.action_docs_custom_command.triggered.connect(
            lambda: actions.browse_docs_creating_custom_commands()
        )

        # Add actions.
        self.menu_reveal = QtWidgets.QMenu("Reveal", self.menu)
        helper.set_icon(self.menu_reveal, "reveal")

        self.action_run_task = QtWidgets.QAction(
            "Render                            Ctrl+R", self.menu)
        self.action_run_task.setShortcut("")
        helper.set_icon(self.action_run_task, "play")

        self.action_setup_task = QtWidgets.QAction(
            "Setup only                      Alt+R", self.menu)
        helper.set_icon(self.action_setup_task, "setup")
        self.action_reveal_task = QtWidgets.QAction(
            "Task                     Alt+T", self.menu
        )
        self.action_reveal_output = QtWidgets.QAction(
            "Output                 Alt+O", self.menu
        )
        self.action_edit_details_visibility = QtWidgets.QAction(
            "Change details layout",
            self.menu
        )
        helper.set_icon(self.action_edit_details_visibility, "eye")
        self.action_lock = QtWidgets.QAction(
            "Lock                                Alt+L", self.menu)
        helper.set_icon(self.action_lock, "locked")

        self.action_delete = QtWidgets.QAction(
            "Remove                          Alt+D", self.menu
        )
        helper.set_icon(self.action_delete, "delete")

        # Add actions to menu.
        # At the moment support edit mode only for one element.
        self.menu_reveal.addAction(self.action_reveal_output)
        self.menu_reveal.addAction(self.action_reveal_task)

        self.menu.addAction(self.action_run_task)
        self.menu.addAction(self.action_setup_task)
        self.menu.addMenu(self.menu_reveal)
        self.menu.addMenu(self.menu_media_commands)
        self.menu.addAction(self.action_edit_details_visibility)
        self.menu.addAction(self.action_lock)
        self.menu.addAction(self.action_delete)

        # Create Signals.
        self.action_run_task.triggered.connect(
            lambda: actions.run_task(self))
        self.action_setup_task.triggered.connect(
            lambda: actions.run_task(self, setup_only=True))
        self.action_reveal_output.triggered.connect(
            lambda: actions.reveal_task_output(self))
        self.action_reveal_task.triggered.connect(
            lambda: actions.reveal_task_root(self))
        self.action_delete.triggered.connect(
            lambda: actions.delete_task(self))
        self.action_edit_details_visibility.triggered.connect(
            lambda: actions.show_details_visibility(self.main_ctrl)
        )
        self.action_lock.triggered.connect(lambda: self.set_task_locked(True))

        # Show menu.
        current_loc = QtCore.QPoint((QtGui.QCursor.pos()))
        self.menu.popup(current_loc)

    def run(self, setup_only=False):
        """Run the task.

        Args:
            setup_only (bool, optional): If True set up the task only but don't
                process any rendering of media. Defaults to False so that we
                render media by default.

        """
        if not self.preset["target_path"]:
            message.show(self.view, "Please set an output path.")
            return

        self.view.setDisabled(True)

        self.view.button_start.setVisible(False)
        self.view.label_processing.setVisible(True)

        render = True
        if setup_only:
            render = False

        label = "rendering..." if render else "setting up..."
        self.update_task_status(label)
        self.set_status_label()

        helper.force_update_view()

        self.timer_render_progress.start()

        task = job.Job(self.task_root, ctrl=self, render=render)
        thread = threading.Thread(target=task.run)
        thread.start()

    def on_task_finished(self, success):
        """Update UI's status when the job has finished.

        Args:
            success (bool): If True the job finished without raising any
                errors, otherwise errors where raised.

        """
        status = self.STATUS_FINISHED if success else self.STATUS_FAILED
        self.update_task_status(status)

        self.set_status_label()
        self.set_render_log_button_visibility()
        self.set_reveal_button_visibility()

        self.view.label_processing.setVisible(False)
        self.view.button_start.setVisible(True)

        self.view.setDisabled(False)
        self.timer_render_progress.stop()

        # Delete render progress file.
        render_progress_file = os.path.join(self.task_root, "progress")
        try:
            os.remove(render_progress_file)
        except Exception:
            pass

    def open_render_log(self):
        """Open render log."""
        if not self._show_details:
            self.toggle_details()

        show_log = not self.view.widget_log.isVisible()
        if show_log:
            icon = "menu"
        else:
            icon = "document"

        helper.set_icon(self.view.button_render_log, icon)

        self.view.widget_log.setVisible(show_log)
        self.view.widget_details.setVisible(not show_log)

        if self.view.widget_log.isVisible():
            self.load_render_log()

    def load_render_log(self):
        """Load the render log.

        Load with every click to ensure we always display the latest render
        log content.

        """
        log_path = os.path.join(self.preset.root, "render.log")
        if not os.path.isfile(log_path):
            return

        with open(log_path, "r") as file_:
            self.view.text_log.setText(file_.read())

    def set_render_log_button_visibility(self):
        """Set the visibility of the render log button."""
        self.view.button_render_log.setVisible(
            os.path.isfile(self.render_log)
        )

    def set_reveal_button_visibility(self):
        """Set up the visibility of the reveal button."""
        if self.preset["status"] == "finished":
            helper.set_icon(self.view.button_reveal, "reveal")

    def set_status_label(self):
        """Set the status label on the task widget."""
        status = self.preset["status"]
        self.view.label_status.setText(status)
        self.view.label_status.setProperty(
            "style", self.STATUS_COLORS.get(status, "status_default")
        )
        self.view.label_status.setVisible(bool(status))
        helper.set_style_sheet(self.view)

    def update_task_status(self, status):
        """Add the given status to the task's data file.

        Args:
            status (str): The status to add.

        """
        self.preset["status"] = status
        self.preset.save()

    def apply_preset_by_name(self, preset_name):
        self.view.combo_presets.setCurrentText(preset_name)

    def apply_preset(self):
        """Apply the current preset to the task.

        This will remove all previous preset files in the preset root and
        copy the files of the selected preset into the task. Caution on
        the data.json file. As it already might contain senible data about
        render path, etc, we must update the existing data.json instead of
        deleting it.

        """
        basename = os.path.basename(self.preset["source_path"]).split(".")[0]
        preset_name = self.view.combo_presets.currentText()
        self.main_ctrl.logger.info(
            "%s: Applying preset '%s'",
            basename,
            preset_name
        )

        if not preset_name:
            selected_preset = preset.Preset(paths.get_preset_default())
        else:
            selected_preset = self.main_ctrl.presets.get(preset_name)
            if not selected_preset:
                return

        # Remove all task files, but not the data.json as it contains sensible
        # information like the render path and we want to preserve this data.
        # We cannot simply delete it but need to keep and update it.
        for name in os.listdir(self.task_root):
            if name == "data.json":
                continue
            paths.delete(os.path.join(self.task_root, name))

        # Caution, we must apply the default preset here and then override
        # it, otherwise we could be running into situations where we take over
        # data from a previously selected preset and updating it with a newer
        # one.
        self.job_widget_ctrl.apply_default_preset(
            self.task_root, target_path=self.preset["target_path"]
        )

        self.preset = preset.Preset(self.task_root)

        self.preset.apply_preset(selected_preset)
        self.preset["preset"] = preset_name
        self.preset["naming"] = \
            self.job_widget_ctrl.meta.get("naming_values") or {}

        path, ext = os.path.splitext(self.preset["target_path"])

        # Target path has not been set, yet. Use the source path with the
        # preset's file extension.
        if not path:
            path = os.path.splitext(self.job_widget_ctrl.meta["source"])[0]

        self.preset["target_path"] = "{}.{}".format(
            path, self.preset["target_ext"].replace(".", "")
        )

        # Add or remove file sequence counter based on the incoming file
        # extension.
        self.preset["target_path"] = paths.add_remove_filesequence_counter(
            self.preset["target_path"]
        )

        self.view.button_output_path.setText(os.path.basename(self.preset["target_path"]))
        self.view.button_output_path.setToolTip(self.preset["target_path"])

        self.preset.save()

        # Copy all files from preset into task root.
        for name in os.listdir(selected_preset.root):
            # As we have previously already updated the data.json we don't
            # need to override it here because we need the updated file.
            if name == "data.json":
                continue

            src = os.path.join(selected_preset.root, name)
            dst = os.path.join(self.task_root, name)

            if os.path.isfile(src):
                shutil.copy(src, dst)
            elif os.path.isdir(src):
                shutil.copytree(src, dst)

        # Apply data.py if the selected preset contains one.
        datapy_path = os.path.join(selected_preset.root, "data.py")

        if os.path.isfile(datapy_path):
            self.main_ctrl.logger.info("Applying preset's data.py")
            data_module = importfile(datapy_path)

            try:
                data_updated_by_data_module = data_module.main(
                    self.preset.data,
                    self.main_ctrl.logger
                )
            except Exception as error:
                raise RuntimeError(
                    "Error executing {}:\n{}".format(
                        datapy_path,
                        str(error)
                    )
                )

            self.preset.data.update(data_updated_by_data_module)
            self.preset.save()

        self.rebuild_details()

    def populate_widgets(self):
        """Populate widgets."""
        self.view.combo_presets.addItem("")

        for preset in self.main_ctrl.presets:
            self.view.combo_presets.addItem(preset, userData=preset)

        helper.fix_combo_menu_width(self.view.combo_presets)

        preset_names = [
            os.path.basename(preset)
            for preset in self.main_ctrl.presets
        ]

        template = self.preset["preset"]

        if template in preset_names:
            helper.combo_set_text(self.view.combo_presets, template)

        self.view.button_output_path.setText(os.path.basename(
            self.preset.data.get("target_path", ""))
        )

        self.view.button_output_path.setToolTip(
            self.preset.data.get("target_path", "")
        )

        self.set_lock_state(self.preset.data.get("locked", False))

    def set_task_locked(self, is_locked):
        """Set task locked/unlocked."""
        self.preset["locked"] = is_locked
        self.preset.save()

        self.set_lock_state(is_locked)

    def set_lock_state(self, is_locked):
        if is_locked:
            self.view.button_lock.setVisible(True)
        else:
            self.view.button_lock.setVisible(False)
            self.view.widget_details.setDisabled(False)

        widgets = (
            self.view.widget_details,
            self.view.combo_presets,
            self.view.button_output_path,
        )
        for widget in widgets:
            widget.setDisabled(is_locked)


    @property
    def task_data_path(self):
        """Return absolute path of the task's data file.

        Returns:
            str: Absolute path of the task's data file.

        """
        return os.path.join(self.task_root, "data.json")

    def load(self):
        """Load the task data file.

        Returns:
            dict: The task data file's content.

        """
        with open(self.task_data_path, "r") as file_:
            return json.load(file_)

    def update(self):
        """Update the data.json with the current task_data."""
        with open(self.task_data_path, "w") as file_:
            json.dump(self.preset.data, file_, indent=constants.JSON_INDENT)

    def browse_output_path(self):
        """Show browser to browse the output path and update task's path."""
        target_directory = self.preset["target_path"]
        basename = os.path.basename(self.preset["target_path"])
        if not os.path.isdir(target_directory):
            target_directory = os.path.dirname(target_directory)

        # Ensure the directory exists, otherwise the path browser will not
        # start at that directory.
        paths.ensure_directory(target_directory)

        path = path_browser.show(
            "Browser render path",
            start_directory=target_directory,
            start_filename=basename
        )
        if not path:
            return

        # If the user has not entered a file extension then add the file
        # extension from the preset.
        if os.path.splitext(path)[1] in ("", ".", None):
            if self.preset["preset"] in self.main_ctrl.presets:
                preset = self.main_ctrl.presets[self.preset["preset"]]
                file_type = preset["target_ext"]
                path = "{}.{}".format(path, file_type)

        file_type = os.path.splitext(path)[1].replace(".", "")
        self.preset["target_ext"] = file_type

        # If user used an image based output but did not specify a padding
        # then lets add it.
        if ".{}".format(file_type) not in constants.VIDEO_EXTENSIONS \
                and "#" not in path:
            number_digits = len(str(self.preset.data["target_end"]))
            hash = "#" * number_digits
            path = "{}.{}.{}".format(os.path.splitext(path)[0], hash, file_type)

        self.preset["target_path"] = path

        self.update()

        self.view.button_output_path.setText(os.path.basename(path))
        self.view.button_output_path.setToolTip(path)
        self.update_target_path_details_widget()

    def update_target_path_details_widget(self):
        """Update the output path when the details have been build already.

        If they haven't then we don't need to care about this as the output
        path will then use the latest value, which is the updated path, once
        we show the details.

        """
        if self._details_build:
            if "target_path" in self.detail_widgets:
                self.detail_widgets["target_path"].input.setText(
                    self.preset["target_path"]
                )


class NukeExeFinderController(object):
    """Drive a NukeExefinder view."""

    def __init__(self, view, main_ctrl):
        """Initialize the NukeExeFinderController instance.

        Args:
            view (smartMedia.mvc.widgets.NukeExeFinder): The view to drive.
            main_ctrl (smartMedia.mvc.controller.Controller): The main view
                controller to be able to communicate back to the main view.

        """
        super(NukeExeFinderController, self).__init__()

        self.view = view
        self.main_ctrl = main_ctrl

        self.create_signals()

    def create_signals(self):
        """Create signals."""
        self.view.button_favorite.clicked.connect(lambda: self.set_favorite())
        self.view.button_test.clicked.connect(lambda: self.test_nuke_exe())
        self.view.button_add.clicked.connect(lambda: self.add_new_nuke_exe_finder())
        self.view.button_del.clicked.connect(lambda: self.del_nuke_exe_finder())

    def add_new_nuke_exe_finder(self):
        """Add a new Nuke exe finder to the UI."""
        self.main_ctrl.settings["nuke_executables"].append(["", False])
        self.main_ctrl.settings.save()
        self.main_ctrl.populate_settings_widget()

    def del_nuke_exe_finder(self):
        """Delete the current Nuke exe finder."""
        for index, exe_data in enumerate(self.main_ctrl.settings["nuke_executables"]):
            if exe_data[0] == self.view.input.text():
                del self.main_ctrl.settings["nuke_executables"][index]
                break

        self.main_ctrl.settings.save()
        self.main_ctrl.populate_settings_widget()

    def set_favorite(self):
        """Set favorite for the given Nuke version."""
        if not self.view.input.text():
            return

        self.main_ctrl.save_settings()

        for index, exe_data in enumerate(self.main_ctrl.settings["nuke_executables"]):
            if exe_data[0] == self.view.input.text():
                exe_data[1] = True
            else:
                exe_data[1] = False

        self.main_ctrl.settings.save()
        self.main_ctrl.populate_settings_widget()

    def test_nuke_exe(self):
        """Test if path contains a Nuke executable."""
        path_resolver = PathResolver(self.view.input.text())

        try:
            path_resolver.run()
            self.view.input.setText(path_resolver.nuke)

            msg = (
                "Successfully found Nuke executable in:\n"
                "{}".format(path_resolver.nuke)
            )

        except OSError as error:
            msg = str(error)
        message.show(self.main_ctrl.view.button_search, msg)


class PresetWidgetController(object):
    """Drive a PresetWidget."""

    def __init__(self, view, preset, main_ctrl):
        """Initialize the PresetWidgetController.

        Args:
            view (smartMedia.mvc.widgets.PresetWidget): The view to drive.
            preset (smartMedia.preset.Preset): The preset to use.
            main_ctrl (smartMedia.mvc.controller.Controller): The main
                controller to update the settings for automatic presets.

        """
        self.view = view
        self.preset = preset
        self.main_ctrl = main_ctrl

        self._is_automatic = False

        self.populate_widgets()
        self.create_signals()

    def create_signals(self):
        """Create signals."""
        self.view.button_reveal.clicked.connect(lambda: self.reveal_preset())
        self.view.button_toggle_details.clicked.connect(lambda: self.toggle_details())
        self.view.button_save.clicked.connect(lambda: self.update_preset())
        self.view.button_automatic.clicked.connect(lambda: self.toggle_automatic())

    def toggle_automatic(self):
        """Toggle automatic preset."""
        self.set_automatic(not self._is_automatic)

    def reveal_preset(self):
        """Reveal the preset."""
        paths.reveal(os.path.dirname(self.preset.path))

    def populate_widgets(self):
        """Populate widgets."""
        self.view.label_name.setText(self.preset.name)
        self.view.text_preset_data.setText(
            json.dumps(self.preset.data, indent=constants.JSON_INDENT)
        )

        self.set_automatic(
            self.preset.name in self.main_ctrl.settings["automatic_presets"],
            store=False
        )

    def set_automatic(self, is_automatic, store=True):
        """Set preset as automatic default preset.

        This sets the underlying _is_automatic attribute, the icon and update
        the settings.

        Args:
            is_automatic (bool): If True set the preset as automatic, otherwise
                unset it.
            store (bool, optional): If True store update to settings, otherwise
                don't store it. Don't store on set up as we read from the
                settings before.

        """
        self._is_automatic = is_automatic
        icons = {
            True: "favorite_active",
            False: "favorite_inactive",
        }
        helper.set_icon(self.view.button_automatic, icons[self._is_automatic])

        if not store:
            return

        if self._is_automatic:
            self.main_ctrl.settings["automatic_presets"].append(self.preset.name)

        else:
            try:
                self.main_ctrl.settings["automatic_presets"].remove(self.preset.name)
            except ValueError:
                # Preset was not in the automatic_presets settings list.
                pass

        self.main_ctrl.settings.save()

    def toggle_details(self):
        """Toggle the details section."""
        is_visible = not self.view.group_details.isVisible()

        self.view.group_details.setVisible(is_visible)
        icon = "arrow_right"
        if is_visible:
            icon = "arrow_down"
        helper.set_icon(self.view.button_toggle_details, icon)

    def update_preset(self):
        """Update the preset by the text content when it is valid json."""
        content = self.view.text_preset_data.toPlainText()

        content = content.replace("True", "true")
        content = content.replace("False", "false")

        try:
            json.loads(content)
        except json.decoder.JSONDecodeError as error:
            self.view.text_preset_data.setStyleSheet(
                "border: 1px solid #ff0000;"
                "font-size: 14pt;"
            )

            message.show(
                self.view,
                "Invalid preset content:\n{}".format(str(error))
            )
            return

        with open(self.preset.path, "w") as file_:
            file_.write(content)

        self.view.text_preset_data.setStyleSheet(
            "border: 1px solid #333333;"
            "font-size: 14pt;"
        )

        message.show(self.view, "Successfully updated preset.")

        self.preset.reload()


class PathWidgetController(object):
    """Drive a PathWidget view."""

    def __init__(self, view, main_ctrl, settings_key):
        """Initialize the PathWidgetController instance.

        Args:
            view (smartMedia.mvc.widgets.NukeExeFinder): The view to drive.
            main_ctrl (smartMedia.mvc.controller.Controller): The main view
                controller to be able to communicate back to the main view.
            settings_key (str): The settings key to query and save to for
                path roots.

        """
        super(PathWidgetController, self).__init__()

        self.view = view
        self.main_ctrl = main_ctrl
        self.settings_key = settings_key

        self.create_signals()

    def create_signals(self):
        """Create signals."""
        self.view.button_add.clicked.connect(
            lambda: self.add_new_preset_path_widget()
        )
        self.view.button_del.clicked.connect(
            lambda: self.del_widget()
        )

    def add_new_preset_path_widget(self):
        """Add a new preset path widget to the UI."""
        self.main_ctrl.settings[self.settings_key].append("")
        self.main_ctrl.settings.save()
        self.main_ctrl.populate_settings_widget()

    def del_widget(self):
        """Delete the current preset path widget."""
        for index, path in enumerate(self.main_ctrl.settings[self.settings_key]):
            if path == self.view.input.text():
                del self.main_ctrl.settings[self.settings_key][index]
                break

        self.main_ctrl.settings.save()
        self.main_ctrl.populate_settings_widget()


class TaskDetailsVisibilityViewController(object):
    """Drive a TaskDetailsVisibilityView."""

    def __init__(self, view, main_ctrl):
        """Initialize the TaskDetailsVisibilityViewController instance.

        Args:
            view (smartMedia.mvc.view.TaskDetailsVisibilityView): The view
                to drive.
            main_ctrl (smartMedia.mvc.controller.Controller): The main view
                controller to be able to talk back to the main window.

        """
        self.view = view
        self.main_ctrl = main_ctrl

        self.populate_list()
        self.create_signals()

    def create_signals(self):
        """Create signals."""
        self.view.button_close.clicked.connect(lambda: self.view.close())

    def populate_list(self):
        """Populate the details list."""
        task_details = self.main_ctrl.settings["task_details"]

        self.view.table_details.clear()
        self.view.table_details.setRowCount(len(task_details))

        row = 0
        keys_sorted_by_index = sorted(
            task_details.keys(), key=lambda key: task_details[key]["index"]
        )

        for key in keys_sorted_by_index:
            widget = widgets.TaskDetailVisibilityWidget(
                "{}".format(key), task_details[key]["visible"], self.main_ctrl, self
            )
            self.view.table_details.setCellWidget(row, 0, widget)
            row += 1

        self.view.table_details.resizeRowsToContents()


class JobInfoViewController(object):
    """Drive a JobInfoView."""

    def __init__(self, view, entered_job_or_task_ctrl):
        """Initialize the JobInfoViewController instance.

        Args:
            view (smartMedia.mvc.view.JobInfoView): The view to drive.
            entered_job_or_task_ctrl (
            smartMedia.mvc.controller.JobWidgetController or
            smartMedia.mvc.controller.TaskWidgetController): The Job or Task
                widget controller that we hovered over.

        """
        self.view = view
        self.entered_job_or_task_ctrl = entered_job_or_task_ctrl

        self.populate()
        self.create_signals()

    def create_signals(self):
        """Create signals."""
        self.view.button_reveal.clicked.connect(lambda: self.reveal_source())

    def populate(self):
        """Populate the view with content.

        Notes:
            When hovering over a JobWidget we show the meta data of the job.
            When hovering over a TaskWidget we show the content of the task's
            data.json file.

        """
        ctrl = self.entered_job_or_task_ctrl

        if isinstance(ctrl, JobWidgetController):
            title = "Source: {}".format(os.path.basename(ctrl.meta["source"]))
            content = ctrl.meta

        else:
            title = "Task: {}".format(os.path.basename(ctrl.preset["target_path"]))
            content = ctrl.preset.data

        self.view.label_title.setText(title)
        self.populate_tree(content)

    def reveal_source(self):
        """Reveal the source in explorer anc close view."""
        ctrl = self.entered_job_or_task_ctrl
        if isinstance(ctrl, JobWidgetController):
            path = ctrl.source_path
        else:
            path = ctrl.preset["target_path"]

        paths.reveal(os.path.dirname(path))
        self.view.close()

    def populate_tree(self, data):
        """Populate the tree with the given data.

        Args:
            data (dict): The data to show in the tree.

        """
        self.view.tree.clear()

        for key in sorted(data.keys()):
            value = data[key]

            if isinstance(value, (list, dict)):
                display_value = value
            else:
                display_value = str(value)
            item = QtWidgets.QTreeWidgetItem(None, [key, display_value])
            self.view.tree.addTopLevelItem(item)

            if isinstance(value, list):
                for element in value:
                    QtWidgets.QTreeWidgetItem(item, [str(element)])

            elif isinstance(value, dict):
                for key, value_ in value.items():
                    QtWidgets.QTreeWidgetItem(item, [key, str(value_)])

        self.view.tree.expandAll()


class OutputRootSelectorViewController(object):
    """Drive an OutputRootSelectorView."""

    def __init__(self, view, main_ctrl):
        """Initialize the OutputRootSelectorViewController instance.

        Args:
            view (smartMedia.mvc.view.OutputRootSelectorView): The view to
                drive.
            main_ctrl (smartMedia.mvc.controller.Controller): The main
                view controller to be able to communicate back to the main
                view.

        """
        self.view = view
        self.main_ctrl = main_ctrl

        self.populate_widgets()
        self.create_signals()

        if self.main_ctrl.settings["tooltips"]:
            helper.set_tooltips(self.view, "output_root_selector")

    def populate_widgets(self):
        """Set up and populate widgets."""
        self.view.radio_absolute.setChecked(True)

    def create_signals(self):
        """Create signals."""
        self.view.button_cancel.clicked.connect(
            lambda: self.view.close()
        )
        self.view.button_browse.clicked.connect(
            lambda: self.browse_root()
        )
        self.view.button_create.clicked.connect(
            lambda: self.create_output_root()
        )
        self.view.radio_absolute.clicked.connect(
            lambda: self.view.button_browse.setVisible(True)
        )
        self.view.radio_relative.clicked.connect(
            lambda: self.view.button_browse.setVisible(False)
        )

    def browse_root(self):
        """Browse to an output root."""
        root = path_browser.show("Browse output root", only_directory=True)
        if root:
            self.view.input_path.setText(root)

    def create_output_root(self):
        """Create a new output root."""
        if self.view.input_path.text() == "":
            return

        if self.view.radio_absolute.isChecked():
            mode = "abs"
        else:
            mode = "rel"

        self.main_ctrl.view.combo_output_root.blockSignals(True)
        self.main_ctrl.settings["output_roots"].append(
            {
                "active": False,
                "mode": mode,
                "path": self.view.input_path.text()
            }
        )
        self.main_ctrl.settings.save()
        self.main_ctrl.populate_output_roots()
        helper.fix_combo_menu_width(self.main_ctrl.view.combo_output_root)
        self.main_ctrl.view.combo_output_root.setCurrentIndex(0)
        self.main_ctrl.view.combo_output_root.blockSignals(False)

        self.view.close()


class AddPresetForSourcesController(object):
    """Drive an AddPresetForSources instance."""

    def __init__(self, view, main_ctrl):
        """Initialize the AddPresetForSourcesController instance.

        Args:
            view (smartMedia.mvc.view.AddPresetForSources): The view to drive.
            main_ctrl (smartMedia.mvc.controller.Controller): The main view
                controller to be able to communicate back to the main window.

        """
        self.view = view
        self.main_ctrl = main_ctrl

        self.populate()
        self.create_signals()

    def create_signals(self):
        """Create signals."""
        self.view.button_cancel.clicked.connect(lambda: self.view.reject())
        self.view.button_add.clicked.connect(lambda: self.assign_preset())

    def populate(self):
        """Populate the presets dropdown and sources table."""
        preset_names = sorted(list(self.main_ctrl.presets.keys()))
        self.view.combo_preset.addItems(preset_names)
        helper.fix_combo_menu_width(self.view.combo_preset)

    def assign_preset(self):
        """Assign the selected preset to all sources."""
        preset_name = self.view.combo_preset.currentText()

        for ctrl in self.main_ctrl.job_controller:
            task_ctrl = ctrl.new_default_task()
            task_ctrl.apply_preset_by_name(preset_name)

        self.view.accept()
