"""Provide general information about the preset to create."""

# Import third-party modules
try:
    from PySide import QtCore
    from PySide import QtGui
    from PySide import QtGui as QtWidgets
except ImportError:
    from PySide2 import QtCore
    from PySide2 import QtGui
    from PySide2 import QtWidgets


# Import local modules
from smartMedia import preset
from smartMedia.settings import Settings
from smartMedia.mvc import helper
from smartMedia.mvc.dialogs import message


class PagePresetInfo(QtWidgets.QWizardPage):
    """Provide general information about the preset to create."""

    def __init__(self, page_index, count_all_pages):
        """Provide general information about the preset to create.

        Args:
            page_index (int): The index of the current page shown.
            count_all_pages (int): The number of all pages in the wizard.

        """
        super(PagePresetInfo, self).__init__()

        self.page_index = page_index
        self.count_all_pages = count_all_pages

        self.settings = Settings()
        self.all_existing_presets = preset.load_all_presets()

        self.create_widgets()
        self.create_layouts()
        self.create_signals()
        self.register_fields()
        self.set_preset_root_visibility()

        helper.set_tooltips(self, "wizard_page_preset_info")

    def map_description(self):
        self.input_description_mapper.setText(
            self.input_description.toPlainText()
        )

    def create_widgets(self):
        """Create the page."""
        self.label_title = QtWidgets.QLabel(
            "Preset information (Page: {}/{})".format(
                self.page_index, self.count_all_pages)
        )
        self.label_title.setObjectName("page_title")

        self.label = QtWidgets.QLabel(
            "Tell me about the preset you want to create."
        )
        self.label.setWordWrap(True)

        label_min_width = 100

        self.label_name = QtWidgets.QLabel(
            "Preset name<span style='color: #33ccff'>*</span>: "
        )
        self.label_name.setMinimumWidth(label_min_width)
        self.input_name = QtWidgets.QLineEdit()
        helper.fix_no_outline(self.input_name)

        self.label_preset_root = QtWidgets.QLabel("Preset root:")
        self.label_preset_root.setMinimumWidth(label_min_width)
        self.combo_preset_root = QtWidgets.QComboBox()
        helper.fix_combo_solid_bg(self.combo_preset_root)
        self.combo_preset_root.addItems(self.settings["preset_paths"])
        helper.fix_combo_menu_width(self.combo_preset_root)

        self.input_preset_root_mapper = QtWidgets.QLineEdit()
        self.input_preset_root_mapper.setVisible(False)
        self.input_preset_root_mapper.setText(
            self.combo_preset_root.currentText()
        )

        self.input_description_mapper = QtWidgets.QLineEdit()
        self.input_description_mapper.setVisible(False)

        self.label_description = QtWidgets.QLabel("Description: ")
        self.label_description.setMinimumWidth(label_min_width)
        self.label_description.setAlignment(QtCore.Qt.AlignTop)
        self.input_description = QtWidgets.QTextEdit()

        self.input_description.textChanged.connect(lambda: self.map_description())

    def create_layouts(self):
        """Create layouts."""
        self.layout_name = QtWidgets.QHBoxLayout()
        self.layout_name.addWidget(self.label_name)
        self.layout_name.addWidget(self.input_name)

        self.layout_preset_root = QtWidgets.QHBoxLayout()
        self.layout_preset_root.addWidget(self.label_preset_root)
        self.layout_preset_root.addWidget(self.combo_preset_root)
        self.layout_preset_root.addWidget(self.input_preset_root_mapper)

        self.layout_description = QtWidgets.QHBoxLayout()
        self.layout_description.addWidget(self.label_description)
        self.layout_description.addWidget(self.input_description)
        self.layout_description.addWidget(self.input_description_mapper)

        self.layout_main = QtWidgets.QVBoxLayout()
        self.layout_main.addWidget(self.label_title)
        self.layout_main.addWidget(self.label)
        self.layout_main.addLayout(self.layout_name)
        self.layout_main.addLayout(self.layout_preset_root)
        self.layout_main.addLayout(self.layout_description)

        self.setLayout(self.layout_main)

    def create_signals(self):
        """Create signals."""
        self.combo_preset_root.currentIndexChanged.connect(
            lambda: self.update_preset_root_mapper()
        )
        self.input_name.textChanged.connect(
            lambda: self.remove_invalid_characters()
        )

    def remove_invalid_characters(self):
        """Remove any invalid characters live from the preset name input."""
        self.input_name.setText(self.input_name.text().replace(" ", "_"))

    def set_preset_root_visibility(self):
        """Set up the visibility of the preset root combo.

        The combo only needs to be shown when the user has set to read from
        more than one preset root. Then we give the user a chance to specify
        where to store the new preset root to.

        """
        for widget in (self.label_preset_root, self.combo_preset_root):
            widget.setVisible(len(self.settings["preset_paths"]) > 1)

    def update_preset_root_mapper(self):
        """Update the preset root mapper with the currently selected root."""
        self.input_preset_root_mapper.setText(
            self.combo_preset_root.currentText()
        )

    def register_fields(self):
        """Register fields."""
        self.registerField("preset_name", self.input_name)
        self.registerField("preset_info", self.input_description_mapper)
        self.registerField("preset_root", self.input_preset_root_mapper)

    def validatePage(self):
        """Override validation to ensure a preset name exists and is unique.

        Returns:
            bool: True when the user input is valid, False otherwise.

        """
        preset_name = self.input_name.text()
        if not preset_name:
            message.show(self, "Please enter a preset name")
            return False

        elif preset_name in self.all_existing_presets:
            message.show(
                self,
                "The preset name '{}' is already used, please use a "
                "different name.".format(preset_name)
            )
            return False

        return True
