"""Ask for time specific behavior."""

# Import third-party modules
try:
    from PySide import QtCore
    from PySide import QtGui
    from PySide import QtGui as QtWidgets
except ImportError:
    from PySide2 import QtCore
    from PySide2 import QtGui
    from PySide2 import QtWidgets

# Import local modules
from smartMedia.constants import PATTERN_ALL_NUMBERS
from smartMedia.mvc import helper


class PageTime(QtWidgets.QWizardPage):
    """Ask for time specific behavior."""

    def __init__(self, page_index, count_all_pages):
        """Ask for time specific behavior.

        Args:
            page_index (int): The index of the current page shown.
            count_all_pages (int): The number of all pages in the wizard.

        """
        super(PageTime, self).__init__()

        self.page_index = page_index
        self.count_all_pages = count_all_pages

        self.create_widgets()
        self.create_layouts()
        self.create_signals()
        self.register_fields()

        helper.set_tooltips(self, "wizard_page_time")

    def create_widgets(self):
        """Create the page."""
        self.label_title = QtWidgets.QLabel(
            "Time options (Page: {}/{})".format(
                self.page_index, self.count_all_pages)
        )
        self.label_title.setObjectName("page_title")

        self.label_page_description = QtWidgets.QLabel(
            "Do you need to apply anything time related?"
        )

        self.label_frame_range = QtWidgets.QLabel("Frame range for output")
        self.combo_frame_range = QtWidgets.QComboBox()
        self.combo_frame_range.addItems(["same as source", "start at"])
        helper.fix_combo_solid_bg(self.combo_frame_range)
        self.input_frame_start = QtWidgets.QLineEdit()
        helper.fix_no_outline(self.input_frame_start)
        self.input_frame_start.setVisible(False)
        self.input_frame_start.setValidator(
            QtGui.QRegExpValidator(QtCore.QRegExp(PATTERN_ALL_NUMBERS))
        )
        self.input_frame_range_mapper = QtWidgets.QLineEdit()
        self.input_frame_range_mapper.setText(
            self.combo_frame_range.currentText()
        )
        self.input_frame_range_mapper.setVisible(False)

    def create_layouts(self):
        """Create layouts."""
        self.layout_main = QtWidgets.QVBoxLayout()

        self.layout_main.addWidget(self.label_title)
        self.layout_main.addWidget(self.label_page_description)

        self.layout_frame_range = QtWidgets.QHBoxLayout()
        self.layout_frame_range.addWidget(self.label_frame_range)
        self.layout_frame_range.addWidget(self.combo_frame_range)
        self.layout_frame_range.addWidget(self.input_frame_start)
        self.layout_frame_range.addStretch()

        self.layout_main.addLayout(self.layout_frame_range)
        self.layout_main.addWidget(self.input_frame_range_mapper)

        self.setLayout(self.layout_main)

    def create_signals(self):
        """Create signals."""
        self.combo_frame_range.currentIndexChanged.connect(
            lambda: self.set_start_input_visibility()
        )
        self.combo_frame_range.currentIndexChanged.connect(
            lambda: self.update_frame_range_mapper()
        )

    def update_frame_range_mapper(self):
        """Update the frame range mapper."""
        self.input_frame_range_mapper.setText(
            self.combo_frame_range.currentText()
        )

    def set_start_input_visibility(self):
        """Set the visibility of the frame start input."""
        self.input_frame_start.setVisible(
            self.combo_frame_range.currentText() == "start at"
        )

    def register_fields(self):
        """Register fields."""
        self.registerField("source_frame_mode", self.input_frame_range_mapper)
        self.registerField("source_start", self.input_frame_start)
