"""Gui related helper functionality."""

# Import built-in modules
import os

# PySide import switch
try:
    from PySide import QtGui as QtWidgets
    from PySide import QtGui
    from PySide import QtCore
except ImportError as error:
    from PySide2 import QtWidgets
    from PySide2 import QtGui
    from PySide2 import QtCore

# Import local modules
from smartMedia.mvc.tooltips import TOOLTIPS


def load_icons():
    """Load all icons from the icons folder.

    This scans the icons directory and creates an icon dictionary using the
    file names without extension as keys.

    Returns:
        dict: Dictionary of icons in the format:
            {
                "icon1": "path1",
                "icon2": "path2",
                "icon3": "path3",
                ...
            }

    """
    this_dir = os.path.dirname(__file__)
    dir_icon = os.path.join(this_dir, "..", "icons")
    dir_icon = os.path.normpath(dir_icon)

    icons = {}
    for file_ in os.listdir(dir_icon):
        name = os.path.splitext(file_)[0]
        path = os.path.join(dir_icon, file_)
        icons[name] = path

    return icons


_ICONS = load_icons()
def set_icon(widget, name):
    """Update the icon of the given widget to the given path.

    Args:
        widget (QtWidgets.QWidget): Widget to update the icon on.
        name (str): Name of the icon to use. Needs to be located in the
            package's icons folder. File extension will be found automatically.

    """
    widget.setIcon(QtGui.QIcon(_ICONS[name]))


def clear_layout(layout):
    for i in reversed(range(layout.count())):
        widgetToRemove = layout.itemAt(i).widget()
        # remove it from the layout list
        layout.removeWidget(widgetToRemove)
        # remove it from the gui
        widgetToRemove.setParent(None)


def set_style_sheet(widget, style="styles.qss"):
    """Apply css style sheet to given widget.

    Args:
        widget (QtWidgets.QWidget): Widget to apply styles to.
        style (str): Name of styles file to apply.

    """
    this_dir = os.path.join(os.path.dirname(__file__))
    styles = os.path.join(this_dir, "..", "styles", style)
    styles = os.path.normpath(styles)

    with open(styles) as file_:
        widget.setStyleSheet(file_.read())


def combo_set_text(combo, text):
    """Set the index of the given text of the given combo.

    Args:
        combo (QtWidgets.QComboBox): ComboBox to set the current drop down menu
            entry to.
        text (str): Text to set as active entry.

    """
    combo.setCurrentIndex(combo.findText(text))


def set_tooltips(widget, section):
    """Set tooltips for the widget within the given section.

    Args:
        widget (QtWidgets.QWidget): Widget to assign tooltips to.
        section (str): Name of the section that contains tooltips for the
            given widget.

    """
    def set_tooltip(name_):
        widget_ = getattr(widget, name_)
        if not widget_:
            return
        widget_.setToolTip(tooltip)

    for name, tooltip in TOOLTIPS[section].items():
        if isinstance(name, str):
            set_tooltip(name)

        elif isinstance(name, tuple):
            for name_ in name:
                set_tooltip(name_)


def move_widget_under_cursor(widget, center_window=False, offset=0):
    """Move given widget under the current cursor position.

    Args:
        widget (QtWidgets.QWidget): The widget to move to the current cursor
            position.
        center_window (bool, optional): If True perform an additional centering
            of the widget to the current cursor location, otherwise leave the
            widget's top left corner at the current cursor position.

    """
    position = QtGui.QCursor().pos()

    if center_window:
        window = QtCore.QPoint(0.5 * widget.width(), 0.5 * widget.height())
        position -= window

    elif offset:
        offset_ = QtCore.QPoint(offset, offset)
        position += offset_

    widget.move(position)


def force_move_on_top(window):
    """Force move a window on top of all other windows.

    Notes:
        Just setting the QtCore.Qt.WindowStaysOnTopHint does not automatically
        set a window onto another window when the other window has the flag
        set as well. Using this function we can force it to be on top of the
        other.
        Drawn from: https://stackoverflow.com/a/56895345/4919733

    Args:
        window (QtWidgets.QWidget): The window to move to the top.

    """
    window.setWindowState(
        window.windowState()
        & ~QtCore.Qt.WindowMinimized
        | QtCore.Qt.WindowActive
    )
    window.activateWindow()


def force_update_view():
    """Force updating the view, releasing the app lock."""
    # Process this multiple times because just processing it once sometimes
    # runs in not releasing the app lock.
    for _ in range(3):
        QtCore.QCoreApplication.processEvents()


def fix_combo_solid_bg(combo):
    """Make fully grey background, remove border at the top and bottom.

    Args:
        combo (QtWidgets.QComboWidget): The combo widget to adjust in style.

    """
    combo.view().parentWidget().setStyleSheet(
        "background-color: #333333;"
    )


def fix_no_outline(widget):
    """Show no outline on widget when it has focus.

    Args:
        widget (QtWidgets.QWidget): The widget to remove the outline on.

    """
    widget.setAttribute(QtCore.Qt.WA_MacShowFocusRect, 0)


def darken_context_menu(context_menu):
    """Darken the given context menu.

    Adding the very same style via a qss files does not apply the style.
    Applying it dynamically works.

    Args:
        context_menu (QtWidgets.QMenu): The menu to darken.

    """
    context_menu.setStyleSheet(
        """
        QMenu { background-color: #232323; color: #999999; }
        QMenu::item::selected { background-color: #191919; color: #567089; }
        """
    )


def fix_combo_menu_width(combo):
    """Ensure all items are completely visible in dropdown and won't cut off.

    Args:
        combo (QtWidgets.QComboBox): The dropdown menu to ensure all items
            are completely shown.

    """
    choices = [
        combo.itemText(index)
        for index in range(combo.count())
    ]

    width = combo.fontMetrics().boundingRect(max(choices, key=len)).width()
    additional_offset = 30
    combo.view().setFixedWidth(width + additional_offset)


# Menu bar styles. For some reason, they cannot be set via the styles.qss file
# but must be set via setStyleSheet.
MENUBAR_STYLES = """
QMenuBar
    {
        background-color: #232323;
        color: #999;
        border-bottom: 1px solid #333333;
        padding-top: 5px;
        padding-left: 5px;
    }
    QMenuBar::item
    {
        background-color: #232323;
        color: #999;
    }
    QMenuBar::item::selected
    {
        background-color: #232323;
        color: #fff;
    }
    QMenu
    {
        background-color: #232323;
        color: #fff;
    }
    QMenu::item::selected
    {
        background-color: #232323;
        color: #33ccff;
    }
"""
