"""Create job data in separate processing threads."""

# Import built-in modules
try:
    # Python-3
    import queue as Queue
except ImportError:
    # Python-2
    import Queue
import threading
import time


class Processor(object):
    """Ingest data to disk and the elements browser."""

    def __init__(self, num_threads=4, controller=None):
        """Initialize the Processor instance.

        Args:
            num_threads (int, optional): The number of ingesting threads to
                spawn. If not provided we span 4 parallel threads.
            controller (smartMedia.mvc.controller.Controller, optional): The
                controller to communicate back to the view.

        """
        self.num_threads = num_threads
        self.controller = controller

        self.queue = Queue.Queue()
        self.workers = None
        self.created_workers = False
        self.num_jobs = 0

        self._create_workers()

    def add_job(self, job_):
        """Add the given job to the queue.

        Args:
            job_ (smartMedia.job.Job): The job to add to the queue.

        """
        # Create workers lazily and only when we add a job.
        if not self.created_workers:
            self._create_workers()
            self.created_workers = True

        self.queue.put(job_)
        self.num_jobs += 1

        if self.controller:
            self.controller.status(
                "Ingesting ({}) media".format(self.num_jobs)
            )

    def _process_job(self, index):
        """Process the next job.

        Args:
            index (int): Index of the processing thread.

        """
        while True:
            if self.queue.empty():
                # Heads up, we need to wait a bit in Nuke, otherwise the
                # threads will freeze Nuke. Probably because they will
                # immediately start over. This seems to be only a problem
                # in Nuke, not when running standalone.
                time.sleep(3)

                continue

            job__ = self.queue.get()
            job__.run(wait=True)

            self.queue.task_done()

            self.decrease_num_jobs()

            if self.controller:
                # Add the job to the UI.
                self.controller.view.load_job_by_root.emit(job__.job_root)
                self.controller.view.create_auto_tasks.emit(job__.job_root)

                message = ""
                if self.num_jobs > 0:
                    message = "Ingesting ({}) media".format(self.num_jobs)

                self.controller.status(message)

    def decrease_num_jobs(self):
        """Decrease num jobs and set the status bar visibility."""
        self.num_jobs -= 1
        if self.num_jobs < 0:
            self.num_jobs = 0

    def _create_workers(self):
        """Create worker threads."""
        for index in range(self.num_threads):
            worker = threading.Thread(target=self._process_job, args=(index,))
            worker.setDaemon(True)
            worker.start()
