"""Execute custom code before opening the task's template.nk file.

Any code in the main function of this module will be executed right after
having loaded the task's template.nk file but before applying any data
translation and before rendering.

Each pre_render.py should have at least one function called `main`. This is
the entry point for logic injection. You can build any amount of additional
functions or calls to other modules or python packages (internal or external)
around this to have absolute control over a task and do whatever you want
and need. The main function contains two arguments - the incoming data key
value pairs of your task and a logger that you can use for logging purposes.
Logs appear in the job log.

"""

# Import built-in modules
import os

# Import third-party modules
import nuke


def main(data, logger):
    """This is the main entry point for pre render code injections.

    Args:
        data (dict): This is the key-value pairs of the task which can be used
            to make some last minute changes before rendering the task.
        logger (logging.Logger): The logger to use for logging to std out.

    """
    logger.info(
        "Hello, I am executing custom code before opening the "
        "task's template.nk file."
    )

    # TODO Add any code here to execute before rendering.

    # Let's set up some environment variables which get picked up in the
    # working file to render.
    os.environ["SOMETHING"] = "hello-world"
    os.environ["ELSE"] = "Jupiter and Mars"

    # Force using a slate no matter what the user sets in the UI.
    data["use_slate"] = True

    # Ensure missing frames in the source don't raise any error but rather
    # show a checkerboard because the source might not be ready from 3D
    # department but we already want to see all frames that exist.
    name = "smartMedia_source"
    node = nuke.toNode(name)
    logger.info("Node name '%s' exists: %s", name, bool(node))
    if node:
        logger.info("Updating source's on_error to checkerboard...")
        node["on_error"].setValue("checkerboard")

    logger.info(
        "I am done executing custom code before opening the task's "
        "template.nk file."
    )
