"""Represent on media element.

This module contains the Media class that will automatically detect the
full sequence of a given image path. This will use a regex to split of the
frame padding of a given path and glob for files with the same name but a
different file sequence number in order to get the full file sequence. It will
also provide information about the first- and last frame as well as frames that
are missing in the sequence.

Usage:
    >>> from smartMedia import Media
    >>> media = Media("path/to/image.1001.exr")

    # We can get the hash padding path using a hash padding instead of a fixed
    # counter.
    >>> media.path
    'path/to/image.####.exr'

    # Let's print all image sequence files for the given path.
    >>> media.files
    [
        'path/to/image.1001.exr',
        'path/to/image.1002.exr',
        'path/to/image.1003.exr',
        ...
        'path/to/image.1010.exr',
    ]

    # Let's print the first sequence number of our image sequence.
    >>> media.first
    1001

    # Let's print the last sequence number of our image sequence.
    >>> media.last
    1010

    # Let's get information about the file size of all images.
    >>> media.get_size()
    17.332 MB

    # After some files have changed we can perform a rescan. This will
    # calculate all the corresponding files of the image sequnce as well as the
    # first and last frame again.
    >>> media.scan()

"""

# Import built-in modules
import glob
import os
import re

# Import local modules
from smartMedia.constants import FILE_SEQUENCE_PATTERN
from smartMedia.constants import VIDEO_EXTENSIONS


# We want to keep the member calculation private as the user doesn't need to
# handle these calculation steps themselves.
# pylint: disable=too-few-public-methods
class Media(object):
    """Represent on media element."""

    def __init__(self, path, autoscan=True):
        """Initialize the SequenceFinder instance.

        Args:
            path (str): Absolute path of file to search for file sequence.
            autoscan (bool, optional): If True scan for files immediately on
                initialization. Otherwise, don't perform a scan and leave it to
                the caller.

        """
        self._path = path
        self.is_sequence = None
        self.path = self.hash_path(path)

        self.files = []
        self.first = 0
        self.last = 0
        self.size = 0.0

        if autoscan:
            self.scan()

    def hash_path(self, path):
        """Get the path using a hash pattern.

        Notes:
            Make sure to always return a sanitized path with forward slashes
            as Nuke does not understand file paths with backward slashes which
            appear under windows.

        Returns:
            str: The path using a hash pattern.

        """
        try:
            if os.path.splitext(self._path)[1] in VIDEO_EXTENSIONS:
                raise ValueError("")
            filename_base, counter, extension = self._split_groups()
            self.is_sequence = True
            hash_ = "#" * len(str(counter))
            path = "{}{}{}".format(filename_base, hash_, extension)
        except ValueError:
            self.is_sequence = False

        return path.replace("\\", "/")

    @property
    def base(self):
        """Get the base, meaning everything up to the counter.

        Notes:
            if the media is no sequence, meaning it does not contain any
            counter then we will get the full name up to the file extension.

        Returns:
            str: The base, meaning everything up to the counter.

        """
        if not self.is_sequence:
            return os.path.splitext(os.path.basename(self.path))[0]

        base = os.path.basename(self._split_groups()[0])
        if base.endswith("."):
            base = base[:-1]
        return base

    @property
    def sequence_path(self):
        """Get the media's sequence path, i.e. the root and base.

        Returns:
            str: The media's sequence path, i.e. the root and base.

        """
        return os.path.join(os.path.dirname(self.path), self.base)

    def scan(self):
        """Perform scan process and search for all images of the sequences."""
        if self.is_sequence:
            self.files = self._scan()
        else:
            # When we are dealing with a video then we only want to include
            # the one path. i.e. the video, to the files. This prevents us
            # from adding several videos that are named equally with an
            # increasing counter. This happens for instance with VCP's
            # ActionEssentials: Atmosphere_01.mov, Atmosphere_02.mov,
            # Atmosphere_03.mov, etc. Otherwise we would be adding all the
            # video clips into the files member.
            self.files = [self.path]

        if self.files:
            self.first = self._get_frame(self.files[0])
            self.last = self._get_frame(self.files[-1])

    def _split_groups(self):
        """Split groups for file path to file name base, padding and ext.

        These values are processed further in order to detect the full file
        sequence of the path member.

        Returns:
            str, int, str: The file name base, the file sequence number and the
                file extension from the path member.

        Raises:
            ValueError: When the file sequence pattern doesn't match due to the
                fact that the path member does not contain a file sequence
                number.

        """
        match = re.search(FILE_SEQUENCE_PATTERN, self._path)
        if not match:
            raise ValueError("File path does not contain a file sequence "
                             "number.")

        # The counter could be an integer, e.g. 1001, however, we also support
        # %XXd and # patterns, e.g. %04d and ####. In that case we can't cast
        # the string to an integer and instead, we pass the raw counter to the
        # return value.
        counter = match.group(2)

        return match.group(1), counter, match.group(3)

    def _scan(self):
        """Glob for files with the file path structure of the path member.

        This will use the split file name, file sequence number and file
        extension to create a glob pattern where the file sequence number is
        variable.

        Returns:
            list: Absolute file paths of matching file sequences, sorted
                alphabetically.

        """
        try:
            filename_base, _, extension = self._split_groups()
        except ValueError:
            return [self._path]

        pattern = "{}*{}".format(filename_base, extension)

        return sorted(path for path in glob.glob(pattern))

    def _get_frame(self, path):
        """Extract the file sequence number of the given path.

        Args:
            path (str): Path to extract file sequence number of.

        Returns:
            int: the extracted file sequence number if the given path contains
                one, otherwise 1.

        """
        match = re.search(FILE_SEQUENCE_PATTERN, path)
        if not match:
            return 1
        try:
            return int(match.group(2))
        except ValueError:
            return 1

    def get_size(self):
        """Get the overall size of the media including all frames.

        Returns:
            double: The Size of the media including all frames.

        """
        size = 0.0
        for path in self.files:
            size += os.path.getsize(path)
        return size


def human_readable_size(size, decimal_places=3):
    """Get the size in the nearest unit in human readable format.

    Notes:
        Taken from: https://stackoverflow.com/a/43690506/4919733

    Args:
        size (double): The number of bytes to convert to human readable.
        decimal_places (int, optional): The number of decimal places to use.
            Defaults to 3.

    Returns:
        str: The human readable size from the given size.

    """
    for unit in ['B','KiB','MiB','GiB','TiB']:
        if size < 1024.0:
            break
        size /= 1024.0

    return "{size:.{decimal_places}f}{unit}".format(
        size=size, decimal_places=decimal_places, unit=unit
    )
