"""Provide SpinBoxes to update the target frame range."""

# Import third-party modules
try:
    from PySide import QtCore
    from PySide import QtGui as QtWidgets
except ImportError:
    from PySide2 import QtCore
    from PySide2 import QtGui
    from PySide2 import QtWidgets

# Import local modules
from smartMedia.mvc import helper
from smartMedia.mvc.details_widgets.base import Base


class SpinboxRangeTarget(Base):
    """Provide SpinBoxes to update the target frame range."""

    def create_widgets(self):
        """Create widgets."""
        super(SpinboxRangeTarget, self).create_widgets()

        self.label_0 = QtWidgets.QLabel(self.label_text)
        self.label_0.setMinimumWidth(120)

        self.combo_mode = QtWidgets.QComboBox()
        helper.fix_combo_solid_bg(self.combo_mode)
        self.combo_mode.addItems([
            "same as source",
            "custom",
        ])
        if self.preset["target_frame_mode"]:
            self.combo_mode.setCurrentText(self.preset["target_frame_mode"])

        self.label_1 = QtWidgets.QLabel(" - ")

        self.spin_start = QtWidgets.QSpinBox()
        self.spin_end = QtWidgets.QSpinBox()

        index = 0
        for widget in (self.spin_start, self.spin_end):
            widget.setMinimum(-9999999)
            widget.setMaximum(9999999)
            widget.setMinimumWidth(100)
            index += 1

        self.set_range_editable()
        self.set_range_values()

    def create_layouts(self):
        """Create layouts."""
        super(SpinboxRangeTarget, self).create_layouts()

        self.layout.addWidget(self.label_0)
        self.layout.addWidget(self.combo_mode)
        self.layout.addWidget(self.spin_start)

        self.layout.addWidget(self.label_1)
        self.layout.addWidget(self.spin_end)

        self.layout.addStretch()

    def create_signals(self):
        """Create signals."""
        super(SpinboxRangeTarget, self).create_signals()

        self.spin_start.valueChanged.connect(lambda: self.update())
        self.spin_end.valueChanged.connect(lambda: self.update())
        self.combo_mode.currentIndexChanged.connect(lambda: self.set_range_values())
        self.combo_mode.currentIndexChanged.connect(lambda: self.set_range_editable())
        self.combo_mode.currentIndexChanged.connect(lambda: self.update())

    def set_range_values(self):
        """Set up the ranges based on mode."""
        mode = self.combo_mode.currentText()
        if mode == "same as source":
            self.spin_start.setValue(self.preset["source_start"])
            self.spin_end.setValue(self.preset["source_end"])
        elif mode == "custom":
            self.spin_start.setValue(self.preset["target_start"])
            self.spin_end.setValue(self.preset["target_end"])

    def set_range_editable(self):
        """Set the spinboxes editable based on mode."""
        current_mode = self.combo_mode.currentText()
        self.spin_start.setEnabled(current_mode in {"custom", "start at"})
        self.spin_end.setEnabled(current_mode == "custom")

    def update(self):
        """Update the preset and thus the task's data.json"""
        self.preset[self.keys[0]] = self.spin_start.value()
        self.preset[self.keys[1]] = self.spin_end.value()
        self.preset["target_frame_mode"] = self.combo_mode.currentText()
        self.preset.save()
