"""Show a dialog to add a new format."""

# Import third-party modules
try:
    from PySide import QtCore
    from PySide import QtGui as QtWidgets
    from PySide import QtGui
except ImportError:
    from PySide2 import QtCore
    from PySide2 import QtGui
    from PySide2 import QtWidgets

# Import local modules
from smartMedia.constants import PATTERN_ALL_NUMBERS
from smartMedia.constants import PATTERN_FLOATS
from smartMedia.mvc import helper


class NewFormatDialog(QtWidgets.QDialog):
    """Show a dialog to add a new format."""

    def __init__(self):
        """Initialize the NewFormatDialog instance."""
        super(NewFormatDialog, self).__init__()

        self.label_min_width = 100

        self.setup()
        self.create_widgets()
        self.set_validators()
        self.create_layouts()
        self.create_signals()

    @classmethod
    def show(cls):
        """Show a dialog and return entered values.

        Returns:
            dict: The entered values in the format:
            {
                "name": <entered name>,
                "width": <entered width>,
                "height": <entered height>,
                "pixel_aspect": <entered pixel aspect ratio>,
            }

        """
        obj = NewFormatDialog()

        if obj.exec_() == QtWidgets.QDialog.Accepted:
            return {
                "name": obj.input_name.text(),
                "width": int(obj.input_width.text()),
                "height": int(obj.input_height.text()),
                "pixel_aspect": float(obj.input_pixel_aspect.text()),
            }

    def setup(self):
        """Setup UI."""
        self.setMinimumWidth(400)
        self.setWindowTitle("Create new format")
        self.setModal(True)

    def create_widgets(self):
        """Create widgets."""
        self.label_name = QtWidgets.QLabel("Name")
        self.label_name.setMinimumWidth(self.label_min_width)
        self.input_name = QtWidgets.QLineEdit()
        self.label_resolution = QtWidgets.QLabel("Resolution")
        self.label_resolution.setMinimumWidth(self.label_min_width)
        self.label_width = QtWidgets.QLabel("w")
        self.input_width = QtWidgets.QLineEdit()
        self.label_height = QtWidgets.QLabel("h")
        self.input_height = QtWidgets.QLineEdit()
        self.label_pixel_aspect = QtWidgets.QLabel("Pixel Aspect")
        self.label_pixel_aspect.setMinimumWidth(self.label_min_width)
        self.input_pixel_aspect = QtWidgets.QLineEdit("1")
        self.label_error = QtWidgets.QLabel()
        self.label_error.setProperty("style", "error")

        self.button_create = QtWidgets.QPushButton("Create")
        self.button_create.setProperty("style", "action_button")
        self.button_cancel = QtWidgets.QPushButton("Cancel")

        helper.fix_no_outline(self.input_name)
        helper.fix_no_outline(self.input_width)
        helper.fix_no_outline(self.input_height)
        helper.fix_no_outline(self.input_pixel_aspect)

    def set_validators(self):
        """Set validators for various inputs."""
        # Allow only certain name characters that are also allowed in Nuke.
        # https://regex101.com/r/1xkIdy/1
        pattern = r"[a-zA-Z0-9\._\(\)\-]+"
        self.input_name.setValidator(
            QtGui.QRegExpValidator(QtCore.QRegExp(pattern))
        )

        # Allow all numbers for width and height widget.
        for widget in (self.input_width, self.input_height):
            widget.setValidator(
                QtGui.QRegExpValidator(QtCore.QRegExp(PATTERN_ALL_NUMBERS))
            )

        # Allow numbers and floating numbers for pixel aspect.
        self.input_pixel_aspect.setValidator(
            QtGui.QRegExpValidator(QtCore.QRegExp(PATTERN_FLOATS))
        )

    def create_layouts(self):
        """Create layouts."""
        self.layout_main = QtWidgets.QVBoxLayout()
        self.layout_name = QtWidgets.QHBoxLayout()
        self.layout_resolution = QtWidgets.QHBoxLayout()
        self.layout_pixel_aspect = QtWidgets.QHBoxLayout()
        self.layout_button = QtWidgets.QHBoxLayout()

        self.layout_name.addWidget(self.label_name)
        self.layout_name.addWidget(self.input_name)

        self.layout_resolution.addWidget(self.label_resolution)
        self.layout_resolution.addWidget(self.label_width)
        self.layout_resolution.addWidget(self.input_width)
        self.layout_resolution.addWidget(self.label_height)
        self.layout_resolution.addWidget(self.input_height)

        self.layout_pixel_aspect.addWidget(self.label_pixel_aspect)
        self.layout_pixel_aspect.addWidget(self.input_pixel_aspect)

        self.layout_button.addWidget(self.button_create)
        self.layout_button.addWidget(self.button_cancel)

        self.layout_main.addLayout(self.layout_name)
        self.layout_main.addLayout(self.layout_resolution)
        self.layout_main.addLayout(self.layout_pixel_aspect)
        self.layout_main.addWidget(self.label_error)
        self.layout_main.addLayout(self.layout_button)

        self.setLayout(self.layout_main)
        helper.set_style_sheet(self)

    def create_signals(self):
        """Create signals."""
        self.button_cancel.clicked.connect(lambda: self.reject())
        self.button_create.clicked.connect(lambda: self.check_and_accept())

    def check_and_accept(self):
        """Check inputs and accept."""
        error = ""
        if not self.input_name.text():
            error = "Please enter a name"

        elif not self.input_width.text():
            error = "Please enter a width"

        elif not self.input_height.text():
            error = "Please enter a height"

        elif not self.input_pixel_aspect.text():
            error = "Please enter a pixel aspect"

        if error:
            self.label_error.setText(error)
            return

        self.accept()
