"""Ask for using code injections."""

# Import third-party modules
try:
    from PySide import QtCore
    from PySide import QtGui
    from PySide import QtGui as QtWidgets
except ImportError:
    from PySide2 import QtCore
    from PySide2 import QtGui
    from PySide2 import QtWidgets

# Import local modules
from smartMedia.mvc import helper


class PageCodeInjections(QtWidgets.QWizardPage):
    """Ask for using code injections."""

    def __init__(self, page_index, count_all_pages):
        """Ask for using code injections.

        Args:
            page_index (int): The index of the current page shown.
            count_all_pages (int): The number of all pages in the wizard.

        """
        super(PageCodeInjections, self).__init__()

        self.page_index = page_index
        self.count_all_pages = count_all_pages

        self.create_widgets()
        self.create_layouts()
        self.create_signals()
        self.register_fields()

    def create_widgets(self):
        """Create the page."""
        self.label_title = QtWidgets.QLabel(
            "Injecting custom code (Page: {}/{})".format(
                self.page_index, self.count_all_pages)
        )
        self.label_title.setObjectName("page_title")

        self.label_page_description = QtWidgets.QLabel(
            "Do you want to inject custom Python code into your presets? "
            "Applying code injections here will copy blueprint python modules "
            "into the preset that you can then fill up with any logic."
        )
        self.label_page_description.setWordWrap(True)

        self.label_page_description_more = QtWidgets.QLabel(
            "For more information about injecting custom code please have a "
            "read on<br /><a href=\"https://www.cragl.com/smartMediaInjectCode\">"
            "<span style=\"color:#33ccff;\">"
            "https://www.cragl.com/smartMediaInjectCode</span></a>"
        )
        self.label_page_description_more.setWordWrap(True)

        #self.label_page_description_more.setTextFormat(QtCore.Qt.RichText)
        self.label_page_description_more.setTextInteractionFlags(
            QtCore.Qt.TextBrowserInteraction)

        self.label_page_description_more.setOpenExternalLinks(True)

        self.combo_ask = QtWidgets.QComboBox()
        self.combo_ask.addItems(["no", "yes"])
        helper.fix_combo_solid_bg(self.combo_ask)

        self.group_data = QtWidgets.QGroupBox()
        self.group_data.setVisible(False)
        self.check_data = QtWidgets.QCheckBox(" data.py")
        self.label_data = QtWidgets.QLabel(
            "Inject custom code when loading a preset and when reloading it."
        )

        self.group_pre_render = QtWidgets.QGroupBox()
        self.group_pre_render.setVisible(False)
        self.check_pre_render = QtWidgets.QCheckBox(" pre_render.py")
        self.label_pre_render = QtWidgets.QLabel(
            "Inject custom code before rendering the task.")

        self.group_post_render = QtWidgets.QGroupBox()
        self.group_post_render.setVisible(False)
        self.check_post_render = QtWidgets.QCheckBox(" post_render.py")
        self.label_post_render = QtWidgets.QLabel(
            "Inject custom code after the task rendering has finished.")

    def create_layouts(self):
        """Create layouts."""
        self.layout_main = QtWidgets.QVBoxLayout()

        self.layout_ask = QtWidgets.QHBoxLayout()

        self.layout_main.addWidget(self.label_title)
        self.layout_ask.addWidget(self.label_page_description)
        self.layout_ask.addWidget(self.combo_ask)

        self.layout_main.addLayout(self.layout_ask)

        self.layout_checks = QtWidgets.QVBoxLayout()

        self.layout_data = QtWidgets.QVBoxLayout()
        self.group_data.setLayout(self.layout_data)
        self.layout_data.addWidget(self.check_data)
        self.layout_data.addWidget(self.label_data)

        self.layout_pre_render = QtWidgets.QVBoxLayout()
        self.group_pre_render.setLayout(self.layout_pre_render)
        self.layout_pre_render.addWidget(self.check_pre_render)
        self.layout_pre_render.addWidget(self.label_pre_render)

        self.layout_post_render = QtWidgets.QVBoxLayout()
        self.group_post_render.setLayout(self.layout_post_render)
        self.layout_post_render.addWidget(self.check_post_render)
        self.layout_post_render.addWidget(self.label_post_render)

        self.layout_main.addWidget(self.group_data)
        self.layout_main.addWidget(self.group_pre_render)
        self.layout_main.addWidget(self.group_post_render)
        self.layout_main.addStretch()
        self.layout_main.addWidget(self.label_page_description_more)

        self.setLayout(self.layout_main)

    def create_signals(self):
        """Create signals."""
        self.combo_ask.currentIndexChanged.connect(
            lambda: self.toggle_checks_visbility()
        )

    def toggle_checks_visbility(self):
        """Toggle the visibility of the code injection checkboxes."""
        groups = (
            self.group_data,
            self.group_pre_render,
            self.group_post_render,
        )
        for group in groups:
            group.setVisible(self.combo_ask.currentText() == "yes")

    def register_fields(self):
        """Register fields."""
        self.registerField("code_inject_data", self.check_data)
        self.registerField("code_inject_pre_render", self.check_pre_render)
        self.registerField("code_inject_post_render", self.check_post_render)
