"""The settings widget that stores information about the settings."""

# Import third-party modules
try:
    from PySide import QtCore
    from PySide import QtGui as QtWidgets
except ImportError:
    from PySide2 import QtCore
    from PySide2 import QtGui
    from PySide2 import QtWidgets

# Import local modules
from smartMedia.mvc import helper
from smartMedia.mvc import widgets


class SettingsWidget(QtWidgets.QWidget):
    """The settings widget that stores information about the settings."""

    def __init__(self):
        """Initialize the Settings instance."""
        super(SettingsWidget, self).__init__()

        self.create_widgets()
        self.create_layouts()

    def create_widgets(self):
        """Create widgets."""
        self._create_widgets_general()
        self._create_widgets_paths()
        self._create_widgets_naming()

        self.button_save = QtWidgets.QPushButton("Save settings")
        self.button_save.setProperty("style", "action_button")

    def _create_widgets_general(self):
        """Create widgets general section."""
        self.check_show_tooltips = QtWidgets.QCheckBox(" Show tooltips")
        self.check_always_on_top = QtWidgets.QCheckBox(" Always on top")
        self.check_auto_load_last_workfile = QtWidgets.QCheckBox(
            " Auto load last workfile"
        )
        self.label_number_recent_files = QtWidgets.QLabel("Number of recent files")
        self.label_number_recent_files.setProperty("style", "settings_label_spin")
        self.spin_number_recent_files = QtWidgets.QSpinBox()
        self.spin_number_recent_files.setMinimum(1)
        self.spin_number_recent_files.setMaximum(99)
        helper.fix_no_outline(self.spin_number_recent_files)

        self.label_ingest_threads = QtWidgets.QLabel(
            "Number of ingest threads"
        )
        self.label_ingest_threads.setProperty("style", "settings_label_spin")
        self.spin_ingest_threads = QtWidgets.QSpinBox()
        self.spin_ingest_threads.setMinimum(1)
        self.spin_ingest_threads.setMaximum(99)
        helper.fix_no_outline(self.spin_ingest_threads)

        self.label_logging_level = QtWidgets.QLabel("Logging level")
        self.label_logging_level.setProperty("style", "settings_label")
        self.combo_logging_level = QtWidgets.QComboBox()
        helper.fix_combo_solid_bg(self.combo_logging_level)
        self.combo_logging_level.addItems([
            "debug", "info", "warning", "error",
        ])

        self.label_new_jobs_behavior = QtWidgets.QLabel("New jobs go to ")
        self.label_new_jobs_behavior.setProperty("style", "settings_label")
        self.combo_new_jobs_behavior = QtWidgets.QComboBox()
        helper.fix_combo_solid_bg(self.combo_new_jobs_behavior)
        self.combo_new_jobs_behavior.addItems(["top", "bottom"])

        self.label_thumbnail_index = QtWidgets.QLabel("Use job thumbnail")
        self.label_thumbnail_index.setProperty("style", "settings_label")
        self.combo_thumbnail_index = QtWidgets.QComboBox()
        helper.fix_combo_solid_bg(self.combo_thumbnail_index)
        self.combo_thumbnail_index.addItems([
            "first", "middle", "last",
        ])
        self.label_preset_dropdown_type = QtWidgets.QLabel(
            "Preset dropdown"
        )
        self.label_preset_dropdown_type.setProperty("style", "settings_label")
        self.combo_preset_dropdown_type = QtWidgets.QComboBox()
        self.combo_preset_dropdown_type.addItems(
            ["completer", "no completer"]
        )
        helper.fix_combo_solid_bg(self.combo_preset_dropdown_type)

    def _create_widgets_paths(self):
        """Create widgets paths section."""
        self.group_nuke_executables = QtWidgets.QGroupBox("Nuke executables")
        self.group_nuke_executables.setProperty("style", "with_header")
        self.table_nuke_executables = widgets.SimpleTable()

        self.group_presets = QtWidgets.QGroupBox("Presets root directories")
        self.group_presets.setProperty("style", "with_header")
        self.table_presets = widgets.SimpleTable()

        self.group_nodes = QtWidgets.QGroupBox("Nodes roots")
        self.group_nodes.setProperty("style", "with_header")
        self.table_nodes = widgets.SimpleTable()

        self.label_jobs_root = QtWidgets.QLabel("Jobs root")
        self.input_jobs_root = QtWidgets.QLineEdit()
        helper.fix_no_outline(self.input_jobs_root)
        self.button_browse_jobs_root = QtWidgets.QToolButton()
        helper.set_icon(self.button_browse_jobs_root, "folder")

    def _create_widgets_naming(self):
        """Create widgets naming section."""
        self.group_naming = QtWidgets.QGroupBox("Naming")
        self.group_naming.setProperty("style", "with_header")
        self.check_auto_extract_naming = QtWidgets.QCheckBox(" Auto extract naming")
        self.combo_naming_preset = QtWidgets.QComboBox()
        helper.fix_combo_solid_bg(self.combo_naming_preset)
        self.label_naming_preset = QtWidgets.QLabel("Naming preset")
        self.text_naming_values = QtWidgets.QTextEdit()
        self.label_naming_root = QtWidgets.QLabel("Naming root")
        self.input_naming_root= QtWidgets.QLineEdit()
        helper.fix_no_outline(self.input_naming_root)
        self.browse_naming_root = QtWidgets.QToolButton()
        helper.set_icon(self.browse_naming_root, "folder")

    def create_layouts(self):
        """Create layouts."""
        self.layout_main = QtWidgets.QVBoxLayout()
        self.layout_main.setContentsMargins(0,0,0,0)
        self.tab_sections = QtWidgets.QTabWidget()

        self._create_layouts_general()
        self._create_layouts_paths_section()
        self._create_layouts_naming_section()

        self.tab_sections.addTab(self.tab_general, "General")
        self.tab_sections.addTab(self.tab_paths, "Paths")
        self.tab_sections.addTab(self.tab_naming, "Naming")

        self.layout_main.addWidget(self.tab_sections)
        self.layout_main.addWidget(self.button_save)

        helper.set_style_sheet(self)
        self.setLayout(self.layout_main)

    def _create_layouts_general(self):
        """"Create layouts general section."""
        self.layout_general = QtWidgets.QVBoxLayout()
        self.layout_general.setContentsMargins(0,10,0,0)
        self.tab_general = QtWidgets.QGroupBox()
        self.tab_general.setProperty("style", "plain")
        self.tab_general.setLayout(self.layout_general)

        self.layout_logging_level = QtWidgets.QHBoxLayout()
        self.layout_logging_level.addWidget(self.label_logging_level)
        self.layout_logging_level.addWidget(self.combo_logging_level)
        self.layout_logging_level.addStretch()

        self.layout_new_jobs_behavior = QtWidgets.QHBoxLayout()
        self.layout_new_jobs_behavior.addWidget(self.label_new_jobs_behavior)
        self.layout_new_jobs_behavior.addWidget(self.combo_new_jobs_behavior)
        self.layout_new_jobs_behavior.addStretch()

        self.layout_thumbnail_index = QtWidgets.QHBoxLayout()
        self.layout_thumbnail_index.addWidget(self.label_thumbnail_index)
        self.layout_thumbnail_index.addWidget(self.combo_thumbnail_index)
        self.layout_thumbnail_index.addStretch()

        self.layout_preset_dropdown_use_completer = QtWidgets.QHBoxLayout()
        self.layout_preset_dropdown_use_completer.addWidget(
            self.label_preset_dropdown_type
        )
        self.layout_preset_dropdown_use_completer.addWidget(
            self.combo_preset_dropdown_type
        )
        self.layout_preset_dropdown_use_completer.addStretch()

        self.layout_number_recent_files = QtWidgets.QHBoxLayout()
        self.layout_number_recent_files.addWidget(self.label_number_recent_files)
        self.layout_number_recent_files.addWidget(self.spin_number_recent_files)
        self.layout_number_recent_files.addStretch()

        self.layout_ingest_threads = QtWidgets.QHBoxLayout()
        self.layout_ingest_threads.addWidget(self.label_ingest_threads)
        self.layout_ingest_threads.addWidget(self.spin_ingest_threads)
        self.layout_ingest_threads.addStretch()

        self.layout_jobs_root = QtWidgets.QHBoxLayout()
        self.layout_jobs_root.addWidget(self.label_jobs_root)
        self.layout_jobs_root.addWidget(self.input_jobs_root, 10)
        self.layout_jobs_root.addWidget(self.button_browse_jobs_root)

        self.layout_general.addWidget(self.check_show_tooltips)
        self.layout_general.addWidget(self.check_always_on_top)
        self.layout_general.addWidget(self.check_auto_load_last_workfile)
        self.layout_general.addLayout(self.layout_number_recent_files)
        self.layout_general.addLayout(self.layout_ingest_threads)
        self.layout_general.addLayout(self.layout_logging_level)
        self.layout_general.addLayout(self.layout_new_jobs_behavior)
        self.layout_general.addLayout(self.layout_thumbnail_index)
        self.layout_general.addLayout(self.layout_preset_dropdown_use_completer)
        self.layout_general.addStretch()

    def _create_layouts_paths_section(self):
        """Create layouts paths section."""
        self.layout_paths = QtWidgets.QVBoxLayout()
        self.layout_paths.setContentsMargins(0,10,0,0)
        self.tab_paths = QtWidgets.QGroupBox()
        self.tab_paths.setProperty("style", "plain")
        self.tab_paths.setLayout(self.layout_paths)

        self.layout_nuke_executables = QtWidgets.QVBoxLayout()
        self.layout_nuke_executables.setContentsMargins(0,0,0,0)
        self.layout_nuke_executables.addWidget(self.table_nuke_executables)
        self.group_nuke_executables.setLayout(self.layout_nuke_executables)

        self.layout_presets = QtWidgets.QVBoxLayout()
        self.layout_presets.setContentsMargins(0,0,0,0)
        self.group_presets.setLayout(self.layout_presets)
        self.layout_presets.addWidget(self.table_presets)

        self.layout_nodes = QtWidgets.QVBoxLayout()
        self.layout_nodes.setContentsMargins(0,0,0,0)
        self.group_nodes.setLayout(self.layout_nodes)
        self.layout_nodes.addWidget(self.table_nodes)

        self.layout_paths.addWidget(self.group_nuke_executables)
        self.layout_paths.addWidget(self.group_presets)
        self.layout_paths.addWidget(self.group_nodes)
        self.layout_paths.addLayout(self.layout_jobs_root)

    def _create_layouts_naming_section(self):
        """Create layout naming section."""
        self.layout_naming = QtWidgets.QVBoxLayout()
        self.layout_naming.setContentsMargins(0, 10, 0, 0)
        self.tab_naming = QtWidgets.QGroupBox()
        self.tab_naming.setProperty("style", "plain")
        self.tab_naming.setLayout(self.layout_naming)

        self.layout_naming_top = QtWidgets.QHBoxLayout()
        self.layout_naming_top.addWidget(self.check_auto_extract_naming)
        self.layout_naming_top.addWidget(self.label_naming_preset, 1)
        self.layout_naming_top.addWidget(self.combo_naming_preset, 5)
        self.layout_naming_top.addWidget(self.label_naming_root, 1)
        self.layout_naming_top.addWidget(self.input_naming_root, 10)
        self.layout_naming_top.addWidget(self.browse_naming_root, 1)

        self.layout_naming.addLayout(self.layout_naming_top)
        self.layout_naming.addWidget(self.text_naming_values)
