"""Extractor that will find out naming values from a path.

Usage:


    >>> path = "path/to/render/der_001_002_comp_v001.1001.exr"
    >>> naming = Naming(path)

    # Given the naming root directory contains a named pattern file 'standard'
    # then let's apply its pattern to the given path.
    >>> naming.apply("standard")

    # Now that we have extracted the naming values, let's print them:
    >>> print naming.values
    {
        'task': 'comp',
        'shot': '002',
        'sequence': '001',
        'project': 'der',
        'ext': 'exr',
        'version': '001'
    }

"""

# Import built-in modules
import os
import re

# Import local modules
from smartMedia.settings import Settings


class Naming(object):
    """Extractor that will find out naming values from a path."""

    def __init__(self, source_path, naming_root=None):
        """Initialize the NamingExtractor instance.

        Args:
            source_path (str): Absolute path of source to extract naming
                values from.
            naming_root (str, optional): Absolute path of the directory that
                contains all named pattern files for naming extractions. If
                not provided we use the path from settings.

        """
        self.source_path = source_path

        self.naming_root = naming_root or Settings()["naming_root"]
        self.ensure_naming_root()
        self.all_pattern_names = self.get_all_pattern_names()
        self.values = {}

    def ensure_naming_root(self):
        """Ensure the naming root directory exists."""
        if not self.naming_root:
            return

        try:
            os.makedirs(self.naming_root)
        except OSError:
            pass

    def get_all_pattern_names(self):
        """Get names of all files in naming root.

        Returns:
            :obj:`list` of :obj:`str`: Sequence of named files in naming root.

        """
        if not self.naming_root or not os.path.isdir(self.naming_root):
            return []

        return [
            os.path.splitext(name)[0]
            for name
            in os.listdir(self.naming_root)
            if os.path.isfile(os.path.join(self.naming_root, name))
            and name[0] not in ("_", ".")
        ]

    def apply(self, name, extract_from_basename=True):
        """Extract the source by the data of the pattern with the named file.

        Args:
            name (str): Name of the file to use as a pattern to extract
                naming values.
            extract_from_basename (True, optional): If True apply the pattern
                only on the source path basename. Otherwise apply it to the
                whole path.

        Raises:
            RuntimeError: When the source path does not match the pattern.

        """
        pattern = self.get_pattern(name)
        if not pattern:
            raise RuntimeError("No pattern defined, nothing to apply.")

        if extract_from_basename:
            path = os.path.basename(self.source_path)
        else:
            path = self.source_path

        match = re.match(pattern, path)

        if not match:
            raise RuntimeError(
                "{0} does not match pattern {1} -> '{2}'".format(
                    path, name, pattern
                )
            )

        self.values = match.groupdict()

    def get_pattern(self, name):
        """Get the pattern from the given named file.

        Args:
            name (str): Name of the file to use as a pattern to extract
                naming values.

        Raises:
            RuntimeError: When no named pattern file with the given name
                exists in the naming root directory.

        """
        content = self.get_content(name)

        for line in content.split("\n"):
            if line.startswith("#") or line == "":
                continue
            return line

        return ""

    def get_content(self, name):
        """Get the whole file content from the given named file.

        Args:
            name (str): Name of the file to load.

        Raises:
            RuntimeError: When no named pattern file with the given name
                exists in the naming root directory.

        Returns:
            str: The whole file content from the given named file.

        """
        if not os.path.isdir(self.naming_root) or not name:
            return ""

        for path_ in os.listdir(self.naming_root):
            if os.path.splitext(path_)[0] == name:
                path = os.path.join(self.naming_root, path_)
                break
        else:
            return ""

        with open(path, "r") as file_:
            return file_.read()
