"""A preset to render a job with."""

# Import built-in modules
from collections import OrderedDict
import json
import logging
import os

# Import local modules
from smartMedia.settings import Settings
from smartMedia.constants import JSON_INDENT
from smartMedia.constants import PRESETS_PATH
from smartMedia.clogging import LEVEL

LOGGER = logging.getLogger()
LOGGER.setLevel(LEVEL)


class Preset(object):
    """A preset to render a job with."""

    def __init__(self, preset_root):
        """Initialize the Preset instance.

        Args:
            preset_root (str): Absolute path of the preset root directory.

        """
        self.root = preset_root
        self.path = os.path.join(preset_root, "data.json")
        self.name = os.path.basename(preset_root)

        self.load()

    def reload(self):
        """Convenience method to reload the preset from disk."""
        self.load()

    def load(self):
        """Load the preset from disk."""
        with open(self.path, "r") as file_:
            self.data = json.load(file_)

    def __getitem__(self, key):
        """Safely get the value for the given key.

        Args:
            key (str): Name of the key to retrieve the value from.

        Returns:
             multiple: The value of the key if it exists, otherwise None.

        """
        return self.data.get(key)

    def __setitem__(self, key, value):
        """Set the given key to the given value.

        Args:
            key (str): The key to set.
            value (multiple): The value to assign to the key.

        """
        self.data[key] = value

    def apply_preset(self, preset):
        """Merge the preset onto the task."""
        if isinstance(preset, dict):
            preset = preset
        else:
            preset = preset.data

        self.data.update(preset)

    def save(self):
        """Save the current state of the preset to disk."""
        with open(self.path, "w") as file_:
            json.dump(self.data, file_, indent=JSON_INDENT)


def is_valid_preset(preset_root):
    """Check if the preset is a valid preset.

    Args:
        preset_root (str): Absolute path of the preset root to check for
            validity.

    Returns:
        bool: True when the preset is valid, otherwise False.

    """
    # Ensure th data.json file exists.
    data_path = os.path.join(preset_root, "data.json")
    if not os.path.isfile(data_path):
        LOGGER.warning(
            "Skipping preset as it does not contain a data.json: %s",
            preset_root
        )
        return False

    # Ensure the data.json is well formed.
    with open(data_path, "r") as file_:
        try:
            json.load(file_)
        except:
            LOGGER.warning("Skipping non well formed preset: %s", data_path)

    return True


def load_all_presets():
    """Load all available and valid presets from settings and env vars.

    Returns:
        collections.OrderedDict: OrderedDict containing key-value pairs of
            preset names and presets.

    """
    settings = Settings()

    preset_roots = [
        root for root in settings["preset_paths"]
        if os.path.isdir(root)
    ]

    preset_roots.extend([
        root for root
        in os.environ.get(PRESETS_PATH, "").split(os.pathsep)
        if os.path.isdir(root)
    ])

    presets = {}
    for preset_root in preset_roots:
        valid_preset_roots = [
            os.path.join(preset_root, name)
            for name in os.listdir(preset_root)
            if os.path.isdir(os.path.join(preset_root, name))
            and not name.startswith("_")
            and is_valid_preset(os.path.join(preset_root, name))
        ]

        for preset_root in valid_preset_roots:
            presets[os.path.basename(preset_root)] = Preset(preset_root)

    presets_ordered = OrderedDict()
    for name in sorted(presets):
        presets_ordered[name] = presets[name]

    return presets_ordered
