"""Settings being loaded and saved to settings.json file for this package."""

# Import built-in modules
import json
import logging
import os

# Import local modules
from smartMedia import paths
from smartMedia.constants import JSON_INDENT
from smartMedia.constants import LOGGING_LEVELS


class Settings():
    """Settings being loaded and saved to settings.json file."""

    SETTINGS_PATH = os.path.join(paths.get_package_dir(), "settings.json")
    LOGGER = logging.getLogger(__name__)

    def __init__(self):
        """Initialize the Settings instance."""
        if not os.path.isfile(self.SETTINGS_PATH):
            self.check_settings()

        self._settings = self.load_settings()

        self.LOGGER.setLevel(LOGGING_LEVELS[self._settings["logging_level"]])

    @classmethod
    def check_settings(cls):
        """Ensure settings file exists and all key-values are contained.

        We will add any key-value pair from the DEFAULT_SETTINGS that are not
        contained inside the physical settings.json on disk. This ensures we
        can add additional key-value pairs in the future and ensure they exist
        for the user.

        """
        paths.ensure_directory(paths.get_package_dir())

        if not os.path.isfile(cls.SETTINGS_PATH):
            with open(cls.SETTINGS_PATH, "w") as file_:
                cls.LOGGER.debug("Creating setting file in %s",
                                 cls.SETTINGS_PATH)

                json.dump(DEFAULT_SETTINGS, file_, indent=JSON_INDENT)
                return

        settings = Settings().load_settings()
        for key, value in DEFAULT_SETTINGS.items():
            if key not in settings:
                cls.LOGGER.debug("Adding to settings.json: %s=%s", key, value)
                settings[key] = value

        with open(cls.SETTINGS_PATH, "w") as file_:
            json.dump(settings, file_, indent=JSON_INDENT)

    @classmethod
    def load_settings(cls):
        """Load settings file without the need of a model.

        Returns:
            dict: The parsed settings file.

        """
        with open(cls.SETTINGS_PATH, "r") as file_:
            return json.load(file_)

    def __setitem__(self, key, value):
        """Set the settings key using the given value.

        Args:
            key (str): The name of the key to update.
            value (multiple): The new value to set for the given key.

        """
        self._settings[key] = value

    def __getitem__(self, key):
        """Get settings value for given key.

        Args:
            key (str): Name of key to get settings value for.

        Returns:
            multiple: The value of the asked settings key.

        """
        return self._settings[key]

    def save(self):
        """Save the current settings state back to disk."""
        with open(self.SETTINGS_PATH, "w") as file_:
            json.dump(self._settings, file_, indent=JSON_INDENT)

    def reload(self):
        """Reload the settings."""
        self._settings = self.load_settings()


# Default settings values that will be written when no settings.json file
# exists on disk. Each key-value pair will be checked on existence and will
# be added to the settings.json file in case it does not exist, yet.
DEFAULT_SETTINGS = {
    # If True show tooltips for all major widgets.
    "tooltips": True,

    # If True show the elements browser always on top.
    "always_on_top": False,

    # Number of ingesting threads to use in the processor.
    "ingest_processor_threads": 4,

    # The slider size in px for all library elements.
    "slider_size": 300,

    # The default logging level to use. Can be changed in the settings.
    "logging_level": "info",

    # Absolute paths of the Nuke executables to run jobs with.
    "nuke_executables": [
        ["", False]
    ],

    # The default root where all jobs live.
    "jobs_root": paths.get_default_jobs_root(),

    # Absolute paths of where presets live.
    # Additional paths can be added by adding to CRAGL_SMARTMEDIA_PRESET_PATHS
    # and via settings.
    "preset_paths": [
        os.path.normpath(
            os.path.join(
                os.path.dirname(__file__), "..", "presets"
            )
        )
    ],

    # Fields to show on a source.
    # Order matters as these fields will be shown in that order.
    "source_fields": [
        "start",
        "end",
        "length",
        "resolution"
    ],

    # Create a thumbnail from this frame of a sequence.
    # Choose from ("first", "middle", "last").
    "thumbnail_index": "first",

    # New jobs go to "top" or "bottom" of jobs list.
    "new_jobs_order": "top",

    # When True and closing the main window then auto load the latest working
    # file once the main window gets re-launched.
    "auto_load_last_workfile": True,

    # A list that stores absolute paths to all recent work files.
    "recent_files": [],

    # The number of recent files to store. After exceeding, the oldest path
    # gets removed.
    "number_recent_files": 5,

    # Formats to use in the reformat drop downs.
    "formats": [
        # First argument: The name of the format.
        # Second argument: The internal value to set.
        ("PC_Video 640x480", "640 480 0 0 640 480 1 PC_Video"),
        ("HD_720 1280x720", "1280 720 0 0 1280 720 1 HD_720"),
        ("HD_1080 1920x1080", "1920 1080 0 0 1920 1080 1 HD_1080"),
        ("UHD_4K 3840x2160", "3840 2160 0 0 3840 2160 1 UHD_4K"),
    ],

    # The visibility of details widgets in any task and the order to show
    # the details widgets.
    # NOTE: No need to add any details widgets into here as we will
    # automatically add all details widgets on the fly once the details view
    # with all details widgets for a task opens.
    "task_details": {},

    # Absolute path of root directory that points to the folder that contains
    # naming files.
    "naming_root": os.path.join(
        paths.get_package_dir(private=False),
        "naming"
    ),

    # Currently selected naming. This is what new tasks will automatically
    # be applied with.
    "current_naming": "",

    # The output roots to apply for all jobs.
    # Each item is a dict in the following format:
    # {
    #     "active": <True/False>
    #     "mode": <"abs"/"rel">
    #     "path": <str path>
    # }
    "output_roots": [
        {
            "active": False,
            "mode": "rel",
            "path": "../proxy",
        }
    ],

    # Setting to choose if you want to have a completer or a normal dropdown
    # with no completer for preset dropdowns. 'Yes' uses a completer, 'no'
    # uses no completer
    "preset_dropdown_type": "no completer",

    # Roots directory for all nodes to use for task renderings, e.g. slates,
    # burnins, reformats, etc.
    "nodes_roots": [
        paths.get_nodes_directory()
    ],

    # When set to True then automatically apply naming extraction when
    # ingesting a new source by cycling through all naming patterns to see if
    # one pattern applies.
    "auto_apply_naming_extraction": True,

    # All automatic presets get added automatically to any new source.
    # Any preset can be set as automatic in the presets section.
    "automatic_presets": [],

}

